@@ -10,13 +10,11 @@ configuration files for Astropy and affiliated packages.
 """
 
 import io
-import re
-import hashlib
-import pathlib
 import pkgutil
 import warnings
 import importlib
 import contextlib
+import os
 from os import path
 from textwrap import TextWrapper
 from warnings import warn
@@ -25,15 +23,14 @@ from contextlib import contextmanager
 from astropy.extern.configobj import configobj, validate
 from astropy.utils import find_current_module, silence
 from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyWarning
-from astropy.utils.introspection import resolve_name
 from astropy.utils.compat.context import nullcontext
+from astropy.utils.introspection import resolve_name
 
 from .paths import get_config_dir
 
-__all__ = ['InvalidConfigurationItemWarning',
-           'ConfigurationMissingWarning', 'get_config',
-           'reload_config', 'ConfigNamespace', 'ConfigItem',
-           'generate_config']
+__all__ = ('InvalidConfigurationItemWarning', 'ConfigurationMissingWarning',
+           'get_config', 'reload_config', 'ConfigNamespace', 'ConfigItem',
+           'generate_config', 'create_config_file')
 
 
 class InvalidConfigurationItemWarning(AstropyWarning):
@@ -43,6 +40,8 @@ class InvalidConfigurationItemWarning(AstropyWarning):
     """
 
 
+# This was raised with Astropy < 4.3 when the configuration file was not found.
+# It is kept for compatibility and should be removed at some point.
 class ConfigurationMissingWarning(AstropyWarning):
     """ A Warning that is issued when the configuration directory cannot be
     accessed (usually due to a permissions problem). If this warning appears,
@@ -363,8 +362,9 @@ class ConfigItem:
 
         Returns
         -------
-        val
+        val : object
             The new value loaded from the configuration file.
+
         """
         self.set(self.defaultvalue)
         baseobj = get_config(self.module, True, rootname=self.rootname)
@@ -407,7 +407,7 @@ class ConfigItem:
 
         Returns
         -------
-        val
+        val : object
             This item's value, with a type determined by the ``cfgtype``
             attribute.
 
@@ -415,6 +415,7 @@ class ConfigItem:
         ------
         TypeError
             If the configuration value as stored is not this item's type.
+
         """
         def section_name(section):
             if section == '':
@@ -483,7 +484,7 @@ class ConfigItem:
         return self._validator.check(self.cfgtype, val)
 
 
-# this dictionary stores the master copy of the ConfigObj's for each
+# this dictionary stores the primary copy of the ConfigObj's for each
 # root package
 _cfgobjs = {}
 
@@ -510,7 +511,7 @@ def get_config(packageormod=None, reload=False, rootname=None):
     package or module.
 
     Parameters
-    -----------
+    ----------
     packageormod : str or None
         The package for which to retrieve the configuration object. If a
         string, it must be a valid package name, or if ``None``, the package from
@@ -574,17 +575,12 @@ def get_config(packageormod=None, reload=False, rootname=None):
             else:
                 cfgfn = path.join(get_config_dir(rootname=rootname), pkgname + '.cfg')
             cobj = configobj.ConfigObj(cfgfn, interpolation=False)
-        except OSError as e:
-            msg = ('Configuration defaults will be used due to ')
-            errstr = '' if len(e.args) < 1 else (':' + str(e.args[0]))
-            msg += e.__class__.__name__ + errstr
-            msg += f' on {cfgfn}'
-            warn(ConfigurationMissingWarning(msg))
-
-            # This caches the object, so if the file becomes accessible, this
-            # function won't see it unless the module is reloaded
+        except OSError:
+            # This can happen when HOME is not set
             cobj = configobj.ConfigObj(interpolation=False)
 
+        # This caches the object, so if the file becomes accessible, this
+        # function won't see it unless the module is reloaded
         _cfgobjs[pkgname] = cobj
 
     if secname:  # not the root package
@@ -605,7 +601,7 @@ def generate_config(pkgname='astropy', filename=None, verbose=False):
     ----------
     pkgname : str or None
         The package for which to retrieve the configuration object.
-    filename : str or file object or None
+    filename : str or file-like or None
         If None, the default configuration path is taken from `get_config`.
 
     """
@@ -637,10 +633,10 @@ def generate_config(pkgname='astropy', filename=None, verbose=False):
                           width=78)
 
     if filename is None:
-        filename = get_config(pkgname).filename
+        filename = get_config_filename(pkgname)
 
     with contextlib.ExitStack() as stack:
-        if isinstance(filename, (str, pathlib.Path)):
+        if isinstance(filename, (str, os.PathLike)):
             fp = stack.enter_context(open(filename, 'w'))
         else:
             # assume it's a file object, or io.StringIO
@@ -707,52 +703,18 @@ def reload_config(packageormod=None, rootname=None):
 def is_unedited_config_file(content, template_content=None):
     """
     Determines if a config file can be safely replaced because it doesn't
-    actually contain any meaningful content.
-
-    To meet this criteria, the config file must be either:
-
-    - All comments or completely empty
-
-    - An exact match to a "legacy" version of the config file prior to
-      Astropy 0.4, when APE3 was implemented and the config file
-      contained commented-out values by default.
+    actually contain any meaningful content, i.e. if it contains only comments
+    or is completely empty.
     """
-    # We want to calculate the md5sum using universal line endings, so
-    # that even if the files had their line endings converted to \r\n
-    # on Windows, this will still work.
-
-    content = content.encode('latin-1')
-
-    # The jquery_url setting, present in 0.3.2 and later only, is
-    # effectively auto-generated by the build system, so we need to
-    # ignore it in the md5sum calculation for 0.3.2.
-    content = re.sub(br'\njquery_url\s*=\s*[^\n]+', b'', content)
-
-    # First determine if the config file has any effective content
-    buffer = io.BytesIO(content)
-    buffer.seek(0)
+    buffer = io.StringIO(content)
     raw_cfg = configobj.ConfigObj(buffer, interpolation=True)
-    for v in raw_cfg.values():
-        if len(v):
-            break
-    else:
-        return True
-
-    # Now determine if it matches the md5sum of a known, unedited
-    # config file.
-    known_configs = set([
-        '7d4b4f1120304b286d71f205975b1286',  # v0.3.2
-        '5df7e409425e5bfe7ed041513fda3288',  # v0.3
-        '8355f99a01b3bdfd8761ef45d5d8b7e5',  # v0.2
-        '4ea5a84de146dc3fcea2a5b93735e634'   # v0.2.1, v0.2.2, v0.2.3, v0.2.4, v0.2.5
-    ])
-
-    md5 = hashlib.md5()
-    md5.update(content)
-    digest = md5.hexdigest()
-    return digest in known_configs
+    # If any of the items is set, return False
+    return not any(len(v) > 0 for v in raw_cfg.values())
 
 
+# This function is no more used by astropy but it is kept for the other
+# packages that may use it (e.g. astroquery). It should be removed at some
+# point.
 # this is not in __all__ because it's not intended that a user uses it
 def update_default_config(pkg, default_cfg_dir_or_fn, version=None, rootname='astropy'):
     """
@@ -852,3 +814,59 @@ def update_default_config(pkg, default_cfg_dir_or_fn, version=None, rootname='as
             return True
 
     return False
+
+
+def create_config_file(pkg, rootname='astropy', overwrite=False):
+    """
+    Create the default configuration file for the specified package.
+    If the file already exists, it is updated only if it has not been
+    modified.  Otherwise the ``overwrite`` flag is needed to overwrite it.
+
+    Parameters
+    ----------
+    pkg : str
+        The package to be updated.
+    rootname : str
+        Name of the root configuration directory.
+    overwrite : bool
+        Force updating the file if it already exists.
+
+    Returns
+    -------
+    updated : bool
+        If the profile was updated, `True`, otherwise `False`.
+
+    """
+
+    # local import to prevent using the logger before it is configured
+    from astropy.logger import log
+
+    cfgfn = get_config_filename(pkg, rootname=rootname)
+
+    # generate the default config template
+    template_content = io.StringIO()
+    generate_config(pkg, template_content)
+    template_content.seek(0)
+    template_content = template_content.read()
+
+    doupdate = True
+
+    # if the file already exists, check that it has not been modified
+    if cfgfn is not None and path.exists(cfgfn):
+        with open(cfgfn, 'rt', encoding='latin-1') as fd:
+            content = fd.read()
+
+        doupdate = is_unedited_config_file(content, template_content)
+
+    if doupdate or overwrite:
+        with open(cfgfn, 'wt', encoding='latin-1') as fw:
+            fw.write(template_content)
+        log.info('The configuration file has been successfully written '
+                 f'to {cfgfn}')
+        return True
+    elif not doupdate:
+        log.warning('The configuration file already exists and seems to '
+                    'have been customized, so it has not been updated. '
+                    'Use overwrite=True if you really want to update it.')
+
+    return False
