@@ -21,21 +21,22 @@ import pytest
 from astropy import __version__
 from astropy.tests.helper import enable_deprecations_as_exceptions
 
-try:
-    # This is needed to silence a warning from matplotlib caused by
-    # PyInstaller's matplotlib runtime hook.  This can be removed once the
-    # issue is fixed upstream in PyInstaller, and only impacts us when running
-    # the tests from a PyInstaller bundle.
-    # See https://github.com/astropy/astropy/issues/10785
-    if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
-        # The above checks whether we are running in a PyInstaller bundle.
-        warnings.filterwarnings("ignore", "(?s).*MATPLOTLIBDATA.*",
-                                category=UserWarning)
+# This is needed to silence a warning from matplotlib caused by
+# PyInstaller's matplotlib runtime hook.  This can be removed once the
+# issue is fixed upstream in PyInstaller, and only impacts us when running
+# the tests from a PyInstaller bundle.
+# See https://github.com/astropy/astropy/issues/10785
+if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
+    # The above checks whether we are running in a PyInstaller bundle.
+    warnings.filterwarnings("ignore", "(?s).*MATPLOTLIBDATA.*",
+                            category=UserWarning)
+
+# Note: while the filterwarnings is required, this import has to come after the
+# filterwarnings above, because this attempts to import matplotlib:
+from astropy.utils.compat.optional_deps import HAS_MATPLOTLIB
+
+if HAS_MATPLOTLIB:
     import matplotlib
-except ImportError:
-    HAS_MATPLOTLIB = False
-else:
-    HAS_MATPLOTLIB = True
 
 enable_deprecations_as_exceptions(
     include_astropy_deprecations=False,
@@ -56,6 +57,18 @@ def ignore_matplotlibrc():
         yield
 
 
+@pytest.fixture
+def fast_thread_switching():
+    """Fixture that reduces thread switching interval.
+
+    This makes it easier to provoke race conditions.
+    """
+    old = sys.getswitchinterval()
+    sys.setswitchinterval(1e-6)
+    yield
+    sys.setswitchinterval(old)
+
+
 def pytest_configure(config):
     from astropy.utils.iers import conf as iers_conf
 
@@ -132,7 +145,7 @@ def pytest_terminal_summary(terminalreporter):
 
     terminalreporter.ensure_newline()
     terminalreporter.write_line(
-        'Some tests are known to fail when run from the IPython prompt; '
+        'Some tests may fail when run from the IPython prompt; '
         'especially, but not limited to tests involving logging and warning '
         'handling.  Unless you are certain as to the cause of the failure, '
         'please check that the failure occurs outside IPython as well.  See '
