@@ -7,10 +7,9 @@ classes.
 
 
 # Standard library
-import abc
 import copy
 import inspect
-from collections import namedtuple, OrderedDict, defaultdict
+from collections import namedtuple, defaultdict
 import warnings
 
 # Dependencies
@@ -21,8 +20,7 @@ from astropy.utils.compat.misc import override__dir__
 from astropy.utils.decorators import lazyproperty, format_doc
 from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
 from astropy import units as u
-from astropy.utils import (OrderedDescriptorContainer, ShapedLikeNDArray,
-                           check_broadcast)
+from astropy.utils import ShapedLikeNDArray, check_broadcast
 from .transformations import TransformGraph
 from . import representation as r
 from .angles import Angle
@@ -137,170 +135,6 @@ def _normalize_representation_type(kwargs):
         kwargs['representation_type'] = kwargs.pop('representation')
 
 
-# Need to subclass ABCMeta as well, so that this meta class can be combined
-# with ShapedLikeNDArray below (which is an ABC); without it, one gets
-# "TypeError: metaclass conflict: the metaclass of a derived class must be a
-#  (non-strict) subclass of the metaclasses of all its bases"
-class FrameMeta(OrderedDescriptorContainer, abc.ABCMeta):
-    def __new__(mcls, name, bases, members):
-        if 'default_representation' in members:
-            default_repr = members.pop('default_representation')
-            found_default_repr = True
-        else:
-            default_repr = None
-            found_default_repr = False
-
-        if 'default_differential' in members:
-            default_diff = members.pop('default_differential')
-            found_default_diff = True
-        else:
-            default_diff = None
-            found_default_diff = False
-
-        if 'frame_specific_representation_info' in members:
-            repr_info = members.pop('frame_specific_representation_info')
-            found_repr_info = True
-        else:
-            repr_info = None
-            found_repr_info = False
-
-        # somewhat hacky, but this is the best way to get the MRO according to
-        # https://mail.python.org/pipermail/python-list/2002-December/167861.html
-        tmp_cls = super().__new__(mcls, name, bases, members)
-
-        # now look through the whole MRO for the class attributes, raw for
-        # frame_attr_names, and leading underscore for others
-        for m in (c.__dict__ for c in tmp_cls.__mro__):
-            if not found_default_repr and '_default_representation' in m:
-                default_repr = m['_default_representation']
-                found_default_repr = True
-
-            if not found_default_diff and '_default_differential' in m:
-                default_diff = m['_default_differential']
-                found_default_diff = True
-
-            if (not found_repr_info
-                    and '_frame_specific_representation_info' in m):
-                # create a copy of the dict so we don't mess with the contents
-                repr_info = m['_frame_specific_representation_info'].copy()
-                found_repr_info = True
-
-            if found_default_repr and found_default_diff and found_repr_info:
-                break
-        else:
-            raise ValueError(
-                'Could not find all expected BaseCoordinateFrame class '
-                'attributes.  Are you mis-using FrameMeta?')
-
-        # Unless overridden via `frame_specific_representation_info`, velocity
-        # name defaults are (see also docstring for BaseCoordinateFrame):
-        #   * ``pm_{lon}_cos{lat}``, ``pm_{lat}`` for
-        #     `SphericalCosLatDifferential` proper motion components
-        #   * ``pm_{lon}``, ``pm_{lat}`` for `SphericalDifferential` proper
-        #     motion components
-        #   * ``radial_velocity`` for any `d_distance` component
-        #   * ``v_{x,y,z}`` for `CartesianDifferential` velocity components
-        # where `{lon}` and `{lat}` are the frame names of the angular
-        # components.
-        if repr_info is None:
-            repr_info = {}
-
-        # the tuple() call below is necessary because if it is not there,
-        # the iteration proceeds in a difficult-to-predict manner in the
-        # case that one of the class objects hash is such that it gets
-        # revisited by the iteration.  The tuple() call prevents this by
-        # making the items iterated over fixed regardless of how the dict
-        # changes
-        for cls_or_name in tuple(repr_info.keys()):
-            if isinstance(cls_or_name, str):
-                # TODO: this provides a layer of backwards compatibility in
-                # case the key is a string, but now we want explicit classes.
-                cls = _get_repr_cls(cls_or_name)
-                repr_info[cls] = repr_info.pop(cls_or_name)
-
-        # The default spherical names are 'lon' and 'lat'
-        repr_info.setdefault(r.SphericalRepresentation,
-                             [RepresentationMapping('lon', 'lon'),
-                              RepresentationMapping('lat', 'lat')])
-
-        sph_component_map = {m.reprname: m.framename
-                             for m in repr_info[r.SphericalRepresentation]}
-
-        repr_info.setdefault(r.SphericalCosLatDifferential, [
-            RepresentationMapping(
-                'd_lon_coslat',
-                'pm_{lon}_cos{lat}'.format(**sph_component_map),
-                u.mas/u.yr),
-            RepresentationMapping('d_lat',
-                                  'pm_{lat}'.format(**sph_component_map),
-                                  u.mas/u.yr),
-            RepresentationMapping('d_distance', 'radial_velocity',
-                                  u.km/u.s)
-        ])
-
-        repr_info.setdefault(r.SphericalDifferential, [
-            RepresentationMapping('d_lon',
-                                  'pm_{lon}'.format(**sph_component_map),
-                                  u.mas/u.yr),
-            RepresentationMapping('d_lat',
-                                  'pm_{lat}'.format(**sph_component_map),
-                                  u.mas/u.yr),
-            RepresentationMapping('d_distance', 'radial_velocity',
-                                  u.km/u.s)
-        ])
-
-        repr_info.setdefault(r.CartesianDifferential, [
-            RepresentationMapping('d_x', 'v_x', u.km/u.s),
-            RepresentationMapping('d_y', 'v_y', u.km/u.s),
-            RepresentationMapping('d_z', 'v_z', u.km/u.s)])
-
-        # Unit* classes should follow the same naming conventions
-        # TODO: this adds some unnecessary mappings for the Unit classes, so
-        # this could be cleaned up, but in practice doesn't seem to have any
-        # negative side effects
-        repr_info.setdefault(r.UnitSphericalRepresentation,
-                             repr_info[r.SphericalRepresentation])
-
-        repr_info.setdefault(r.UnitSphericalCosLatDifferential,
-                             repr_info[r.SphericalCosLatDifferential])
-
-        repr_info.setdefault(r.UnitSphericalDifferential,
-                             repr_info[r.SphericalDifferential])
-
-        # Make read-only properties for the frame class attributes that should
-        # be read-only to make them immutable after creation.
-        # We copy attributes instead of linking to make sure there's no
-        # accidental cross-talk between classes
-        mcls.readonly_prop_factory(members, 'default_representation',
-                                   default_repr)
-        mcls.readonly_prop_factory(members, 'default_differential',
-                                   default_diff)
-        mcls.readonly_prop_factory(members,
-                                   'frame_specific_representation_info',
-                                   copy.deepcopy(repr_info))
-
-        # now set the frame name as lower-case class name, if it isn't explicit
-        if 'name' not in members:
-            members['name'] = name.lower()
-
-        # A cache that *must be unique to each frame class* - it is
-        # insufficient to share them with superclasses, hence the need to put
-        # them in the meta
-        members['_frame_class_cache'] = {}
-
-        return super().__new__(mcls, name, bases, members)
-
-    @staticmethod
-    def readonly_prop_factory(members, attr, value):
-        private_attr = '_' + attr
-
-        def getter(self):
-            return getattr(self, private_attr)
-
-        members[private_attr] = value
-        members[attr] = property(getter)
-
-
 _RepresentationMappingBase = \
     namedtuple('RepresentationMapping',
                ('reprname', 'framename', 'defaultunit'))
@@ -359,7 +193,7 @@ _components = """
 
 
 @format_doc(base_doc, components=_components, footer="")
-class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
+class BaseCoordinateFrame(ShapedLikeNDArray):
     """
     The base class for coordinate frames.
 
@@ -407,11 +241,76 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
     # attributes.
     frame_specific_representation_info = {}
 
-    _inherit_descriptors_ = (Attribute,)
-
-    frame_attributes = OrderedDict()
+    frame_attributes = {}
     # Default empty frame_attributes dict
 
+    def __init_subclass__(cls, **kwargs):
+
+        # We first check for explicitly set values for these:
+        default_repr = getattr(cls, 'default_representation', None)
+        default_diff = getattr(cls, 'default_differential', None)
+        repr_info = getattr(cls, 'frame_specific_representation_info', None)
+        # Then, to make sure this works for subclasses-of-subclasses, we also
+        # have to check for cases where the attribute names have already been
+        # replaced by underscore-prefaced equivalents by the logic below:
+        if default_repr is None or isinstance(default_repr, property):
+            default_repr = getattr(cls, '_default_representation', None)
+
+        if default_diff is None or isinstance(default_diff, property):
+            default_diff = getattr(cls, '_default_differential', None)
+
+        if repr_info is None or isinstance(repr_info, property):
+            repr_info = getattr(cls, '_frame_specific_representation_info', None)
+
+        repr_info = cls._infer_repr_info(repr_info)
+
+        # Make read-only properties for the frame class attributes that should
+        # be read-only to make them immutable after creation.
+        # We copy attributes instead of linking to make sure there's no
+        # accidental cross-talk between classes
+        cls._create_readonly_property('default_representation', default_repr)
+        cls._create_readonly_property('default_differential', default_diff)
+        cls._create_readonly_property('frame_specific_representation_info',
+                                      copy.deepcopy(repr_info))
+
+        # Set the frame attributes. We first construct the attributes from
+        # superclasses, going in reverse order to keep insertion order,
+        # and then add any attributes from the frame now being defined
+        # (if any old definitions are overridden, this keeps the order).
+        # Note that we cannot simply start with the inherited frame_attributes
+        # since we could be a mixin between multiple coordinate frames.
+        # TODO: Should this be made to use readonly_prop_factory as well or
+        # would it be inconvenient for getting the frame_attributes from
+        # classes?
+        frame_attrs = {}
+        for basecls in reversed(cls.__bases__):
+            if issubclass(basecls, BaseCoordinateFrame):
+                frame_attrs.update(basecls.frame_attributes)
+
+        for k, v in cls.__dict__.items():
+            if isinstance(v, Attribute):
+                frame_attrs[k] = v
+
+        cls.frame_attributes = frame_attrs
+
+        # Deal with setting the name of the frame:
+        if not hasattr(cls, 'name'):
+            cls.name = cls.__name__.lower()
+        elif (BaseCoordinateFrame not in cls.__bases__ and
+                cls.name in [getattr(base, 'name', None)
+                             for base in cls.__bases__]):
+            # This may be a subclass of a subclass of BaseCoordinateFrame,
+            # like ICRS(BaseRADecFrame). In this case, cls.name will have been
+            # set by init_subclass
+            cls.name = cls.__name__.lower()
+
+        # A cache that *must be unique to each frame class* - it is
+        # insufficient to share them with superclasses, hence the need to put
+        # them in the meta
+        cls._frame_class_cache = {}
+
+        super().__init_subclass__(**kwargs)
+
     def __init__(self, *args, copy=True, representation_type=None,
                  differential_type=None, **kwargs):
         self._attr_names_with_defaults = []
@@ -423,32 +322,103 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         _normalize_representation_type(kwargs)
         representation_type = kwargs.pop('representation_type', representation_type)
 
-        if representation_type is not None or differential_type is not None:
+        self._representation = self._infer_representation(representation_type, differential_type)
+        self._data = self._infer_data(args, copy, kwargs)  # possibly None.
 
-            if representation_type is None:
-                representation_type = self.default_representation
+        # Set frame attributes, if any
 
-            if (inspect.isclass(differential_type)
-                    and issubclass(differential_type, r.BaseDifferential)):
-                # TODO: assumes the differential class is for the velocity
-                # differential
-                differential_type = {'s': differential_type}
+        values = {}
+        for fnm, fdefault in self.get_frame_attr_names().items():
+            # Read-only frame attributes are defined as FrameAttribute
+            # descriptors which are not settable, so set 'real' attributes as
+            # the name prefaced with an underscore.
 
-            elif isinstance(differential_type, str):
-                # TODO: assumes the differential class is for the velocity
-                # differential
-                diff_cls = r.DIFFERENTIAL_CLASSES[differential_type]
-                differential_type = {'s': diff_cls}
+            if fnm in kwargs:
+                value = kwargs.pop(fnm)
+                setattr(self, '_' + fnm, value)
+                # Validate attribute by getting it. If the instance has data,
+                # this also checks its shape is OK. If not, we do it below.
+                values[fnm] = getattr(self, fnm)
+            else:
+                setattr(self, '_' + fnm, fdefault)
+                self._attr_names_with_defaults.append(fnm)
 
-            elif differential_type is None:
-                if representation_type == self.default_representation:
-                    differential_type = {'s': self.default_differential}
+        if kwargs:
+            raise TypeError(
+                f'Coordinate frame {self.__class__.__name__} got unexpected '
+                f'keywords: {list(kwargs)}')
+
+        # We do ``is None`` because self._data might evaluate to false for
+        # empty arrays or data == 0
+        if self._data is None:
+            # No data: we still need to check that any non-scalar attributes
+            # have consistent shapes. Collect them for all attributes with
+            # size > 1 (which should be array-like and thus have a shape).
+            shapes = {fnm: value.shape for fnm, value in values.items()
+                      if getattr(value, 'shape', ())}
+            if shapes:
+                if len(shapes) > 1:
+                    try:
+                        self._no_data_shape = check_broadcast(*shapes.values())
+                    except ValueError:
+                        raise ValueError(
+                            f"non-scalar attributes with inconsistent shapes: {shapes}")
+
+                    # Above, we checked that it is possible to broadcast all
+                    # shapes.  By getting and thus validating the attributes,
+                    # we verify that the attributes can in fact be broadcast.
+                    for fnm in shapes:
+                        getattr(self, fnm)
                 else:
-                    differential_type = {'s': 'base'}  # see set_representation_cls()
+                    self._no_data_shape = shapes.popitem()[1]
 
-            self.set_representation_cls(representation_type,
-                                        **differential_type)
+            else:
+                self._no_data_shape = ()
 
+        # The logic of this block is not related to the previous one
+        if self._data is not None:
+            # This makes the cache keys backwards-compatible, but also adds
+            # support for having differentials attached to the frame data
+            # representation object.
+            if 's' in self._data.differentials:
+                # TODO: assumes a velocity unit differential
+                key = (self._data.__class__.__name__,
+                       self._data.differentials['s'].__class__.__name__,
+                       False)
+            else:
+                key = (self._data.__class__.__name__, False)
+
+            # Set up representation cache.
+            self.cache['representation'][key] = self._data
+
+    def _infer_representation(self, representation_type, differential_type):
+        if representation_type is None and differential_type is None:
+            return {'base': self.default_representation, 's': self.default_differential}
+
+        if representation_type is None:
+            representation_type = self.default_representation
+
+        if (inspect.isclass(differential_type)
+                and issubclass(differential_type, r.BaseDifferential)):
+            # TODO: assumes the differential class is for the velocity
+            # differential
+            differential_type = {'s': differential_type}
+
+        elif isinstance(differential_type, str):
+            # TODO: assumes the differential class is for the velocity
+            # differential
+            diff_cls = r.DIFFERENTIAL_CLASSES[differential_type]
+            differential_type = {'s': diff_cls}
+
+        elif differential_type is None:
+            if representation_type == self.default_representation:
+                differential_type = {'s': self.default_differential}
+            else:
+                differential_type = {'s': 'base'}  # see set_representation_cls()
+
+        return _get_repr_classes(representation_type, **differential_type)
+
+    def _infer_data(self, args, copy, kwargs):
         # if not set below, this is a frame with no data
         representation_data = None
         differential_data = None
@@ -456,7 +426,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         args = list(args)  # need to be able to pop them
         if (len(args) > 0) and (isinstance(args[0], r.BaseRepresentation) or
                                 args[0] is None):
-            representation_data = args.pop(0)
+            representation_data = args.pop(0)  # This can still be None
             if len(args) > 0:
                 raise TypeError(
                     'Cannot create a frame with both a representation object '
@@ -473,7 +443,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
                                      'velocity differential is supported. Got: '
                                      '{}'.format(diffs))
 
-        elif self.representation_type:
+        else:
             representation_cls = self.get_representation_cls()
             # Get any representation data passed in to the frame initializer
             # using keyword or positional arguments for the component names
@@ -582,73 +552,99 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
                         raise ValueError(
                             "Differential data units are not compatible with"
                             "time-derivative of representation data units")
-            self._data = representation_data.with_differentials(
-                {'s': differential_data})
-        else:
-            self._data = representation_data  # possibly None.
 
-        values = {}
-        for fnm, fdefault in self.get_frame_attr_names().items():
-            # Read-only frame attributes are defined as FrameAttribue
-            # descriptors which are not settable, so set 'real' attributes as
-            # the name prefaced with an underscore.
+            representation_data = representation_data.with_differentials({'s': differential_data})
 
-            if fnm in kwargs:
-                value = kwargs.pop(fnm)
-                setattr(self, '_' + fnm, value)
-                # Validate attribute by getting it.  If the instance has data,
-                # this also checks its shape is OK.  If not, we do it below.
-                values[fnm] = getattr(self, fnm)
-            else:
-                setattr(self, '_' + fnm, fdefault)
-                self._attr_names_with_defaults.append(fnm)
+        return representation_data
 
-        if kwargs:
-            raise TypeError(
-                'Coordinate frame got unexpected keywords: {}'.format(
-                    list(kwargs)))
+    @classmethod
+    def _infer_repr_info(cls, repr_info):
+        # Unless overridden via `frame_specific_representation_info`, velocity
+        # name defaults are (see also docstring for BaseCoordinateFrame):
+        #   * ``pm_{lon}_cos{lat}``, ``pm_{lat}`` for
+        #     `SphericalCosLatDifferential` proper motion components
+        #   * ``pm_{lon}``, ``pm_{lat}`` for `SphericalDifferential` proper
+        #     motion components
+        #   * ``radial_velocity`` for any `d_distance` component
+        #   * ``v_{x,y,z}`` for `CartesianDifferential` velocity components
+        # where `{lon}` and `{lat}` are the frame names of the angular
+        # components.
+        if repr_info is None:
+            repr_info = {}
 
-        # We do ``is None`` because self._data might evaluate to false for
-        # empty arrays or data == 0
-        if self._data is None:
-            # No data: we still need to check that any non-scalar attributes
-            # have consistent shapes. Collect them for all attributes with
-            # size > 1 (which should be array-like and thus have a shape).
-            shapes = {fnm: value.shape for fnm, value in values.items()
-                      if getattr(value, 'shape', ())}
-            if shapes:
-                if len(shapes) > 1:
-                    try:
-                        self._no_data_shape = check_broadcast(*shapes.values())
-                    except ValueError:
-                        raise ValueError(
-                            "non-scalar attributes with inconsistent "
-                            "shapes: {}".format(shapes))
+        # the tuple() call below is necessary because if it is not there,
+        # the iteration proceeds in a difficult-to-predict manner in the
+        # case that one of the class objects hash is such that it gets
+        # revisited by the iteration.  The tuple() call prevents this by
+        # making the items iterated over fixed regardless of how the dict
+        # changes
+        for cls_or_name in tuple(repr_info.keys()):
+            if isinstance(cls_or_name, str):
+                # TODO: this provides a layer of backwards compatibility in
+                # case the key is a string, but now we want explicit classes.
+                _cls = _get_repr_cls(cls_or_name)
+                repr_info[_cls] = repr_info.pop(cls_or_name)
 
-                    # Above, we checked that it is possible to broadcast all
-                    # shapes.  By getting and thus validating the attributes,
-                    # we verify that the attributes can in fact be broadcast.
-                    for fnm in shapes:
-                        getattr(self, fnm)
-                else:
-                    self._no_data_shape = shapes.popitem()[1]
+        # The default spherical names are 'lon' and 'lat'
+        repr_info.setdefault(r.SphericalRepresentation,
+                             [RepresentationMapping('lon', 'lon'),
+                              RepresentationMapping('lat', 'lat')])
 
-            else:
-                self._no_data_shape = ()
-        else:
-            # This makes the cache keys backwards-compatible, but also adds
-            # support for having differentials attached to the frame data
-            # representation object.
-            if 's' in self._data.differentials:
-                # TODO: assumes a velocity unit differential
-                key = (self._data.__class__.__name__,
-                       self._data.differentials['s'].__class__.__name__,
-                       False)
-            else:
-                key = (self._data.__class__.__name__, False)
+        sph_component_map = {m.reprname: m.framename
+                             for m in repr_info[r.SphericalRepresentation]}
 
-            # Set up representation cache.
-            self.cache['representation'][key] = self._data
+        repr_info.setdefault(r.SphericalCosLatDifferential, [
+            RepresentationMapping(
+                'd_lon_coslat',
+                'pm_{lon}_cos{lat}'.format(**sph_component_map),
+                u.mas/u.yr),
+            RepresentationMapping('d_lat',
+                                  'pm_{lat}'.format(**sph_component_map),
+                                  u.mas/u.yr),
+            RepresentationMapping('d_distance', 'radial_velocity',
+                                  u.km/u.s)
+        ])
+
+        repr_info.setdefault(r.SphericalDifferential, [
+            RepresentationMapping('d_lon',
+                                  'pm_{lon}'.format(**sph_component_map),
+                                  u.mas/u.yr),
+            RepresentationMapping('d_lat',
+                                  'pm_{lat}'.format(**sph_component_map),
+                                  u.mas/u.yr),
+            RepresentationMapping('d_distance', 'radial_velocity',
+                                  u.km/u.s)
+        ])
+
+        repr_info.setdefault(r.CartesianDifferential, [
+            RepresentationMapping('d_x', 'v_x', u.km/u.s),
+            RepresentationMapping('d_y', 'v_y', u.km/u.s),
+            RepresentationMapping('d_z', 'v_z', u.km/u.s)])
+
+        # Unit* classes should follow the same naming conventions
+        # TODO: this adds some unnecessary mappings for the Unit classes, so
+        # this could be cleaned up, but in practice doesn't seem to have any
+        # negative side effects
+        repr_info.setdefault(r.UnitSphericalRepresentation,
+                             repr_info[r.SphericalRepresentation])
+
+        repr_info.setdefault(r.UnitSphericalCosLatDifferential,
+                             repr_info[r.SphericalCosLatDifferential])
+
+        repr_info.setdefault(r.UnitSphericalDifferential,
+                             repr_info[r.SphericalDifferential])
+
+        return repr_info
+
+    @classmethod
+    def _create_readonly_property(cls, attr_name, value):
+        private_attr = '_' + attr_name
+
+        def getter(self):
+            return getattr(self, private_attr)
+
+        setattr(cls, private_attr, value)
+        setattr(cls, attr_name, property(getter))
 
     @lazyproperty
     def cache(self):
@@ -713,8 +709,8 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
     @classmethod
     def get_frame_attr_names(cls):
-        return OrderedDict((name, getattr(cls, name))
-                           for name in cls.frame_attributes)
+        return {name: getattr(cls, name)
+                for name in cls.frame_attributes}
 
     def get_representation_cls(self, which='base'):
         """The class used for part of this frame's data.
@@ -731,10 +727,6 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         -------
         representation : `~astropy.coordinates.BaseRepresentation` or `~astropy.coordinates.BaseDifferential`.
         """
-        if not hasattr(self, '_representation'):
-            self._representation = {'base': self.default_representation,
-                                    's': self.default_differential}
-
         if which is not None:
             return self._representation[which]
         else:
@@ -848,7 +840,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         return self._get_representation_info()
 
     def get_representation_component_names(self, which='base'):
-        out = OrderedDict()
+        out = {}
         repr_or_diff_cls = self.get_representation_cls(which)
         if repr_or_diff_cls is None:
             return out
@@ -859,7 +851,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         return out
 
     def get_representation_component_units(self, which='base'):
-        out = OrderedDict()
+        out = {}
         repr_or_diff_cls = self.get_representation_cls(which)
         if repr_or_diff_cls is None:
             return out
@@ -883,7 +875,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Parameters
         ----------
-        data : `~astropy.coordinates.BaseRepresentation` or `None`
+        data : `~astropy.coordinates.BaseRepresentation` or None
             Data to use in the new frame instance.  If `None`, it will be
             a data-less frame.
         copy : bool, optional
@@ -939,7 +931,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Returns
         -------
-        frameobj : same as this frame
+        frameobj : `BaseCoordinateFrame` subclass instance
             Replica of this object, but possibly with new frame attributes.
         """
         return self._replicate(self.data, copy=copy, **kwargs)
@@ -967,7 +959,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Returns
         -------
-        frameobj : same as this frame
+        frameobj : `BaseCoordinateFrame` subclass instance
             Replica of this object, but without data and possibly with new frame
             attributes.
         """
@@ -989,9 +981,10 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Returns
         -------
-        frameobj : same as this frame
-            A new object with the same frame attributes as this one, but
-            with the ``data`` as the coordinate data.
+        frameobj : `BaseCoordinateFrame` subclass instance
+            A new object in *this* frame, with the same frame attributes as
+            this one, but with the ``data`` as the coordinate data.
+
         """
         return self._replicate(data, **kwargs)
 
@@ -1016,7 +1009,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
             a *class* (not an instance), or the string name of the
             differential class.  If equal to 'base' (default), inferred from
             the base class.  If `None`, all velocity information is dropped.
-        in_frame_units : bool, keyword only
+        in_frame_units : bool, keyword-only
             Force the representation units to match the specified units
             particular to this frame
 
@@ -1063,7 +1056,15 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         representation_cls = repr_classes['base']
         # We only keep velocity information
         if 's' in self.data.differentials:
-            differential_cls = repr_classes['s']
+            # For the default 'base' option in which _get_repr_classes has
+            # given us a best guess based on the representation class, we only
+            # use it if the class we had already is incompatible.
+            if (s == 'base'
+                and (self.data.differentials['s'].__class__
+                     in representation_cls._compatible_differentials)):
+                differential_cls = self.data.differentials['s'].__class__
+            else:
+                differential_cls = repr_classes['s']
         elif s is None or s == 'base':
             differential_cls = None
         else:
@@ -1175,14 +1176,16 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Parameters
         ----------
-        new_frame : frame object or SkyCoord object
+        new_frame : coordinate-like or `BaseCoordinateFrame` subclass instance
             The frame to transform this coordinate frame into.
+            The frame class option is deprecated.
 
         Returns
         -------
-        transframe
+        transframe : coordinate-like
             A new object with the coordinate data represented in the
-            ``newframe`` system.
+            ``newframe`` system (meaning a `~astropy.coordinates.SkyCoord` will
+            remain a SkyCoord).
 
         Raises
         ------
@@ -1235,7 +1238,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Parameters
         ----------
-        new_frame : class or frame object
+        new_frame : `BaseCoordinateFrame` subclass or instance
             The proposed frame to transform into.
 
         Returns
@@ -1260,7 +1263,6 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         transformation between two objects of the same frame class but with
         different attributes.
         """
-
         new_frame_cls = new_frame if inspect.isclass(new_frame) else new_frame.__class__
         trans = frame_transform_graph.get_transform(self.__class__, new_frame_cls)
 
@@ -1295,7 +1297,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
     def _frameattr_equiv(left_fattr, right_fattr):
         """
         Determine if two frame attributes are equivalent.  Implemented as a
-        staticmethod mainly as a convenient location, althought conceivable it
+        staticmethod mainly as a convenient location, although conceivable it
         might be desirable for subclasses to override this behavior.
 
         Primary purpose is to check for equality of representations.  This
@@ -1362,7 +1364,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
 
         Parameters
         ----------
-        other : BaseCoordinateFrame
+        other : :class:`~astropy.coordinates.BaseCoordinateFrame`
             the other frame to check
 
         Returns
@@ -1395,11 +1397,9 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
             frameattrs = f' ({frameattrs})'
 
         if data_repr:
-            return '<{} Coordinate{}: {}>'.format(self.__class__.__name__,
-                                                  frameattrs, data_repr)
+            return f'<{self.__class__.__name__} Coordinate{frameattrs}: {data_repr}>'
         else:
-            return '<{} Frame{}>'.format(self.__class__.__name__,
-                                         frameattrs)
+            return f'<{self.__class__.__name__} Frame{frameattrs}>'
 
     def _data_repr(self):
         """Returns a string representation of the coordinate data."""
@@ -1483,9 +1483,7 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
                 attrstr = attr._astropy_repr_in_frame()
             else:
                 attrstr = str(attr)
-            attr_strs.append("{attribute_name}={attrstr}".format(
-                attribute_name=attribute_name,
-                attrstr=attrstr))
+            attr_strs.append(f"{attribute_name}={attrstr}")
 
         return ', '.join(attr_strs)
 
@@ -1509,9 +1507,9 @@ class BaseCoordinateFrame(ShapedLikeNDArray, metaclass=FrameMeta):
         method : str or callable
             If str, it is the name of a method that is applied to the internal
             ``components``. If callable, the function is applied.
-        args : tuple
+        *args : tuple
             Any positional arguments for ``method``.
-        kwargs : dict
+        **kwargs : dict
             Any keyword arguments for ``method``.
         """
         def apply_method(value):
@@ -1899,7 +1897,7 @@ class GenericFrame(BaseCoordinateFrame):
     name = None  # it's not a "real" frame so it doesn't have a name
 
     def __init__(self, frame_attrs):
-        self.frame_attributes = OrderedDict()
+        self.frame_attributes = {}
         for name, default in frame_attrs.items():
             self.frame_attributes[name] = Attribute(default)
             setattr(self, '_' + name, default)
