@@ -9,11 +9,10 @@ rotations without aberration corrections or offsets.
 import numpy as np
 import erfa
 
-import astropy.units as u
 from astropy.coordinates.baseframe import frame_transform_graph
 from astropy.coordinates.transformations import FunctionTransformWithFiniteDifference
 from astropy.coordinates.matrix_utilities import matrix_transpose
-from astropy.coordinates.representation import CartesianDifferential
+
 
 from .gcrs import GCRS, PrecessedGeocentric
 from .cirs import CIRS
@@ -31,7 +30,7 @@ def teme_to_itrs_mat(time):
     gst = erfa.gmst82(*get_jd12(time, 'ut1'))
 
     # Polar Motion
-    # Do not include TIO locator s' because it is not used in Vallado 2006
+    # Do not include TIO locator s' because it is not used in Vallado 2006
     xp, yp = get_polar_motion(time)
     pmmat = erfa.pom00(xp, yp, 0)
 
@@ -63,8 +62,12 @@ def cirs_to_itrs_mat(time):
     return erfa.c2tcio(np.eye(3), era, pmmat)
 
 
-def tete_to_itrs_mat(time):
-    # compute the polar motion p-matrix
+def tete_to_itrs_mat(time, rbpn=None):
+    """Compute the polar motion p-matrix at the given time.
+
+    If the nutation-precession matrix is already known, it should be passed in,
+    as this is by far the most expensive calculation.
+    """
     xp, yp = get_polar_motion(time)
     sp = erfa.sp00(*get_jd12(time, 'tt'))
     pmmat = erfa.pom00(xp, yp, sp)
@@ -73,7 +76,12 @@ def tete_to_itrs_mat(time):
     # we use the 2006A model for consistency with RBPN matrix use in GCRS <-> TETE
     ujd1, ujd2 = get_jd12(time, 'ut1')
     jd1, jd2 = get_jd12(time, 'tt')
-    gast = erfa.gst06a(ujd1, ujd2, jd1, jd2)
+    if rbpn is None:
+        # erfa.gst06a calls pnm06a to calculate rbpn and then gst06. Use it in
+        # favour of getting rbpn with erfa.pnm06a to avoid a possibly large array.
+        gast = erfa.gst06a(ujd1, ujd2, jd1, jd2)
+    else:
+        gast = erfa.gst06(ujd1, ujd2, jd1, jd2, rbpn)
 
     # c2tcio expects a GCRS->CIRS matrix, but we just set that to an I-matrix
     # because we're already in CIRS equivalent frame
@@ -85,25 +93,21 @@ def gcrs_precession_mat(equinox):
     return erfa.fw2m(gamb, phib, psib, epsa)
 
 
-def get_location_gcrs(reference, gcrs_to_reference_matrix):
-    """Create a GCRS frame at the location and obstime of a given reference.
+def get_location_gcrs(location, obstime, ref_to_itrs, gcrs_to_ref):
+    """Create a GCRS frame at the location and obstime.
+
+    The reference frame z axis must point to the Celestial Intermediate Pole
+    (as is the case for CIRS and TETE).
+
+    This function is here to avoid location.get_gcrs(obstime), which would
+    recalculate matrices that are already available below (and return a GCRS
+    coordinate, rather than a frame with obsgeoloc and obsgeovel).  Instead,
+    it uses the private method that allows passing in the matrices.
 
-    Helper function that avoids location.get_gcrs (which would trigger
-    infinite recursion in some cases), and uses an already calculated
-    GCRS to Reference matrix to calculate obsgeoloc and obsgeovel
-    required for GCRS.
     """
-    # TODO: ideally, GCRS and other frames would share how they describe
-    # the location.  See gh-10996.
-    obstime = reference.obstime
-    loc_itrs = reference.location.get_itrs(obstime)
-    zeros = np.broadcast_to(0. * u.km / u.s, (3,) + loc_itrs.shape, subok=True)
-    loc_itrs.data.differentials['s'] = CartesianDifferential(zeros)
-    gcrs_cart = (loc_itrs.transform_to(reference.__class__(obstime=obstime))
-                 .cartesian.transform(matrix_transpose(gcrs_to_reference_matrix)))
-    return GCRS(obstime=obstime,
-                obsgeoloc=gcrs_cart.without_differentials(),
-                obsgeovel=gcrs_cart.differentials['s'].to_cartesian())
+    obsgeoloc, obsgeovel = location._get_gcrs_posvel(obstime,
+                                                     ref_to_itrs, gcrs_to_ref)
+    return GCRS(obstime=obstime, obsgeoloc=obsgeoloc, obsgeovel=obsgeovel)
 
 
 # now the actual transforms
@@ -114,7 +118,9 @@ def gcrs_to_tete(gcrs_coo, tete_frame):
     # (same as in builtin_frames.utils.get_cip).
     rbpn = erfa.pnm06a(*get_jd12(tete_frame.obstime, 'tt'))
     # Get GCRS coordinates for the target observer location and time.
-    loc_gcrs = get_location_gcrs(tete_frame, rbpn)
+    loc_gcrs = get_location_gcrs(tete_frame.location, tete_frame.obstime,
+                                 tete_to_itrs_mat(tete_frame.obstime, rbpn=rbpn),
+                                 rbpn)
     gcrs_coo2 = gcrs_coo.transform_to(loc_gcrs)
     # Now we are relative to the correct observer, do the transform to TETE.
     # These rotations are defined at the geocenter, but can be applied to
@@ -131,7 +137,9 @@ def tete_to_gcrs(tete_coo, gcrs_frame):
     newrepr = tete_coo.cartesian.transform(matrix_transpose(rbpn))
     # We now have a GCRS vector for the input location and obstime.
     # Turn it into a GCRS frame instance.
-    loc_gcrs = get_location_gcrs(tete_coo, rbpn)
+    loc_gcrs = get_location_gcrs(tete_coo.location, tete_coo.obstime,
+                                 tete_to_itrs_mat(tete_coo.obstime, rbpn=rbpn),
+                                 rbpn)
     gcrs = loc_gcrs.realize_frame(newrepr)
     # Finally, do any needed offsets (no-op if same obstime and location)
     return gcrs.transform_to(gcrs_frame)
@@ -162,30 +170,36 @@ def itrs_to_tete(itrs_coo, tete_frame):
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference, GCRS, CIRS)
 def gcrs_to_cirs(gcrs_coo, cirs_frame):
-    # first get us to a 0 pos/vel GCRS at the target obstime
-    gcrs_coo2 = gcrs_coo.transform_to(GCRS(obstime=cirs_frame.obstime))
-
-    # now get the pmatrix
+    # first get the pmatrix
     pmat = gcrs_to_cirs_mat(cirs_frame.obstime)
+    # Get GCRS coordinates for the target observer location and time.
+    loc_gcrs = get_location_gcrs(cirs_frame.location, cirs_frame.obstime,
+                                 cirs_to_itrs_mat(cirs_frame.obstime), pmat)
+    gcrs_coo2 = gcrs_coo.transform_to(loc_gcrs)
+    # Now we are relative to the correct observer, do the transform to CIRS.
     crepr = gcrs_coo2.cartesian.transform(pmat)
     return cirs_frame.realize_frame(crepr)
 
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference, CIRS, GCRS)
 def cirs_to_gcrs(cirs_coo, gcrs_frame):
-    # compute the pmatrix, and then multiply by its transpose
+    # Compute the pmatrix, and then multiply by its transpose,
     pmat = gcrs_to_cirs_mat(cirs_coo.obstime)
     newrepr = cirs_coo.cartesian.transform(matrix_transpose(pmat))
-    gcrs = GCRS(newrepr, obstime=cirs_coo.obstime)
-
-    # now do any needed offsets (no-op if same obstime and 0 pos/vel)
+    # We now have a GCRS vector for the input location and obstime.
+    # Turn it into a GCRS frame instance.
+    loc_gcrs = get_location_gcrs(cirs_coo.location, cirs_coo.obstime,
+                                 cirs_to_itrs_mat(cirs_coo.obstime), pmat)
+    gcrs = loc_gcrs.realize_frame(newrepr)
+    # Finally, do any needed offsets (no-op if same obstime and location)
     return gcrs.transform_to(gcrs_frame)
 
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference, CIRS, ITRS)
 def cirs_to_itrs(cirs_coo, itrs_frame):
-    # first get us to CIRS at the target obstime
-    cirs_coo2 = cirs_coo.transform_to(CIRS(obstime=itrs_frame.obstime))
+    # first get us to geocentric CIRS at the target obstime
+    cirs_coo2 = cirs_coo.transform_to(CIRS(obstime=itrs_frame.obstime,
+                                           location=EARTH_CENTER))
 
     # now get the pmatrix
     pmat = cirs_to_itrs_mat(itrs_frame.obstime)
@@ -210,11 +224,6 @@ def itrs_to_itrs(from_coo, to_frame):
     # goes back to ICRS
     return from_coo.transform_to(CIRS()).transform_to(to_frame)
 
-# TODO: implement GCRS<->CIRS if there's call for it.  The thing that's awkward
-# is that they both have obstimes, so an extra set of transformations are necessary.
-# so unless there's a specific need for that, better to just have it go through the above
-# two steps anyway
-
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference, GCRS, PrecessedGeocentric)
 def gcrs_to_precessedgeo(from_coo, to_frame):
