@@ -225,10 +225,10 @@ class SkyCoord(ShapedLikeNDArray):
         Note that passing only one unit might lead to unit conversion errors
         if the coordinate values are expected to have mixed physical meanings
         (e.g., angles and distances).
-    obstime : valid `~astropy.time.Time` initializer, optional
+    obstime : time-like, optional
         Time(s) of observation.
-    equinox : valid `~astropy.time.Time` initializer, optional
-        Coordinate frame equinox.
+    equinox : time-like, optional
+        Coordinate frame equinox time.
     representation_type : str or Representation class
         Specifies the representation, e.g. 'spherical', 'cartesian', or
         'cylindrical'.  This affects the positional args and other keyword args
@@ -240,24 +240,24 @@ class SkyCoord(ShapedLikeNDArray):
         Other keyword arguments as applicable for user-defined coordinate frames.
         Common options include:
 
-        ra, dec : valid `~astropy.coordinates.Angle` initializer, optional
+        ra, dec : angle-like, optional
             RA and Dec for frames where ``ra`` and ``dec`` are keys in the
             frame's ``representation_component_names``, including `ICRS`,
             `FK5`, `FK4`, and `FK4NoETerms`.
-        pm_ra_cosdec, pm_dec  : `~astropy.units.Quantity`, optional
+        pm_ra_cosdec, pm_dec  : `~astropy.units.Quantity` ['angular speed'], optional
             Proper motion components, in angle per time units.
-        l, b : valid `~astropy.coordinates.Angle` initializer, optional
+        l, b : angle-like, optional
             Galactic ``l`` and ``b`` for for frames where ``l`` and ``b`` are
             keys in the frame's ``representation_component_names``, including
             the `Galactic` frame.
-        pm_l_cosb, pm_b : `~astropy.units.Quantity`, optional
+        pm_l_cosb, pm_b : `~astropy.units.Quantity` ['angular speed'], optional
             Proper motion components in the `Galactic` frame, in angle per time
             units.
-        x, y, z : float or `~astropy.units.Quantity`, optional
+        x, y, z : float or `~astropy.units.Quantity` ['length'], optional
             Cartesian coordinates values
-        u, v, w : float or `~astropy.units.Quantity`, optional
+        u, v, w : float or `~astropy.units.Quantity` ['length'], optional
             Cartesian coordinates values for the Galactic frame.
-        radial_velocity : `~astropy.units.Quantity`, optional
+        radial_velocity : `~astropy.units.Quantity` ['speed'], optional
             The component of the velocity along the line-of-sight (i.e., the
             radial direction), in velocity units.
     """
@@ -364,6 +364,8 @@ class SkyCoord(ShapedLikeNDArray):
         equivalent, extra frame attributes are equivalent, and that the
         representation data are exactly equal.
         """
+        if not isinstance(value, SkyCoord):
+            return NotImplemented
         # Make sure that any extra frame attribute names are equivalent.
         for attr in self._extra_frameattr_names | value._extra_frameattr_names:
             if not self.frame._frameattr_equiv(getattr(self, attr),
@@ -483,7 +485,7 @@ class SkyCoord(ShapedLikeNDArray):
         ----------
         obj : int
             Integer index before which ``values`` is inserted.
-        values : array_like
+        values : array-like
             Value(s) to insert.  If the type of ``values`` is different
             from that of quantity, ``values`` is converted to the matching type.
         axis : int, optional
@@ -532,6 +534,43 @@ class SkyCoord(ShapedLikeNDArray):
 
         return out
 
+    def is_transformable_to(self, new_frame):
+        """
+        Determines if this coordinate frame can be transformed to another
+        given frame.
+
+        Parameters
+        ----------
+        new_frame : frame class, frame object, or str
+            The proposed frame to transform into.
+
+        Returns
+        -------
+        transformable : bool or str
+            `True` if this can be transformed to ``new_frame``, `False` if
+            not, or the string 'same' if ``new_frame`` is the same system as
+            this object but no transformation is defined.
+
+        Notes
+        -----
+        A return value of 'same' means the transformation will work, but it will
+        just give back a copy of this object.  The intended usage is::
+
+            if coord.is_transformable_to(some_unknown_frame):
+                coord2 = coord.transform_to(some_unknown_frame)
+
+        This will work even if ``some_unknown_frame``  turns out to be the same
+        frame class as ``coord``.  This is intended for cases where the frame
+        is the same regardless of the frame attributes (e.g. ICRS), but be
+        aware that it *might* also indicate that someone forgot to define the
+        transformation between two objects of the same frame class but with
+        different attributes.
+        """
+        # TODO! like matplotlib, do string overrides for modified methods
+        new_frame = (_get_frame_class(new_frame) if isinstance(new_frame, str)
+                     else new_frame)
+        return self.frame.is_transformable_to(new_frame)
+
     def transform_to(self, frame, merge_attributes=True):
         """Transform this coordinate to a new frame.
 
@@ -624,6 +663,13 @@ class SkyCoord(ShapedLikeNDArray):
                      set(frame_kwargs.keys())):
             frame_kwargs.pop(attr)
 
+        # Always remove the origin frame attribute, as that attribute only makes
+        # sense with a SkyOffsetFrame (in which case it will be stored on the frame).
+        # See gh-11277.
+        # TODO: Should it be a property of the frame attribute that it can
+        # or cannot be stored on a SkyCoord?
+        frame_kwargs.pop('origin', None)
+
         return self.__class__(new_coord, **frame_kwargs)
 
     def apply_space_motion(self, new_obstime=None, dt=None):
@@ -760,7 +806,7 @@ class SkyCoord(ShapedLikeNDArray):
     def __getattr__(self, attr):
         """
         Overrides getattr to return coordinates that this can be transformed
-        to, based on the alias attr in the master transform graph.
+        to, based on the alias attr in the primary transform graph.
         """
         if '_sky_coord_frame' in self.__dict__:
             if self._is_name(attr):
@@ -923,7 +969,7 @@ class SkyCoord(ShapedLikeNDArray):
             lonargs.update(styles[style]['lonargs'])
             latargs.update(styles[style]['latargs'])
         else:
-            raise ValueError('Invalid style.  Valid options are: {}'.format(",".join(styles)))
+            raise ValueError(f"Invalid style.  Valid options are: {','.join(styles)}")
 
         lonargs.update(kwargs)
         latargs.update(kwargs)
@@ -1090,10 +1136,12 @@ class SkyCoord(ShapedLikeNDArray):
         Returns
         -------
         lon_offset : `~astropy.coordinates.Angle`
-            The angular offset in the longitude direction (i.e., RA for
+            The angular offset in the longitude direction. The definition of
+            "longitude" depends on this coordinate's frame (e.g., RA for
             equatorial coordinates).
         lat_offset : `~astropy.coordinates.Angle`
-            The angular offset in the latitude direction (i.e., Dec for
+            The angular offset in the latitude direction. The definition of
+            "latitude" depends on this coordinate's frame (e.g., Dec for
             equatorial coordinates).
 
         Raises
@@ -1126,6 +1174,45 @@ class SkyCoord(ShapedLikeNDArray):
         dlat = acoord.spherical.lat.view(Angle)
         return dlon, dlat
 
+    def spherical_offsets_by(self, d_lon, d_lat):
+        """
+        Computes the coordinate that is a specified pair of angular offsets away
+        from this coordinate.
+
+        Parameters
+        ----------
+        d_lon : angle-like
+            The angular offset in the longitude direction. The definition of
+            "longitude" depends on this coordinate's frame (e.g., RA for
+            equatorial coordinates).
+        d_lat : angle-like
+            The angular offset in the latitude direction. The definition of
+            "latitude" depends on this coordinate's frame (e.g., Dec for
+            equatorial coordinates).
+
+        Returns
+        -------
+        newcoord : `~astropy.coordinates.SkyCoord`
+            The coordinates for the location that corresponds to offsetting by
+            ``d_lat`` in the latitude direction and ``d_lon`` in the longitude
+            direction.
+
+        Notes
+        -----
+        This internally uses `~astropy.coordinates.SkyOffsetFrame` to do the
+        transformation. For a more complete set of transform offsets, use
+        `~astropy.coordinates.SkyOffsetFrame` or `~astropy.wcs.WCS` manually.
+        This specific method can be reproduced by doing
+        ``SkyCoord(SkyOffsetFrame(d_lon, d_lat, origin=self.frame).transform_to(self))``.
+
+        See Also
+        --------
+        spherical_offsets_to : compute the angular offsets to another coordinate
+        directional_offset_by : offset a coordinate by an angle in a direction
+        """
+        return self.__class__(
+            SkyOffsetFrame(d_lon, d_lat, origin=self.frame).transform_to(self))
+
     def directional_offset_by(self, position_angle, separation):
         """
         Computes coordinates at the given offset from this coordinate.
@@ -1197,7 +1284,7 @@ class SkyCoord(ShapedLikeNDArray):
 
         Returns
         -------
-        idx : integer array
+        idx : int array
             Indices into ``catalogcoord`` to get the matched points for
             each of this object's coordinates. Shape matches this
             object.
@@ -1205,7 +1292,7 @@ class SkyCoord(ShapedLikeNDArray):
             The on-sky separation between the closest match for each
             element in this object in ``catalogcoord``. Shape matches
             this object.
-        dist3d : `~astropy.units.Quantity`
+        dist3d : `~astropy.units.Quantity` ['length']
             The 3D distance between the closest match for each element
             in this object in ``catalogcoord``. Shape matches this
             object. Unless both this and ``catalogcoord`` have associated
@@ -1224,14 +1311,12 @@ class SkyCoord(ShapedLikeNDArray):
         """
         from .matching import match_coordinates_sky
 
-        if (isinstance(catalogcoord, (SkyCoord, BaseCoordinateFrame))
+        if not (isinstance(catalogcoord, (SkyCoord, BaseCoordinateFrame))
                 and catalogcoord.has_data):
-            self_in_catalog_frame = self.transform_to(catalogcoord)
-        else:
             raise TypeError('Can only get separation to another SkyCoord or a '
                             'coordinate frame with data')
 
-        res = match_coordinates_sky(self_in_catalog_frame, catalogcoord,
+        res = match_coordinates_sky(self, catalogcoord,
                                     nthneighbor=nthneighbor,
                                     storekdtree='_kdtree_sky')
         return res
@@ -1264,7 +1349,7 @@ class SkyCoord(ShapedLikeNDArray):
 
         Returns
         -------
-        idx : integer array
+        idx : int array
             Indices into ``catalogcoord`` to get the matched points for
             each of this object's coordinates. Shape matches this
             object.
@@ -1272,7 +1357,7 @@ class SkyCoord(ShapedLikeNDArray):
             The on-sky separation between the closest match for each
             element in this object in ``catalogcoord``. Shape matches
             this object.
-        dist3d : `~astropy.units.Quantity`
+        dist3d : `~astropy.units.Quantity` ['length']
             The 3D distance between the closest match for each element
             in this object in ``catalogcoord``. Shape matches this
             object.
@@ -1289,14 +1374,12 @@ class SkyCoord(ShapedLikeNDArray):
         """
         from .matching import match_coordinates_3d
 
-        if (isinstance(catalogcoord, (SkyCoord, BaseCoordinateFrame))
+        if not (isinstance(catalogcoord, (SkyCoord, BaseCoordinateFrame))
                 and catalogcoord.has_data):
-            self_in_catalog_frame = self.transform_to(catalogcoord)
-        else:
             raise TypeError('Can only get separation to another SkyCoord or a '
                             'coordinate frame with data')
 
-        res = match_coordinates_3d(self_in_catalog_frame, catalogcoord,
+        res = match_coordinates_3d(self, catalogcoord,
                                    nthneighbor=nthneighbor,
                                    storekdtree='_kdtree_3d')
 
@@ -1317,27 +1400,27 @@ class SkyCoord(ShapedLikeNDArray):
 
         Parameters
         ----------
-        searcharoundcoords : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
+        searcharoundcoords : coordinate-like
             The coordinates to search around to try to find matching points in
             this `SkyCoord`. This should be an object with array coordinates,
             not a scalar coordinate object.
-        seplimit : `~astropy.units.Quantity` with angle units
+        seplimit : `~astropy.units.Quantity` ['angle']
             The on-sky separation to search within.
 
         Returns
         -------
-        idxsearcharound : integer array
+        idxsearcharound : int array
             Indices into ``searcharoundcoords`` that match the
             corresponding elements of ``idxself``. Shape matches
             ``idxself``.
-        idxself : integer array
+        idxself : int array
             Indices into ``self`` that match the
             corresponding elements of ``idxsearcharound``. Shape matches
             ``idxsearcharound``.
         sep2d : `~astropy.coordinates.Angle`
             The on-sky separation between the coordinates. Shape matches
             ``idxsearcharound`` and ``idxself``.
-        dist3d : `~astropy.units.Quantity`
+        dist3d : `~astropy.units.Quantity` ['length']
             The 3D distance between the coordinates. Shape matches
             ``idxsearcharound`` and ``idxself``.
 
@@ -1380,23 +1463,23 @@ class SkyCoord(ShapedLikeNDArray):
             The coordinates to search around to try to find matching points in
             this `SkyCoord`. This should be an object with array coordinates,
             not a scalar coordinate object.
-        distlimit : `~astropy.units.Quantity` with distance units
+        distlimit : `~astropy.units.Quantity` ['length']
             The physical radius to search within.
 
         Returns
         -------
-        idxsearcharound : integer array
+        idxsearcharound : int array
             Indices into ``searcharoundcoords`` that match the
             corresponding elements of ``idxself``. Shape matches
             ``idxself``.
-        idxself : integer array
+        idxself : int array
             Indices into ``self`` that match the
             corresponding elements of ``idxsearcharound``. Shape matches
             ``idxsearcharound``.
         sep2d : `~astropy.coordinates.Angle`
             The on-sky separation between the coordinates. Shape matches
             ``idxsearcharound`` and ``idxself``.
-        dist3d : `~astropy.units.Quantity`
+        dist3d : `~astropy.units.Quantity` ['length']
             The 3D distance between the coordinates. Shape matches
             ``idxsearcharound`` and ``idxself``.
 
@@ -1476,7 +1559,7 @@ class SkyCoord(ShapedLikeNDArray):
             A sky offset frame of the same type as this `SkyCoord` (e.g., if
             this object has an ICRS coordinate, the resulting frame is
             SkyOffsetICRS, with the origin set to this object)
-        rotation : `~astropy.coordinates.Angle` or `~astropy.units.Quantity` with angle units
+        rotation : angle-like
             The final rotation of the frame about the ``origin``. The sign of
             the rotation is the left-hand rule. That is, an object at a
             particular position angle in the un-rotated system will be sent to
@@ -1566,7 +1649,7 @@ class SkyCoord(ShapedLikeNDArray):
 
         Parameters
         ----------
-        xp, yp : float or `numpy.ndarray`
+        xp, yp : float or ndarray
             The coordinates to convert.
         wcs : `~astropy.wcs.WCS`
             The WCS to use for convert
@@ -1578,7 +1661,7 @@ class SkyCoord(ShapedLikeNDArray):
 
         Returns
         -------
-        coord : an instance of this class
+        coord : `~astropy.coordinates.SkyCoord`
             A new object with sky coordinates corresponding to the input ``xp``
             and ``yp``.
 
@@ -1659,7 +1742,7 @@ class SkyCoord(ShapedLikeNDArray):
 
         Returns
         -------
-        vcorr : `~astropy.units.Quantity` with velocity units
+        vcorr : `~astropy.units.Quantity` ['speed']
             The  correction with a positive sign.  I.e., *add* this
             to an observed radial velocity to get the barycentric (or
             heliocentric) velocity. If m/s precision or better is needed,
@@ -1842,10 +1925,10 @@ class SkyCoord(ShapedLikeNDArray):
         in an astropy Table.
 
         This method matches table columns that start with the case-insensitive
-        names of the the components of the requested frames, if they are also
-        followed by a non-alphanumeric character. It will also match columns
-        that *end* with the component name if a non-alphanumeric character is
-        *before* it.
+        names of the the components of the requested frames (including
+        differentials), if they are also followed by a non-alphanumeric
+        character. It will also match columns that *end* with the component name
+        if a non-alphanumeric character is *before* it.
 
         For example, the first rule means columns with names like
         ``'RA[J2000]'`` or ``'ra'`` will be interpreted as ``ra`` attributes for
@@ -1862,23 +1945,36 @@ class SkyCoord(ShapedLikeNDArray):
 
         Parameters
         ----------
-        table : astropy.Table
+        table : `~astropy.table.Table` or subclass
             The table to load data from.
-        coord_kwargs
+        **coord_kwargs
             Any additional keyword arguments are passed directly to this class's
             constructor.
 
         Returns
         -------
-        newsc : same as this class
+        newsc : `~astropy.coordinates.SkyCoord` or subclass
             The new `SkyCoord` (or subclass) object.
+
+        Raises
+        ------
+        ValueError
+            If more than one match is found in the table for a component,
+            unless the additional matches are also valid frame component names.
+            If a "coord_kwargs" is provided for a value also found in the table.
+
         """
         _frame_cls, _frame_kwargs = _get_frame_without_data([], coord_kwargs)
         frame = _frame_cls(**_frame_kwargs)
         coord_kwargs['frame'] = coord_kwargs.get('frame', frame)
 
+        representation_component_names = (
+            set(frame.get_representation_component_names())
+            .union(set(frame.get_representation_component_names("s")))
+        )
+
         comp_kwargs = {}
-        for comp_name in frame.representation_component_names:
+        for comp_name in representation_component_names:
             # this matches things like 'ra[...]'' but *not* 'rad'.
             # note that the "_" must be in there explicitly, because
             # "alphanumeric" usually includes underscores.
@@ -1887,18 +1983,32 @@ class SkyCoord(ShapedLikeNDArray):
             # 'aura'
             ends_with_comp = r'.*(\W|\b|_)' + comp_name + r'\b'
             # the final regex ORs together the two patterns
-            rex = re.compile('(' + starts_with_comp + ')|(' + ends_with_comp + ')',
+            rex = re.compile(rf"({starts_with_comp})|({ends_with_comp})",
                              re.IGNORECASE | re.UNICODE)
 
-            for col_name in table.colnames:
-                if rex.match(col_name):
-                    if comp_name in comp_kwargs:
-                        oldname = comp_kwargs[comp_name].name
-                        msg = ('Found at least two matches for  component "{0}"'
-                               ': "{1}" and "{2}". Cannot continue with this '
-                               'ambiguity.')
-                        raise ValueError(msg.format(comp_name, oldname, col_name))
-                    comp_kwargs[comp_name] = table[col_name]
+            # find all matches
+            matches = {col_name for col_name in table.colnames
+                       if rex.match(col_name)}
+
+            # now need to select among matches, also making sure we don't have
+            # an exact match with another component
+            if len(matches) == 0:  # no matches
+                continue
+            elif len(matches) == 1:  # only one match
+                col_name = matches.pop()
+            else:  # more than 1 match
+                # try to sieve out other components
+                matches -= representation_component_names - {comp_name}
+                # if there's only one remaining match, it worked.
+                if len(matches) == 1:
+                    col_name = matches.pop()
+                else:
+                    raise ValueError(
+                        'Found at least two matches for component '
+                        f'"{comp_name}": "{matches}". Cannot guess coordinates '
+                        'from a table with this ambiguity.')
+
+            comp_kwargs[comp_name] = table[col_name]
 
         for k, v in comp_kwargs.items():
             if k in coord_kwargs:
