@@ -5,7 +5,6 @@ ephemerides from jplephem.
 """
 
 from urllib.parse import urlparse
-from collections import OrderedDict
 import os.path
 
 import numpy as np
@@ -30,31 +29,32 @@ __all__ = ["get_body", "get_moon", "get_body_barycentric",
 DEFAULT_JPL_EPHEMERIS = 'de430'
 
 """List of kernel pairs needed to calculate positions of a given object."""
-BODY_NAME_TO_KERNEL_SPEC = OrderedDict([
-    ('sun', [(0, 10)]),
-    ('mercury', [(0, 1), (1, 199)]),
-    ('venus', [(0, 2), (2, 299)]),
-    ('earth-moon-barycenter', [(0, 3)]),
-    ('earth', [(0, 3), (3, 399)]),
-    ('moon', [(0, 3), (3, 301)]),
-    ('mars', [(0, 4)]),
-    ('jupiter', [(0, 5)]),
-    ('saturn', [(0, 6)]),
-    ('uranus', [(0, 7)]),
-    ('neptune', [(0, 8)]),
-    ('pluto', [(0, 9)])
-])
+BODY_NAME_TO_KERNEL_SPEC = {
+    'sun': [(0, 10)],
+    'mercury': [(0, 1), (1, 199)],
+    'venus': [(0, 2), (2, 299)],
+    'earth-moon-barycenter': [(0, 3)],
+    'earth': [(0, 3), (3, 399)],
+    'moon': [(0, 3), (3, 301)],
+    'mars': [(0, 4)],
+    'jupiter': [(0, 5)],
+    'saturn': [(0, 6)],
+    'uranus': [(0, 7)],
+    'neptune': [(0, 8)],
+    'pluto': [(0, 9)],
+}
 
 """Indices to the plan94 routine for the given object."""
-PLAN94_BODY_NAME_TO_PLANET_INDEX = OrderedDict(
-    (('mercury', 1),
-     ('venus', 2),
-     ('earth-moon-barycenter', 3),
-     ('mars', 4),
-     ('jupiter', 5),
-     ('saturn', 6),
-     ('uranus', 7),
-     ('neptune', 8)))
+PLAN94_BODY_NAME_TO_PLANET_INDEX = {
+    'mercury': 1,
+    'venus': 2,
+    'earth-moon-barycenter': 3,
+    'mars': 4,
+    'jupiter': 5,
+    'saturn': 6,
+    'uranus': 7,
+    'neptune': 8,
+}
 
 _EPHEMERIS_NOTE = """
 You can either give an explicit ephemeris or use a default, which is normally
@@ -81,7 +81,7 @@ class solar_system_ephemeris(ScienceState):
     This can be one of the following::
 
     - 'builtin': polynomial approximations to the orbital elements.
-    - 'de430' or 'de432s': short-cuts for recent JPL dynamical models.
+    - 'de430', 'de432s', 'de440', 'de440s': short-cuts for recent JPL dynamical models.
     - 'jpl': Alias for the default JPL ephemeris (currently, 'de430').
     - URL: (str) The url to a SPK ephemeris in SPICE binary (.bsp) format.
     - PATH: (str) File path to a SPK ephemeris in SPICE binary (.bsp) format.
@@ -96,8 +96,9 @@ class solar_system_ephemeris(ScienceState):
     Any file required will be downloaded (and cached) when the state is set.
     The default Satellite Planet Kernel (SPK) file from NASA JPL (de430) is
     ~120MB, and covers years ~1550-2650 CE [1]_.  The smaller de432s file is
-    ~10MB, and covers years 1950-2050 [2]_.  Older versions of the JPL
-    ephemerides (such as the widely used de200) can be used via their URL [3]_.
+    ~10MB, and covers years 1950-2050 [2]_ (and similarly for the newer de440
+    and de440s).  Older versions of the JPL ephemerides (such as the widely
+    used de200) can be used via their URL [3]_.
 
     .. [1] https://naif.jpl.nasa.gov/pub/naif/generic_kernels/spk/planets/aareadme_de430-de431.txt
     .. [2] https://naif.jpl.nasa.gov/pub/naif/generic_kernels/spk/planets/aareadme_de432s.txt
@@ -162,7 +163,7 @@ def _get_kernel(value):
     if value.lower() == 'jpl':
         value = DEFAULT_JPL_EPHEMERIS
 
-    if value.lower() in ('de430', 'de432s'):
+    if value.lower() in ('de430', 'de432s', 'de440', 'de440s'):
         value = ('https://naif.jpl.nasa.gov/pub/naif/generic_kernels'
                  '/spk/planets/{:s}.bsp'.format(value.lower()))
 
@@ -307,7 +308,7 @@ def get_body_barycentric_posvel(body, time, ephemeris=None):
 
     Parameters
     ----------
-    body : str or other
+    body : str or list of tuple
         The solar system body for which to calculate positions.  Can also be a
         kernel specifier (list of 2-tuples) if the ``ephemeris`` is a JPL
         kernel.
@@ -333,19 +334,17 @@ def get_body_barycentric_posvel(body, time, ephemeris=None):
     The velocity cannot be calculated for the Moon.  To just get the position,
     use :func:`~astropy.coordinates.get_body_barycentric`.
 
+    {_EPHEMERIS_NOTE}
     """
     return _get_body_barycentric_posvel(body, time, ephemeris)
 
 
-get_body_barycentric_posvel.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
-
-
 def get_body_barycentric(body, time, ephemeris=None):
     """Calculate the barycentric position of a solar system body.
 
     Parameters
     ----------
-    body : str or other
+    body : str or list of tuple
         The solar system body for which to calculate positions.  Can also be a
         kernel specifier (list of 2-tuples) if the ``ephemeris`` is a JPL
         kernel.
@@ -366,14 +365,12 @@ def get_body_barycentric(body, time, ephemeris=None):
 
     Notes
     -----
+    {_EPHEMERIS_NOTE}
     """
     return _get_body_barycentric_posvel(body, time, ephemeris,
                                         get_velocity=False)
 
 
-get_body_barycentric.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
-
-
 def _get_apparent_body_position(body, time, ephemeris, obsgeoloc=None):
     """Calculate the apparent position of body ``body`` relative to Earth.
 
@@ -392,10 +389,15 @@ def _get_apparent_body_position(body, time, ephemeris, obsgeoloc=None):
         ``~astropy.coordinates.solar_system_ephemeris.set``
     obsgeoloc : `~astropy.coordinates.CartesianRepresentation`, optional
         The GCRS position of the observer
+
     Returns
     -------
     cartesian_position : `~astropy.coordinates.CartesianRepresentation`
         Barycentric (ICRS) apparent position of the body in cartesian coordinates
+
+    Notes
+    -----
+    {_EPHEMERIS_NOTE}
     """
     if ephemeris is None:
         ephemeris = solar_system_ephemeris.get()
@@ -423,9 +425,6 @@ def _get_apparent_body_position(body, time, ephemeris, obsgeoloc=None):
     return get_body_barycentric(body, emitted_time, ephemeris)
 
 
-_get_apparent_body_position.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
-
-
 def get_body(body, time, location=None, ephemeris=None):
     """
     Get a `~astropy.coordinates.SkyCoord` for a solar system body as observed
@@ -434,7 +433,7 @@ def get_body(body, time, location=None, ephemeris=None):
 
     Parameters
     ----------
-    body : str or other
+    body : str or list of tuple
         The solar system body for which to calculate positions.  Can also be a
         kernel specifier (list of 2-tuples) if the ``ephemeris`` is a JPL
         kernel.
@@ -458,6 +457,8 @@ def get_body(body, time, location=None, ephemeris=None):
     The coordinate returned is the apparent position, which is the position of
     the body at time *t* minus the light travel time from the *body* to the
     observing *location*.
+
+    {_EPHEMERIS_NOTE}
     """
     if location is None:
         location = time.location
@@ -476,9 +477,6 @@ def get_body(body, time, location=None, ephemeris=None):
     return SkyCoord(gcrs)
 
 
-get_body.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
-
-
 def get_moon(time, location=None, ephemeris=None):
     """
     Get a `~astropy.coordinates.SkyCoord` for the Earth's Moon as observed
@@ -504,12 +502,17 @@ def get_moon(time, location=None, ephemeris=None):
 
     Notes
     -----
+    {_EPHEMERIS_NOTE}
     """
 
     return get_body('moon', time, location=location, ephemeris=ephemeris)
 
 
-get_moon.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
+# Add note about the ephemeris choices to the docstrings of relevant functions.
+# Note: sadly, one cannot use f-strings for docstrings, so we format explicitly.
+for f in [f for f in locals().values() if callable(f) and f.__doc__ is not None
+          and '{_EPHEMERIS_NOTE}' in f.__doc__]:
+    f.__doc__ = f.__doc__.format(_EPHEMERIS_NOTE=indent(_EPHEMERIS_NOTE)[4:])
 
 
 deprecation_msg = """
