@@ -162,7 +162,7 @@ class SpectralCoord(SpectralQuantity):
     value : ndarray or `~astropy.units.Quantity` or `SpectralCoord`
         Spectral values, which should be either wavelength, frequency,
         energy, wavenumber, or velocity values.
-    unit : str or `~astropy.units.Unit`
+    unit : unit-like
         Unit for the given spectral values.
     observer : `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`, optional
         The coordinate (position and velocity) of observer. If no velocities
@@ -172,7 +172,7 @@ class SpectralCoord(SpectralQuantity):
         The coordinate (position and velocity) of target. If no velocities
         are present on this object, the target is assumed to be stationary
         relative to the frame origin.
-    radial_velocity : `~astropy.units.Quantity`, optional
+    radial_velocity : `~astropy.units.Quantity` ['speed'], optional
         The radial velocity of the target with respect to the observer. This
         can only be specified if ``redshift`` is not specified.
     redshift : float, optional
@@ -290,8 +290,7 @@ class SpectralCoord(SpectralQuantity):
         # system.
         if 's' not in coord.data.differentials:
             warnings.warn(
-                "No velocity defined on frame, assuming {}.".format(
-                    ZERO_VELOCITIES),
+                f"No velocity defined on frame, assuming {ZERO_VELOCITIES}.",
                 NoVelocityWarning)
 
             coord = attach_zero_velocities(coord)
@@ -320,13 +319,13 @@ class SpectralCoord(SpectralQuantity):
         value : ndarray or `~astropy.units.Quantity` or `SpectralCoord`, optional
             Spectral values, which should be either wavelength, frequency,
             energy, wavenumber, or velocity values.
-        unit : str or `~astropy.units.Unit`
+        unit : unit-like
             Unit for the given spectral values.
         observer : `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`, optional
             The coordinate (position and velocity) of observer.
         target : `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`, optional
             The coordinate (position and velocity) of target.
-        radial_velocity : `~astropy.units.Quantity`, optional
+        radial_velocity : `~astropy.units.Quantity` ['speed'], optional
             The radial velocity of the target with respect to the observer.
         redshift : float, optional
             The relativistic redshift of the target with respect to the observer.
@@ -448,7 +447,7 @@ class SpectralCoord(SpectralQuantity):
 
         Returns
         -------
-        `~astropy.units.Quantity`
+        `~astropy.units.Quantity` ['speed']
             Radial velocity of target.
 
         Notes
@@ -496,7 +495,7 @@ class SpectralCoord(SpectralQuantity):
 
         Returns
         -------
-        `~astropy.units.Quantity`
+        `~astropy.units.Quantity` ['speed']
             The radial velocity of the target with respect to the observer.
         """
 
@@ -641,9 +640,9 @@ class SpectralCoord(SpectralQuantity):
 
         Parameters
         ----------
-        target_shift : float or `~astropy.units.Quantity`
+        target_shift : float or `~astropy.units.Quantity` ['speed']
             Shift value to apply to current target.
-        observer_shift : float or `~astropy.units.Quantity`
+        observer_shift : float or `~astropy.units.Quantity` ['speed']
             Shift value to apply to current observer.
 
         Returns
