@@ -21,13 +21,7 @@ from astropy import units as u
 from astropy import time
 from astropy import coordinates as coords
 from astropy.units import allclose
-
-try:
-    import scipy  # pylint: disable=W0611  # noqa
-except ImportError:
-    HAS_SCIPY = False
-else:
-    HAS_SCIPY = True
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 
 def test_representations_api():
