@@ -2,6 +2,12 @@
 """Accuracy tests for GCRS coordinate transformations, primarily to/from AltAz.
 
 """
+import os
+try:
+    from importlib import metadata
+except ImportError:
+    import importlib_metadata as metadata
+
 
 import pytest
 import numpy as np
@@ -15,21 +21,19 @@ from astropy.coordinates import (
     PrecessedGeocentric, CartesianRepresentation, SkyCoord,
     CartesianDifferential, SphericalRepresentation, UnitSphericalRepresentation,
     HCRS, HeliocentricMeanEcliptic, TEME, TETE)
-from astropy.coordinates.solar_system import _apparent_position_in_true_coordinates
+from astropy.coordinates.solar_system import _apparent_position_in_true_coordinates, get_body
 from astropy.utils import iers
 from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
+from astropy.utils.compat.optional_deps import HAS_JPLEPHEM
 
-from .utils import randomly_sample_sphere
+from astropy.coordinates.angle_utilities import golden_spiral_grid
+from astropy.coordinates.builtin_frames.intermediate_rotation_transforms import (
+    get_location_gcrs, tete_to_itrs_mat, gcrs_to_cirs_mat, cirs_to_itrs_mat)
 from astropy.coordinates.builtin_frames.utils import get_jd12
 from astropy.coordinates import solar_system_ephemeris
 from astropy.units import allclose
 
-try:
-    import jplephem  # pylint: disable=W0611  # noqa
-except ImportError:
-    HAS_JPLEPHEM = False
-else:
-    HAS_JPLEPHEM = True
+CI = os.environ.get('CI', False) == "true"
 
 
 def test_icrs_cirs():
@@ -39,9 +43,10 @@ def test_icrs_cirs():
     Also includes the CIRS<->CIRS transforms at different times, as those go
     through ICRS
     """
-    ra, dec, dist = randomly_sample_sphere(200)
-    inod = ICRS(ra=ra, dec=dec)
-    iwd = ICRS(ra=ra, dec=dec, distance=dist*u.pc)
+    usph = golden_spiral_grid(200)
+    dist = np.linspace(0., 1, len(usph)) * u.pc
+    inod = ICRS(usph)
+    iwd = ICRS(ra=usph.lon, dec=usph.lat, distance=dist)
 
     cframe1 = CIRS()
     cirsnod = inod.transform_to(cframe1)  # uses the default time
@@ -77,8 +82,9 @@ def test_icrs_cirs():
     assert_allclose(cirsnod.dec, cirsnod5.dec)
 
 
-ra, dec, dist = randomly_sample_sphere(200)
-icrs_coords = [ICRS(ra=ra, dec=dec), ICRS(ra=ra, dec=dec, distance=dist*u.pc)]
+usph = golden_spiral_grid(200)
+dist = np.linspace(0.5, 1, len(usph)) * u.pc
+icrs_coords = [ICRS(usph), ICRS(usph.lon, usph.lat, distance=dist)]
 gcrs_frames = [GCRS(), GCRS(obstime=Time('J2005'))]
 
 
@@ -147,9 +153,10 @@ def test_cirs_to_altaz():
     """
     from astropy.coordinates import EarthLocation
 
-    ra, dec, dist = randomly_sample_sphere(200)
-    cirs = CIRS(ra=ra, dec=dec, obstime='J2000')
-    crepr = SphericalRepresentation(lon=ra, lat=dec, distance=dist)
+    usph = golden_spiral_grid(200)
+    dist = np.linspace(0.5, 1, len(usph)) * u.pc
+    cirs = CIRS(usph, obstime='J2000')
+    crepr = SphericalRepresentation(lon=usph.lon, lat=usph.lat, distance=dist)
     cirscart = CIRS(crepr, obstime=cirs.obstime, representation_type=CartesianRepresentation)
 
     loc = EarthLocation(lat=0*u.deg, lon=0*u.deg, height=0*u.m)
@@ -169,23 +176,24 @@ def test_gcrs_itrs():
     """
     Check basic GCRS<->ITRS transforms for round-tripping.
     """
-    ra, dec, _ = randomly_sample_sphere(200)
-    gcrs = GCRS(ra=ra, dec=dec, obstime='J2000')
-    gcrs6 = GCRS(ra=ra, dec=dec, obstime='J2006')
+    usph = golden_spiral_grid(200)
+    gcrs = GCRS(usph, obstime='J2000')
+    gcrs6 = GCRS(usph, obstime='J2006')
 
     gcrs2 = gcrs.transform_to(ITRS()).transform_to(gcrs)
     gcrs6_2 = gcrs6.transform_to(ITRS()).transform_to(gcrs)
 
     assert_allclose(gcrs.ra, gcrs2.ra)
     assert_allclose(gcrs.dec, gcrs2.dec)
-    assert not allclose(gcrs.ra, gcrs6_2.ra)
-    assert not allclose(gcrs.dec, gcrs6_2.dec)
+    # these should be different:
+    assert not allclose(gcrs.ra, gcrs6_2.ra, rtol=1e-8)
+    assert not allclose(gcrs.dec, gcrs6_2.dec, rtol=1e-8)
 
     # also try with the cartesian representation
     gcrsc = gcrs.realize_frame(gcrs.data)
     gcrsc.representation_type = CartesianRepresentation
     gcrsc2 = gcrsc.transform_to(ITRS()).transform_to(gcrsc)
-    assert_allclose(gcrsc.spherical.lon.deg, gcrsc2.ra.deg)
+    assert_allclose(gcrsc.spherical.lon, gcrsc2.ra)
     assert_allclose(gcrsc.spherical.lat, gcrsc2.dec)
 
 
@@ -193,9 +201,9 @@ def test_cirs_itrs():
     """
     Check basic CIRS<->ITRS transforms for round-tripping.
     """
-    ra, dec, _ = randomly_sample_sphere(200)
-    cirs = CIRS(ra=ra, dec=dec, obstime='J2000')
-    cirs6 = CIRS(ra=ra, dec=dec, obstime='J2006')
+    usph = golden_spiral_grid(200)
+    cirs = CIRS(usph, obstime='J2000')
+    cirs6 = CIRS(usph, obstime='J2006')
 
     cirs2 = cirs.transform_to(ITRS()).transform_to(cirs)
     cirs6_2 = cirs6.transform_to(ITRS()).transform_to(cirs)  # different obstime
@@ -212,17 +220,18 @@ def test_gcrs_cirs():
     Check GCRS<->CIRS transforms for round-tripping.  More complicated than the
     above two because it's multi-hop
     """
-    ra, dec, _ = randomly_sample_sphere(200)
-    gcrs = GCRS(ra=ra, dec=dec, obstime='J2000')
-    gcrs6 = GCRS(ra=ra, dec=dec, obstime='J2006')
+    usph = golden_spiral_grid(200)
+    gcrs = GCRS(usph, obstime='J2000')
+    gcrs6 = GCRS(usph, obstime='J2006')
 
     gcrs2 = gcrs.transform_to(CIRS()).transform_to(gcrs)
     gcrs6_2 = gcrs6.transform_to(CIRS()).transform_to(gcrs)
 
     assert_allclose(gcrs.ra, gcrs2.ra)
     assert_allclose(gcrs.dec, gcrs2.dec)
-    assert not allclose(gcrs.ra, gcrs6_2.ra)
-    assert not allclose(gcrs.dec, gcrs6_2.dec)
+    # these should be different:
+    assert not allclose(gcrs.ra, gcrs6_2.ra, rtol=1e-8)
+    assert not allclose(gcrs.dec, gcrs6_2.dec, rtol=1e-8)
 
     # now try explicit intermediate pathways and ensure they're all consistent
     gcrs3 = gcrs.transform_to(ITRS()).transform_to(CIRS()).transform_to(ITRS()).transform_to(gcrs)
@@ -240,12 +249,12 @@ def test_gcrs_altaz():
     """
     from astropy.coordinates import EarthLocation
 
-    ra, dec, _ = randomly_sample_sphere(1)
-    gcrs = GCRS(ra=ra[0], dec=dec[0], obstime='J2000')
+    usph = golden_spiral_grid(128)
+    gcrs = GCRS(usph, obstime='J2000')[None]  # broadcast with times below
 
     # check array times sure N-d arrays work
     times = Time(np.linspace(2456293.25, 2456657.25, 51) * u.day,
-                 format='jd')
+                 format='jd')[:, None]
 
     loc = EarthLocation(lon=10 * u.deg, lat=80. * u.deg)
     aaframe = AltAz(obstime=times, location=loc)
@@ -532,7 +541,19 @@ def test_earth_orientation_table(monkeypatch):
     # Default: uses IERS_Auto, which will give a prediction.
     # Note: tests run with warnings turned into errors, so it is
     # meaningful if this passes.
-    altaz_auto = sc.transform_to(altaz)
+    with pytest.warns(None) as warning_lines:
+        altaz_auto = sc.transform_to(altaz)
+
+    # Server occasionally blocks IERS download in CI.
+    n_warnings = len(warning_lines)
+    if CI:
+        assert n_warnings <= 1, f'Expected at most one warning but got {n_warnings}'
+        if n_warnings == 1:
+            w_msg = str(warning_lines[0].message)
+            # This also captures unclosed socket warning that is ignored in setup.cfg
+            assert 'using local IERS-B' in w_msg or 'unclosed' in w_msg, f'Got unexpected warning: {w_msg}'
+    else:
+        assert n_warnings == 0, f'Expected no warning but got {n_warnings}'
 
     with iers.earth_orientation_table.set(iers.IERS_B.open()):
         with pytest.warns(AstropyWarning, match='after IERS data'):
@@ -548,7 +569,7 @@ def test_earth_orientation_table(monkeypatch):
 
 
 @pytest.mark.remote_data
-@pytest.mark.skipif('not HAS_JPLEPHEM')
+@pytest.mark.skipif(not HAS_JPLEPHEM, reason='requires jplephem')
 def test_ephemerides():
     """
     We test that using different ephemerides gives very similar results
@@ -616,10 +637,7 @@ def test_tete_transforms():
     # test TETE-ITRS transform by comparing GCRS-CIRS-ITRS to GCRS-TETE-ITRS
     itrs1 = moon.transform_to(CIRS()).transform_to(ITRS())
     itrs2 = moon.transform_to(TETE()).transform_to(ITRS())
-    # this won't be as close since it will round trip through ICRS until
-    # we have some way of translating between the GCRS obsgeoloc/obsgeovel
-    # attributes and the TETE location attributes.
-    assert_allclose(itrs1.separation_3d(itrs2), 0*u.mm, atol=100*u.mm)
+    assert_allclose(itrs1.separation_3d(itrs2), 0*u.mm, atol=1*u.mm)
 
     # test round trip GCRS->TETE->GCRS
     new_moon = moon.transform_to(TETE()).transform_to(moon)
@@ -634,3 +652,148 @@ def test_tete_transforms():
     with pytest.warns(AstropyDeprecationWarning, match='The use of'):
         tete_alt = _apparent_position_in_true_coordinates(moon)
     assert_allclose(tete_coo1.separation_3d(tete_alt), 0*u.mm, atol=100*u.mm)
+
+
+def test_straight_overhead():
+    """
+    With a precise CIRS<->AltAz transformation this should give Alt=90 exactly
+
+    If the CIRS self-transform breaks it won't, due to improper treatment of aberration
+    """
+    t = Time('J2010')
+    obj = EarthLocation(-1*u.deg, 52*u.deg, height=10.*u.km)
+    home = EarthLocation(-1*u.deg, 52*u.deg, height=0.*u.km)
+
+    # An object that appears straight overhead - FOR A GEOCENTRIC OBSERVER.
+    # Note, this won't be overhead for a topocentric observer because of
+    # aberration.
+    cirs_geo = obj.get_itrs(t).transform_to(CIRS(obstime=t))
+
+    # now get the Geocentric CIRS position of observatory
+    obsrepr = home.get_itrs(t).transform_to(CIRS(obstime=t)).cartesian
+
+    # topocentric CIRS position of a straight overhead object
+    cirs_repr = cirs_geo.cartesian - obsrepr
+
+    # create a CIRS object that appears straight overhead for a TOPOCENTRIC OBSERVER
+    topocentric_cirs_frame = CIRS(obstime=t, location=home)
+    cirs_topo = topocentric_cirs_frame.realize_frame(cirs_repr)
+
+    aa = cirs_topo.transform_to(AltAz(obstime=t, location=home))
+    assert_allclose(aa.alt, 90*u.deg)
+
+
+def jplephem_ge(minversion):
+    """Check if jplephem is installed and has version >= minversion."""
+    # This is a separate routine since somehow with pyinstaller the stanza
+    # not HAS_JPLEPHEM or metadata.version('jplephem') < '2.15'
+    # leads to a module not found error.
+    try:
+        return HAS_JPLEPHEM and metadata.version('jplephem') >= minversion
+    except Exception:
+        return False
+
+
+@pytest.mark.remote_data
+@pytest.mark.skipif(not jplephem_ge('2.15'), reason='requires jplephem >= 2.15')
+def test_aa_high_precision():
+    """These tests are provided by @mkbrewer - see issue #10356.
+
+    The code that produces them agrees very well (<0.5 mas) with SkyField once Polar motion
+    is turned off, but SkyField does not include polar motion, so a comparison to Skyfield
+    or JPL Horizons will be ~1" off.
+
+    The absence of polar motion within Skyfield and the disagreement between Skyfield and Horizons
+    make high precision comparisons to those codes difficult.
+
+    Updated 2020-11-29, after the comparison between codes became even better,
+    down to 100 nas.
+
+    NOTE: the agreement reflects consistency in approach between two codes,
+    not necessarily absolute precision.  If this test starts failing, the
+    tolerance can and shouls be weakened *if* it is clear that the change is
+    due to an improvement (e.g., a new IAU precession model).
+
+    """
+    lat = -22.959748*u.deg
+    lon = -67.787260*u.deg
+    elev = 5186*u.m
+    loc = EarthLocation.from_geodetic(lon, lat, elev)
+    # Note: at this level of precision for the comparison, we have to include
+    # the location in the time, as it influences the transformation to TDB.
+    t = Time('2017-04-06T00:00:00.0', location=loc)
+    with solar_system_ephemeris.set('de430'):
+        moon = get_body('moon', t, loc)
+        moon_aa = moon.transform_to(AltAz(obstime=t, location=loc))
+
+    # Numbers from
+    # https://github.com/astropy/astropy/pull/11073#issuecomment-735486271
+    # updated in https://github.com/astropy/astropy/issues/11683
+    TARGET_AZ, TARGET_EL = 15.032673509956*u.deg, 50.303110133923*u.deg
+    TARGET_DISTANCE = 376252883.247239*u.m
+    assert_allclose(moon_aa.az, TARGET_AZ, atol=0.1*u.uas, rtol=0)
+    assert_allclose(moon_aa.alt, TARGET_EL, atol=0.1*u.uas, rtol=0)
+    assert_allclose(moon_aa.distance, TARGET_DISTANCE, atol=0.1*u.mm, rtol=0)
+
+
+def test_aa_high_precision_nodata():
+    """
+    These tests are designed to ensure high precision alt-az transforms.
+
+    They are a slight fudge since the target values come from astropy itself. They are generated
+    with a version of the code that passes the tests above, but for the internal solar system
+    ephemerides to avoid the use of remote data.
+    """
+    TARGET_AZ, TARGET_EL = 15.0321908*u.deg, 50.30263625*u.deg
+    lat = -22.959748*u.deg
+    lon = -67.787260*u.deg
+    elev = 5186*u.m
+    loc = EarthLocation.from_geodetic(lon, lat, elev)
+    t = Time('2017-04-06T00:00:00.0')
+
+    moon = get_body('moon', t, loc)
+    moon_aa = moon.transform_to(AltAz(obstime=t, location=loc))
+    assert_allclose(moon_aa.az - TARGET_AZ, 0*u.mas, atol=0.5*u.mas)
+    assert_allclose(moon_aa.alt - TARGET_EL, 0*u.mas, atol=0.5*u.mas)
+
+
+class TestGetLocationGCRS:
+    # TETE and CIRS use get_location_gcrs to get obsgeoloc and obsgeovel
+    # with knowledge of some of the matrices. Check that this is consistent
+    # with a direct transformation.
+    def setup_class(cls):
+        cls.loc = loc = EarthLocation.from_geodetic(
+            np.linspace(0, 360, 6)*u.deg, np.linspace(-90, 90, 6)*u.deg, 100*u.m)
+        cls.obstime = obstime = Time(np.linspace(2000, 2010, 6), format='jyear')
+        # Get comparison via a full transformation.  We do not use any methods
+        # of EarthLocation, since those depend on the fast transform.
+        loc_itrs = ITRS(loc.x, loc.y, loc.z, obstime=obstime)
+        zeros = np.broadcast_to(0. * (u.km / u.s), (3,) + loc_itrs.shape, subok=True)
+        loc_itrs.data.differentials['s'] = CartesianDifferential(zeros)
+        loc_gcrs_cart = loc_itrs.transform_to(GCRS(obstime=obstime)).cartesian
+        cls.obsgeoloc = loc_gcrs_cart.without_differentials()
+        cls.obsgeovel = loc_gcrs_cart.differentials['s'].to_cartesian()
+
+    def check_obsgeo(self, obsgeoloc, obsgeovel):
+        assert_allclose(obsgeoloc.xyz, self.obsgeoloc.xyz, atol=.1*u.um, rtol=0.)
+        assert_allclose(obsgeovel.xyz, self.obsgeovel.xyz, atol=.1*u.mm/u.s, rtol=0.)
+
+    def test_get_gcrs_posvel(self):
+        # Really just a sanity check
+        self.check_obsgeo(*self.loc.get_gcrs_posvel(self.obstime))
+
+    def test_tete_quick(self):
+        # Following copied from intermediate_rotation_transforms.gcrs_to_tete
+        rbpn = erfa.pnm06a(*get_jd12(self.obstime, 'tt'))
+        loc_gcrs_frame = get_location_gcrs(self.loc, self.obstime,
+                                           tete_to_itrs_mat(self.obstime, rbpn=rbpn),
+                                           rbpn)
+        self.check_obsgeo(loc_gcrs_frame.obsgeoloc, loc_gcrs_frame.obsgeovel)
+
+    def test_cirs_quick(self):
+        cirs_frame = CIRS(location=self.loc, obstime=self.obstime)
+        # Following copied from intermediate_rotation_transforms.gcrs_to_cirs
+        pmat = gcrs_to_cirs_mat(cirs_frame.obstime)
+        loc_gcrs_frame = get_location_gcrs(self.loc, self.obstime,
+                                           cirs_to_itrs_mat(cirs_frame.obstime), pmat)
+        self.check_obsgeo(loc_gcrs_frame.obsgeoloc, loc_gcrs_frame.obsgeovel)
