@@ -2,7 +2,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 from copy import deepcopy
-from collections import OrderedDict
 
 import pytest
 import numpy as np
@@ -15,24 +14,15 @@ from astropy.utils import isiterable
 from astropy.utils.exceptions import DuplicateRepresentationWarning
 from astropy.coordinates.angles import Longitude, Latitude, Angle
 from astropy.coordinates.distances import Distance
-from astropy.coordinates.representation import (REPRESENTATION_CLASSES,
-                                                DIFFERENTIAL_CLASSES,
-                                                DUPLICATE_REPRESENTATIONS,
-                                                BaseRepresentation,
-                                                SphericalRepresentation,
-                                                UnitSphericalRepresentation,
-                                                SphericalCosLatDifferential,
-                                                CartesianRepresentation,
-                                                CylindricalRepresentation,
-                                                PhysicsSphericalRepresentation,
-                                                CartesianDifferential,
-                                                SphericalDifferential,
-                                                RadialDifferential,
-                                                CylindricalDifferential,
-                                                PhysicsSphericalDifferential,
-                                                UnitSphericalDifferential,
-                                                UnitSphericalCosLatDifferential,
-                                                _combine_xyz)
+from astropy.coordinates.matrix_utilities import rotation_matrix
+from astropy.coordinates.representation import (
+    REPRESENTATION_CLASSES, DIFFERENTIAL_CLASSES, DUPLICATE_REPRESENTATIONS,
+    BaseRepresentation, SphericalRepresentation, UnitSphericalRepresentation,
+    SphericalCosLatDifferential, CartesianRepresentation, RadialDifferential,
+    CylindricalRepresentation, PhysicsSphericalRepresentation,
+    CartesianDifferential, SphericalDifferential, CylindricalDifferential,
+    PhysicsSphericalDifferential, UnitSphericalDifferential,
+    UnitSphericalCosLatDifferential)
 
 
 # Preserve the original REPRESENTATION_CLASSES dict so that importing
@@ -58,6 +48,15 @@ def components_equal(rep1, rep2):
     return result
 
 
+def components_allclose(rep1, rep2):
+    result = True
+    if type(rep1) is not type(rep2):
+        return False
+    for component in rep1.components:
+        result &= u.allclose(getattr(rep1, component), getattr(rep2, component))
+    return result
+
+
 def representation_equal(rep1, rep2):
     result = True
     if type(rep1) is not type(rep2):
@@ -73,6 +72,21 @@ def representation_equal(rep1, rep2):
     return result & components_equal(rep1, rep2)
 
 
+def representation_equal_up_to_angular_type(rep1, rep2):
+    result = True
+    if type(rep1) is not type(rep2):
+        return False
+    if getattr(rep1, '_differentials', False):
+        if rep1._differentials.keys() != rep2._differentials.keys():
+            return False
+        for key, diff1 in rep1._differentials.items():
+            result &= components_allclose(diff1, rep2._differentials[key])
+    elif getattr(rep2, '_differentials', False):
+        return False
+
+    return result & components_allclose(rep1, rep2)
+
+
 class TestSphericalRepresentation:
 
     def test_name(self):
@@ -116,6 +130,17 @@ class TestSphericalRepresentation:
         assert s3.lon == -90. * u.degree
         assert s3.lon.wrap_angle == 180 * u.degree
 
+    def test_init_subclass(self):
+        class Longitude180(Longitude):
+            _default_wrap_angle = 180*u.degree
+
+        s = SphericalRepresentation(Longitude180(-90, u.degree),
+                                    Latitude(-45, u.degree),
+                                    Distance(1., u.Rsun))
+        assert isinstance(s.lon, Longitude180)
+        assert s.lon == -90. * u.degree
+        assert s.lon.wrap_angle == 180 * u.degree
+
     def test_init_array(self):
 
         s1 = SphericalRepresentation(lon=[8, 9] * u.hourangle,
@@ -240,6 +265,16 @@ class TestSphericalRepresentation:
         assert_allclose_quantity(s.lat, [2, 2, -2, -3, -4] * u.deg)
         assert_allclose_quantity(s.distance, [5, 5, 1, 1, 1] * u.kpc)
 
+    def test_negative_distance(self):
+        """Only allowed if explicitly passed on."""
+        with pytest.raises(ValueError, match='allow_negative'):
+            SphericalRepresentation(10*u.deg, 20*u.deg, -10*u.m)
+
+        s1 = SphericalRepresentation(10*u.deg, 20*u.deg,
+                                     Distance(-10*u.m, allow_negative=True))
+
+        assert s1.distance == -10.*u.m
+
     def test_nan_distance(self):
         """ This is a regression test: calling represent_as() and passing in the
             same class as the object shouldn't round-trip through cartesian.
@@ -264,6 +299,86 @@ class TestSphericalRepresentation:
         with pytest.raises(TypeError, match='unexpected keyword.*parrot'):
             SphericalRepresentation(1*u.deg, 2*u.deg, 1.*u.kpc, parrot=10)
 
+    def test_representation_shortcuts(self):
+        """Test that shortcuts in ``represent_as`` don't fail."""
+        difs = SphericalCosLatDifferential(4*u.mas/u.yr,5*u.mas/u.yr,6*u.km/u.s)
+        sph = SphericalRepresentation(1*u.deg, 2*u.deg, 3*u.kpc,
+                                      differentials={'s': difs})
+
+        got = sph.represent_as(PhysicsSphericalRepresentation,
+                               PhysicsSphericalDifferential)
+        assert np.may_share_memory(sph.lon, got.phi)
+        assert np.may_share_memory(sph.distance, got.r)
+        expected = BaseRepresentation.represent_as(
+            sph, PhysicsSphericalRepresentation, PhysicsSphericalDifferential)
+        # equal up to angular type
+        assert representation_equal_up_to_angular_type(got, expected)
+
+        got = sph.represent_as(UnitSphericalRepresentation,
+                               UnitSphericalDifferential)
+        assert np.may_share_memory(sph.lon, got.lon)
+        assert np.may_share_memory(sph.lat, got.lat)
+        expected = BaseRepresentation.represent_as(
+            sph, UnitSphericalRepresentation, UnitSphericalDifferential)
+        assert representation_equal_up_to_angular_type(got, expected)
+
+    def test_transform(self):
+
+        s1 = SphericalRepresentation(lon=[1, 2] * u.deg, lat=[3, 4] * u.deg,
+                                     distance=[5, 6] * u.kpc)
+
+        matrix = rotation_matrix(-10, "z", u.deg)
+
+        s2 = s1.transform(matrix)
+
+        assert_allclose_quantity(s2.lon, s1.lon + 10 * u.deg)
+        assert_allclose_quantity(s2.lat, s1.lat)
+        assert_allclose_quantity(s2.distance, s1.distance)
+
+        # now with a non rotation matrix
+        matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
+        s3 = s1.transform(matrix)
+        expected = (s1.to_cartesian().transform(matrix)
+                      .represent_as(SphericalRepresentation))
+
+        assert_allclose_quantity(s3.lon, expected.lon)
+        assert_allclose_quantity(s3.lat, expected.lat)
+        assert_allclose_quantity(s3.distance, expected.distance)
+
+    def test_transform_with_NaN(self):
+        # all over again, but with a NaN in the distance
+
+        s1 = SphericalRepresentation(lon=[1, 2] * u.deg, lat=[3, 4] * u.deg,
+                                     distance=[5, np.nan] * u.kpc)
+
+        matrix = rotation_matrix(-10, "z", u.deg)
+
+        s2 = s1.transform(matrix)
+
+        assert_allclose_quantity(s2.lon, s1.lon + 10 * u.deg)
+        assert_allclose_quantity(s2.lat, s1.lat)
+        assert_allclose_quantity(s2.distance, s1.distance)
+
+        # now with a non rotation matrix
+        matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
+        s3 = s1.transform(matrix)
+        # s3 should not propagate Nan.
+        assert_array_equal(np.isnan(s3.lon.deg), (False, False))
+        assert_array_equal(np.isnan(s3.lat.deg), (False, False))
+        assert_array_equal(np.isnan(s3.distance.value), (False, True))
+
+        # through Cartesian should
+        thruC = (s1.to_cartesian().transform(matrix)
+                   .represent_as(SphericalRepresentation))
+        assert_array_equal(np.isnan(thruC.lon.deg), (False, True))
+        assert_array_equal(np.isnan(thruC.lat.deg), (False, True))
+        assert_array_equal(np.isnan(thruC.distance.value), (False, True))
+        # test that they are close on the first value
+        assert_allclose_quantity(s3.lon[0], thruC.lon[0])
+        assert_allclose_quantity(s3.lat[0], thruC.lat[0])
+
 
 class TestUnitSphericalRepresentation:
 
@@ -376,6 +491,54 @@ class TestUnitSphericalRepresentation:
         with pytest.raises(TypeError):
             s_slc = s[0]
 
+    def test_representation_shortcuts(self):
+        """Test that shortcuts in ``represent_as`` don't fail."""
+        # TODO! representation transformations with differentials cannot
+        # (currently) be implemented due to a mismatch between the UnitSpherical
+        # expected keys (e.g. "s") and that expected in the other class
+        # (here "s / m"). For more info, see PR #11467
+        # We leave the test code commented out for future use.
+        # diffs = UnitSphericalCosLatDifferential(4*u.mas/u.yr, 5*u.mas/u.yr,
+        #                                         6*u.km/u.s)
+        sph = UnitSphericalRepresentation(1*u.deg, 2*u.deg)
+                                          # , differentials={'s': diffs}
+        got = sph.represent_as(PhysicsSphericalRepresentation)
+                               # , PhysicsSphericalDifferential)
+        assert np.may_share_memory(sph.lon, got.phi)
+        expected = BaseRepresentation.represent_as(
+            sph, PhysicsSphericalRepresentation)  # PhysicsSphericalDifferential
+        assert representation_equal_up_to_angular_type(got, expected)
+
+        got = sph.represent_as(SphericalRepresentation)
+                               # , SphericalDifferential)
+        assert np.may_share_memory(sph.lon, got.lon)
+        assert np.may_share_memory(sph.lat, got.lat)
+        expected = BaseRepresentation.represent_as(
+            sph, SphericalRepresentation) # , SphericalDifferential)
+        assert representation_equal_up_to_angular_type(got, expected)
+
+    def test_transform(self):
+
+        s1 = UnitSphericalRepresentation(lon=[1, 2] * u.deg, lat=[3, 4] * u.deg)
+
+        matrix = rotation_matrix(-10, "z", u.deg)
+
+        s2 = s1.transform(matrix)
+
+        assert_allclose_quantity(s2.lon, s1.lon + 10 * u.deg)
+        assert_allclose_quantity(s2.lat, s1.lat)
+
+        # now with a non rotation matrix
+        # note that the result will be a Spherical, not UnitSpherical
+        matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
+        s3 = s1.transform(matrix)
+        expected = s1.to_cartesian().transform(matrix).represent_as(SphericalRepresentation)
+
+        assert_allclose_quantity(s3.lon, expected.lon)
+        assert_allclose_quantity(s3.lat, expected.lat)
+        assert_allclose_quantity(s3.distance, expected.distance)
+
 
 class TestPhysicsSphericalRepresentation:
 
@@ -510,6 +673,92 @@ class TestPhysicsSphericalRepresentation:
         with pytest.raises(TypeError):
             s_slc = s[0]
 
+    def test_representation_shortcuts(self):
+        """Test that shortcuts in ``represent_as`` don't fail."""
+        difs = PhysicsSphericalDifferential(4*u.mas/u.yr,5*u.mas/u.yr,6*u.km/u.s)
+        sph = PhysicsSphericalRepresentation(1*u.deg, 2*u.deg, 3*u.kpc,
+                                             differentials={'s': difs})
+
+        got = sph.represent_as(SphericalRepresentation,
+                               SphericalDifferential)
+        assert np.may_share_memory(sph.phi, got.lon)
+        assert np.may_share_memory(sph.r, got.distance)
+        expected = BaseRepresentation.represent_as(
+            sph, SphericalRepresentation, SphericalDifferential)
+        assert representation_equal_up_to_angular_type(got, expected)
+
+        got = sph.represent_as(UnitSphericalRepresentation,
+                               UnitSphericalDifferential)
+        assert np.may_share_memory(sph.phi, got.lon)
+        expected = BaseRepresentation.represent_as(
+            sph, UnitSphericalRepresentation, UnitSphericalDifferential)
+        assert representation_equal_up_to_angular_type(got, expected)
+
+    def test_initialize_with_nan(self):
+        # Regression test for gh-11558: initialization used to fail.
+        psr = PhysicsSphericalRepresentation([1., np.nan]*u.deg, [np.nan, 2.]*u.deg,
+                                             [3., np.nan]*u.m)
+        assert_array_equal(np.isnan(psr.phi), [False, True])
+        assert_array_equal(np.isnan(psr.theta), [True, False])
+        assert_array_equal(np.isnan(psr.r), [False, True])
+
+    def test_transform(self):
+
+        s1 = PhysicsSphericalRepresentation(
+            phi=[1, 2] * u.deg, theta=[3, 4] * u.deg, r=[5, 6] * u.kpc)
+
+        matrix = rotation_matrix(-10, "z", u.deg)
+
+        s2 = s1.transform(matrix)
+
+        assert_allclose_quantity(s2.phi, s1.phi + 10 * u.deg)
+        assert_allclose_quantity(s2.theta, s1.theta)
+        assert_allclose_quantity(s2.r, s1.r)
+
+        # now with a non rotation matrix
+        matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
+        s3 = s1.transform(matrix)
+        expected = (s1.to_cartesian().transform(matrix)
+                      .represent_as(PhysicsSphericalRepresentation))
+
+        assert_allclose_quantity(s3.phi, expected.phi)
+        assert_allclose_quantity(s3.theta, expected.theta)
+        assert_allclose_quantity(s3.r, expected.r)
+
+    def test_transform_with_NaN(self):
+        # all over again, but with a NaN in the distance
+
+        s1 = PhysicsSphericalRepresentation(
+            phi=[1, 2] * u.deg, theta=[3, 4] * u.deg, r=[5, np.nan] * u.kpc)
+
+        matrix = rotation_matrix(-10, "z", u.deg)
+
+        s2 = s1.transform(matrix)
+
+        assert_allclose_quantity(s2.phi, s1.phi + 10 * u.deg)
+        assert_allclose_quantity(s2.theta, s1.theta)
+        assert_allclose_quantity(s2.r, s1.r)
+
+        # now with a non rotation matrix
+        matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
+        s3 = s1.transform(matrix)
+        # s3 should not propagate Nan.
+        assert_array_equal(np.isnan(s3.phi.deg), (False, False))
+        assert_array_equal(np.isnan(s3.theta.deg), (False, False))
+        assert_array_equal(np.isnan(s3.r.value), (False, True))
+
+        # through Cartesian does
+        thruC = (s1.to_cartesian().transform(matrix)
+                 .represent_as(PhysicsSphericalRepresentation))
+        assert_array_equal(np.isnan(thruC.phi.deg), (False, True))
+        assert_array_equal(np.isnan(thruC.theta.deg), (False, True))
+        assert_array_equal(np.isnan(thruC.r.value), (False, True))
+        # so only test on the first value
+        assert_allclose_quantity(s3.phi[0], thruC.phi[0])
+        assert_allclose_quantity(s3.theta[0], thruC.theta[0])
+
 
 class TestCartesianRepresentation:
 
@@ -896,6 +1145,33 @@ class TestCylindricalRepresentation:
         with pytest.raises(TypeError):
             s_slc = s[0]
 
+    def test_transform(self):
+
+        s1 = CylindricalRepresentation(phi=[1, 2] * u.deg, z=[3, 4] * u.pc,
+                                       rho=[5, 6] * u.kpc)
+
+        matrix = rotation_matrix(-10, "z", u.deg)
+
+        s2 = s1.transform(matrix)
+
+        assert_allclose_quantity(s2.phi, s1.phi + 10 * u.deg)
+        assert_allclose_quantity(s2.z, s1.z)
+        assert_allclose_quantity(s2.rho, s1.rho)
+
+        assert s2.phi.unit is u.rad
+        assert s2.z.unit is u.kpc
+        assert s2.rho.unit is u.kpc
+
+        # now with a non rotation matrix
+        matrix = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
+        s3 = s1.transform(matrix)
+        expected = s1.to_cartesian().transform(matrix).represent_as(CylindricalRepresentation)
+
+        assert_allclose_quantity(s3.phi, expected.phi)
+        assert_allclose_quantity(s3.z, expected.z)
+        assert_allclose_quantity(s3.rho, expected.rho)
+
 
 def test_cartesian_spherical_roundtrip():
 
@@ -1093,9 +1369,9 @@ def test_subclass_representation():
             return self
 
     class SphericalWrap180Representation(SphericalRepresentation):
-        attr_classes = OrderedDict([('lon', Longitude180),
-                                    ('lat', Latitude),
-                                    ('distance', u.Quantity)])
+        attr_classes = {'lon': Longitude180,
+                        'lat': Latitude,
+                        'distance': u.Quantity}
 
     class ICRSWrap180(ICRS):
         frame_specific_representation_info = ICRS._frame_specific_representation_info.copy()
@@ -1114,9 +1390,9 @@ def test_minimal_subclass():
     # Basically to check what we document works;
     # see doc/coordinates/representations.rst
     class LogDRepresentation(BaseRepresentation):
-        attr_classes = OrderedDict([('lon', Longitude),
-                                    ('lat', Latitude),
-                                    ('logd', u.Dex)])
+        attr_classes = {'lon': Longitude,
+                        'lat': Latitude,
+                        'logd': u.Dex}
 
         def to_cartesian(self):
             d = self.logd.physical
@@ -1162,9 +1438,9 @@ def test_minimal_subclass():
     # so we raise
     with pytest.raises(ValueError):
         class LogDRepresentation(BaseRepresentation):
-            attr_classes = OrderedDict([('lon', Longitude),
-                                        ('lat', Latitude),
-                                        ('logr', u.Dex)])
+            attr_classes = {'lon': Longitude,
+                            'lat': Latitude,
+                            'logr': u.Dex}
 
 
 def test_duplicate_warning():
@@ -1173,8 +1449,8 @@ def test_duplicate_warning():
 
     with pytest.warns(DuplicateRepresentationWarning):
         class UnitSphericalRepresentation(BaseRepresentation):
-            attr_classes = OrderedDict([('lon', Longitude),
-                                        ('lat', Latitude)])
+            attr_classes = {'lon': Longitude,
+                            'lat': Latitude}
 
     assert 'unitspherical' in DUPLICATE_REPRESENTATIONS
     assert 'unitspherical' not in REPRESENTATION_CLASSES
@@ -1182,35 +1458,6 @@ def test_duplicate_warning():
     assert __name__ + '.test_duplicate_warning.<locals>.UnitSphericalRepresentation' in REPRESENTATION_CLASSES
 
 
-def test_combine_xyz():
-
-    x, y, z = np.arange(27).reshape(3, 9) * u.kpc
-    xyz = _combine_xyz(x, y, z, xyz_axis=0)
-    assert xyz.shape == (3, 9)
-    assert np.all(xyz[0] == x)
-    assert np.all(xyz[1] == y)
-    assert np.all(xyz[2] == z)
-
-    x, y, z = np.arange(27).reshape(3, 3, 3) * u.kpc
-    xyz = _combine_xyz(x, y, z, xyz_axis=0)
-    assert xyz.ndim == 3
-    assert np.all(xyz[0] == x)
-    assert np.all(xyz[1] == y)
-    assert np.all(xyz[2] == z)
-
-    xyz = _combine_xyz(x, y, z, xyz_axis=1)
-    assert xyz.ndim == 3
-    assert np.all(xyz[:, 0] == x)
-    assert np.all(xyz[:, 1] == y)
-    assert np.all(xyz[:, 2] == z)
-
-    xyz = _combine_xyz(x, y, z, xyz_axis=-1)
-    assert xyz.ndim == 3
-    assert np.all(xyz[..., 0] == x)
-    assert np.all(xyz[..., 1] == y)
-    assert np.all(xyz[..., 2] == z)
-
-
 class TestCartesianRepresentationWithDifferential:
 
     def test_init_differential(self):
@@ -1370,7 +1617,10 @@ class TestCartesianRepresentationWithDifferential:
         for name in REPRESENTATION_CLASSES:
             if name == 'radial':
                 # TODO: Converting a CartesianDifferential to a
-                #       RadialDifferential fails, even on `master`
+                #       RadialDifferential fails, even on `main`
+                continue
+            elif name.endswith("geodetic"):
+                # TODO: Geodetic representations do not have differentials yet
                 continue
             new_rep = rep1.represent_as(REPRESENTATION_CLASSES[name],
                                         DIFFERENTIAL_CLASSES[name])
@@ -1382,6 +1632,26 @@ class TestCartesianRepresentationWithDifferential:
             rep1.represent_as('name')
         assert 'use frame object' in str(excinfo.value)
 
+    @pytest.mark.parametrize('sph_diff,usph_diff', [
+        (SphericalDifferential, UnitSphericalDifferential),
+        (SphericalCosLatDifferential, UnitSphericalCosLatDifferential)])
+    def test_represent_as_unit_spherical_with_diff(self, sph_diff, usph_diff):
+        """Test that differential angles are correctly reduced."""
+        diff = CartesianDifferential(d_x=1 * u.km/u.s,
+                                     d_y=2 * u.km/u.s,
+                                     d_z=3 * u.km/u.s)
+        rep = CartesianRepresentation(x=1 * u.kpc, y=2 * u.kpc, z=3 * u.kpc,
+                                      differentials=diff)
+        sph = rep.represent_as(SphericalRepresentation, sph_diff)
+        usph = rep.represent_as(UnitSphericalRepresentation, usph_diff)
+        assert components_equal(usph, sph.represent_as(UnitSphericalRepresentation))
+        assert components_equal(usph.differentials['s'],
+                                sph.differentials['s'].represent_as(usph_diff))
+        # Just to be sure components_equal and the represent_as work as advertised,
+        # a sanity check: d_lat is always defined and should be the same.
+        assert_array_equal(sph.differentials['s'].d_lat,
+                           usph.differentials['s'].d_lat)
+
     def test_getitem(self):
 
         d = CartesianDifferential(d_x=np.arange(10) * u.m/u.s,
@@ -1528,8 +1798,8 @@ def unitphysics():
         had_unit = True
 
     class UnitPhysicsSphericalRepresentation(BaseRepresentation):
-        attr_classes = OrderedDict([('phi', Angle),
-                                    ('theta', Angle)])
+        attr_classes = {'phi': Angle,
+                        'theta': Angle}
 
         def __init__(self, *args, copy=True, **kwargs):
             super().__init__(*args, copy=copy, **kwargs)
@@ -1557,17 +1827,17 @@ def unitphysics():
         def unit_vectors(self):
             sinphi, cosphi = np.sin(self.phi), np.cos(self.phi)
             sintheta, costheta = np.sin(self.theta), np.cos(self.theta)
-            return OrderedDict(
-                (('phi', CartesianRepresentation(-sinphi, cosphi, 0., copy=False)),
-                 ('theta', CartesianRepresentation(costheta*cosphi,
-                                                   costheta*sinphi,
-                                                   -sintheta, copy=False))))
+            return {
+                'phi': CartesianRepresentation(-sinphi, cosphi, 0., copy=False),
+                'theta': CartesianRepresentation(costheta*cosphi,
+                                                 costheta*sinphi,
+                                                 -sintheta, copy=False)}
 
         def scale_factors(self):
             sintheta = np.sin(self.theta)
             l = np.broadcast_to(1.*u.one, self.shape, subok=True)
-            return OrderedDict((('phi', sintheta),
-                                ('theta', l)))
+            return {'phi', sintheta,
+                    'theta', l}
 
         def to_cartesian(self):
             x = np.sin(self.theta) * np.cos(self.phi)
