@@ -30,6 +30,7 @@ from astropy.units import allclose as quantity_allclose
 from astropy.io import fits
 from astropy.wcs import WCS
 from astropy.io.misc.asdf.tags.helpers import skycoord_equal
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 RA = 1.0 * u.deg
 DEC = 2.0 * u.deg
@@ -44,13 +45,6 @@ def allclose(a, b, rtol=0.0, atol=None):
     return quantity_allclose(a, b, rtol, atol)
 
 
-try:
-    import scipy
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
-
-
 def setup_function(func):
     func.REPRESENTATION_CLASSES_ORIG = deepcopy(REPRESENTATION_CLASSES)
     func.DUPLICATE_REPRESENTATIONS_ORIG = deepcopy(DUPLICATE_REPRESENTATIONS)
@@ -63,6 +57,24 @@ def teardown_function(func):
     DUPLICATE_REPRESENTATIONS.update(func.DUPLICATE_REPRESENTATIONS_ORIG)
 
 
+def test_is_transformable_to_str_input():
+    """Test method ``is_transformable_to`` with string input.
+
+    The only difference from the frame method of the same name is that
+    strings are allowed. As the frame tests cover ``is_transform_to``, here
+    we only test the added string option.
+
+    """
+    # make example SkyCoord
+    c = SkyCoord(90*u.deg, -11*u.deg)
+
+    # iterate through some frames, checking consistency
+    names = frame_transform_graph.get_names()
+    for name in names:
+        frame = frame_transform_graph.lookup_name(name)()
+        assert c.is_transformable_to(name) == c.is_transformable_to(frame)
+
+
 def test_transform_to():
     for frame in (FK5(), FK5(equinox=Time('J1975.0')),
                   FK4(), FK4(equinox=Time('J1975.0')),
@@ -372,6 +384,13 @@ def test_equal():
     assert (sc1[0] != sc2[0]) == False  # noqa
 
 
+def test_equal_different_type():
+    sc1 = SkyCoord([1, 2]*u.deg, [3, 4]*u.deg, obstime='B1955')
+    # Test equals and not equals operators against different types
+    assert sc1 != 'a string'
+    assert not (sc1 == 'a string')
+
+
 def test_equal_exceptions():
     sc1 = SkyCoord(1*u.deg, 2*u.deg, obstime='B1955')
     sc2 = SkyCoord(1*u.deg, 2*u.deg)
@@ -695,7 +714,7 @@ def test_repr_altaz():
                                 "-4641767.) m, pressure=0.0 hPa, "
                          "temperature=0.0 deg_C, relative_humidity=0.0, "
                          "obswl=1.0 micron): (az, alt, distance) in "
-                         "(deg, deg, m)\n")
+                         "(deg, deg, kpc)\n")
 
 
 def test_ops():
@@ -1270,8 +1289,8 @@ def test_guess_from_table():
 
     tab = Table()
     with NumpyRNGContext(987654321):
-        tab.add_column(Column(data=np.random.rand(1000), unit='deg', name='RA[J2000]'))
-        tab.add_column(Column(data=np.random.rand(1000), unit='deg', name='DEC[J2000]'))
+        tab.add_column(Column(data=np.random.rand(10), unit='deg', name='RA[J2000]'))
+        tab.add_column(Column(data=np.random.rand(10), unit='deg', name='DEC[J2000]'))
 
     sc = SkyCoord.guess_from_table(tab)
     npt.assert_array_equal(sc.ra.deg, tab['RA[J2000]'])
@@ -1286,20 +1305,44 @@ def test_guess_from_table():
 
     # but should work if provided
     sc2 = SkyCoord.guess_from_table(tab, unit=u.deg)
-    npt.assert_array_equal(sc.ra.deg, tab['RA[J2000]'])
-    npt.assert_array_equal(sc.dec.deg, tab['DEC[J2000]'])
+    npt.assert_array_equal(sc2.ra.deg, tab['RA[J2000]'])
+    npt.assert_array_equal(sc2.dec.deg, tab['DEC[J2000]'])
 
     # should fail if two options are available - ambiguity bad!
-    tab.add_column(Column(data=np.random.rand(1000), name='RA_J1900'))
+    tab.add_column(Column(data=np.random.rand(10), name='RA_J1900'))
     with pytest.raises(ValueError) as excinfo:
-        sc3 = SkyCoord.guess_from_table(tab, unit=u.deg)
+        SkyCoord.guess_from_table(tab, unit=u.deg)
     assert 'J1900' in excinfo.value.args[0] and 'J2000' in excinfo.value.args[0]
 
+    tab.remove_column('RA_J1900')
+    tab['RA[J2000]'].unit = u.deg
+    tab['DEC[J2000]'].unit = u.deg
+
+    # but should succeed if the ambiguity can be broken b/c one of the matches
+    # is the name of a different component
+    tab.add_column(Column(data=np.random.rand(10)*u.mas/u.yr,
+                          name='pm_ra_cosdec'))
+    tab.add_column(Column(data=np.random.rand(10)*u.mas/u.yr,
+                          name='pm_dec'))
+    sc3 = SkyCoord.guess_from_table(tab)
+    assert u.allclose(sc3.ra, tab['RA[J2000]'])
+    assert u.allclose(sc3.dec, tab['DEC[J2000]'])
+    assert u.allclose(sc3.pm_ra_cosdec, tab['pm_ra_cosdec'])
+    assert u.allclose(sc3.pm_dec, tab['pm_dec'])
+
+    # should fail if stuff doesn't have proper units
+    tab['RA[J2000]'].unit = None
+    tab['DEC[J2000]'].unit = None
+    with pytest.raises(u.UnitTypeError, match="no unit was given."):
+        SkyCoord.guess_from_table(tab)
+
+    tab.remove_column('pm_ra_cosdec')
+    tab.remove_column('pm_dec')
+
     # should also fail if user specifies something already in the table, but
     # should succeed even if the user has to give one of the components
-    tab.remove_column('RA_J1900')
     with pytest.raises(ValueError):
-        sc3 = SkyCoord.guess_from_table(tab, ra=tab['RA[J2000]'], unit=u.deg)
+        SkyCoord.guess_from_table(tab, ra=tab['RA[J2000]'], unit=u.deg)
 
     oldra = tab['RA[J2000]']
     tab.remove_column('RA[J2000]')
@@ -1465,31 +1508,10 @@ def test_getitem_representation():
     assert sc[0].representation_type is CartesianRepresentation
 
 
-def test_spherical_offsets():
+def test_spherical_offsets_to_api():
     i00 = SkyCoord(0*u.arcmin, 0*u.arcmin, frame='icrs')
-    i01 = SkyCoord(0*u.arcmin, 1*u.arcmin, frame='icrs')
-    i10 = SkyCoord(1*u.arcmin, 0*u.arcmin, frame='icrs')
-    i11 = SkyCoord(1*u.arcmin, 1*u.arcmin, frame='icrs')
-    i22 = SkyCoord(2*u.arcmin, 2*u.arcmin, frame='icrs')
-
-    dra, ddec = i00.spherical_offsets_to(i01)
-    assert_allclose(dra, 0*u.arcmin)
-    assert_allclose(ddec, 1*u.arcmin)
-
-    dra, ddec = i00.spherical_offsets_to(i10)
-    assert_allclose(dra, 1*u.arcmin)
-    assert_allclose(ddec, 0*u.arcmin)
-
-    dra, ddec = i10.spherical_offsets_to(i01)
-    assert_allclose(dra, -1*u.arcmin)
-    assert_allclose(ddec, 1*u.arcmin)
-
-    dra, ddec = i11.spherical_offsets_to(i22)
-    assert_allclose(ddec, 1*u.arcmin)
-    assert 0*u.arcmin < dra < 1*u.arcmin
 
     fk5 = SkyCoord(0*u.arcmin, 0*u.arcmin, frame='fk5')
-
     with pytest.raises(ValueError):
         # different frames should fail
         i00.spherical_offsets_to(fk5)
@@ -1507,6 +1529,34 @@ def test_spherical_offsets():
     assert_allclose(ddec, np.arange(4)*u.arcmin)
 
 
+@pytest.mark.parametrize('frame', ['icrs', 'galactic'])
+@pytest.mark.parametrize('comparison_data', [(0*u.arcmin, 1*u.arcmin),
+                                             (1*u.arcmin, 0*u.arcmin),
+                                             (1*u.arcmin, 1*u.arcmin)])
+def test_spherical_offsets_roundtrip(frame, comparison_data):
+    i00 = SkyCoord(0*u.arcmin, 0*u.arcmin, frame=frame)
+    comparison = SkyCoord(*comparison_data, frame=frame)
+
+    dlon, dlat = i00.spherical_offsets_to(comparison)
+    assert_allclose(dlon, comparison.data.lon)
+    assert_allclose(dlat, comparison.data.lat)
+
+    i00_back = comparison.spherical_offsets_by(-dlon, -dlat)
+
+    # This reaches machine precision when only one component is changed, but for
+    # the third parametrized case (both lon and lat change), the transformation
+    # will have finite accuracy:
+    assert_allclose(i00_back.data.lon, i00.data.lon, atol=1e-10*u.rad)
+    assert_allclose(i00_back.data.lat, i00.data.lat, atol=1e-10*u.rad)
+
+    # Test roundtripping the other direction:
+    init_c = SkyCoord(40.*u.deg, 40.*u.deg, frame=frame)
+    new_c = init_c.spherical_offsets_by(3.534*u.deg, 2.2134*u.deg)
+    dlon, dlat = new_c.spherical_offsets_to(init_c)
+    back_c = new_c.spherical_offsets_by(dlon, dlat)
+    assert init_c.separation(back_c) < 1e-10*u.deg
+
+
 def test_frame_attr_changes():
     """
     This tests the case where a frame is added with a new frame attribute after
@@ -1825,3 +1875,21 @@ def test_passing_inconsistent_coordinates_and_units_raises_helpful_error(kwargs,
     # https://github.com/astropy/astropy/issues/10725
     with pytest.raises(ValueError, match=error_message):
         SkyCoord(**kwargs)
+
+
+@pytest.mark.skipif(not HAS_SCIPY, reason="Requires scipy.")
+def test_match_to_catalog_3d_and_sky():
+    # Test for issue #5857. See PR #11449
+    cfk5_default = SkyCoord([1, 2, 3, 4] * u.degree, [0, 0, 0, 0] * u.degree, distance=[1, 1, 1.5, 1] * u.kpc,
+                            frame='fk5')
+    cfk5_J1950 = cfk5_default.transform_to(FK5(equinox='J1950'))
+
+    idx, angle, quantity = cfk5_J1950.match_to_catalog_3d(cfk5_default)
+    npt.assert_array_equal(idx, [0, 1, 2, 3])
+    assert_allclose(angle, 0*u.deg, atol=1e-15*u.deg, rtol=0)
+    assert_allclose(quantity, 0*u.kpc, atol=1e-15*u.kpc, rtol=0)
+
+    idx, angle, distance = cfk5_J1950.match_to_catalog_sky(cfk5_default)
+    npt.assert_array_equal(idx, [0, 1, 2, 3])
+    assert_allclose(angle, 0 * u.deg, atol=1e-15*u.deg, rtol=0)
+    assert_allclose(distance, 0*u.kpc, atol=1e-15*u.kpc, rtol=0)
