@@ -21,7 +21,7 @@ import subprocess
 from warnings import warn
 
 from abc import ABCMeta, abstractmethod
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 from contextlib import suppress, contextmanager
 from inspect import signature
 
@@ -156,7 +156,7 @@ class TransformGraph:
             The coordinate frame class to start from.
         tosys : class
             The coordinate frame class to transform into.
-        transform : CoordinateTransform or similar callable
+        transform : `CoordinateTransform`
             The transformation object. Typically a `CoordinateTransform` object,
             although it may be some other callable that is called with the same
             signature.
@@ -210,15 +210,15 @@ class TransformGraph:
 
         Parameters
         ----------
-        fromsys : class or `None`
+        fromsys : class or None
             The coordinate frame *class* to start from. If `None`,
             ``transform`` will be searched for and removed (``tosys`` must
             also be `None`).
-        tosys : class or `None`
+        tosys : class or None
             The coordinate frame *class* to transform into. If `None`,
             ``transform`` will be searched for and removed (``fromsys`` must
             also be `None`).
-        transform : callable or `None`
+        transform : callable or None
             The transformation object to be removed or `None`.  If `None`
             and ``tosys`` and ``fromsys`` are supplied, there will be no
             check to ensure the correct object is removed.
@@ -242,8 +242,7 @@ class TransformGraph:
                 if fromsys:
                     break
             else:
-                raise ValueError('Could not find transform {} in the '
-                                 'graph'.format(transform))
+                raise ValueError(f'Could not find transform {transform} in the graph')
 
         else:
             if transform is None:
@@ -276,11 +275,11 @@ class TransformGraph:
 
         Returns
         -------
-        path : list of classes or `None`
+        path : list of class or None
             The path from ``fromsys`` to ``tosys`` as an in-order sequence
             of classes.  This list includes *both* ``fromsys`` and
             ``tosys``. Is `None` if there is no possible path.
-        distance : number
+        distance : float or int
             The total distance/priority from ``fromsys`` to ``tosys``.  If
             priorities are not set this is the number of transforms
             needed. Is ``inf`` if there is no possible path.
@@ -394,7 +393,7 @@ class TransformGraph:
 
         Returns
         -------
-        trans : `CompositeTransform` or `None`
+        trans : `CompositeTransform` or None
             If there is a path from ``fromsys`` to ``tosys``, this is a
             transform object for that path.   If no path could be found, this is
             `None`.
@@ -442,7 +441,7 @@ class TransformGraph:
 
         Returns
         -------
-        coordcls
+        `BaseCoordinateFrame` subclass
             The coordinate class corresponding to the ``name`` or `None` if
             no such class exists.
         """
@@ -478,7 +477,7 @@ class TransformGraph:
         addnodes : sequence of str
             Additional coordinate systems to add (this can include systems
             already in the transform graph, but they will only appear once).
-        savefn : `None` or str
+        savefn : None or str
             The file name to save this graph to or `None` to not save
             to a file.
         savelayout : str
@@ -583,8 +582,8 @@ class TransformGraph:
 
         Returns
         -------
-        nxgraph : `networkx.Graph <https://networkx.github.io/documentation/stable/reference/classes/graph.html>`_
-            This `TransformGraph` as a `networkx.Graph`_.
+        nxgraph : ``networkx.Graph``
+            This `TransformGraph` as a `networkx.Graph <https://networkx.github.io/documentation/stable/reference/classes/graph.html>`_.
         """
         import networkx as nx
 
@@ -625,7 +624,7 @@ class TransformGraph:
             The coordinate frame class to start from.
         tosys : class
             The coordinate frame class to transform into.
-        priority : number
+        priority : float or int
             The priority if this transform when finding the shortest
             coordinate transform path - large numbers are lower priorities.
 
@@ -687,7 +686,7 @@ class TransformGraph:
 
         Parameters
         ----------
-        dt : `~astropy.units.Quantity` or callable
+        dt : `~astropy.units.Quantity` ['time'] or callable
             If a quantity, this is the size of the differential used to do the finite difference.
             If a callable, should accept ``(fromcoord, toframe)`` and return the ``dt`` value.
         """
@@ -718,14 +717,14 @@ class CoordinateTransform(metaclass=ABCMeta):
 
     Parameters
     ----------
-    fromsys : class
+    fromsys : `~astropy.coordinates.BaseCoordinateFrame` subclass
         The coordinate frame class to start from.
-    tosys : class
+    tosys : `~astropy.coordinates.BaseCoordinateFrame` subclass
         The coordinate frame class to transform into.
-    priority : number
+    priority : float or int
         The priority if this transform when finding the shortest
         coordinate transform path - large numbers are lower priorities.
-    register_graph : `TransformGraph` or `None`
+    register_graph : `TransformGraph` or None
         A graph to register this transformation with on creation, or
         `None` to leave it unregistered.
     """
@@ -763,7 +762,7 @@ class CoordinateTransform(metaclass=ABCMeta):
 
         Parameters
         ----------
-        graph : a TransformGraph object
+        graph : `TransformGraph` object
             The graph to register this transformation with.
         """
         graph.add_transform(self.fromsys, self.tosys, self)
@@ -793,7 +792,7 @@ class CoordinateTransform(metaclass=ABCMeta):
 
         Parameters
         ----------
-        fromcoord : fromsys object
+        fromcoord : `~astropy.coordinates.BaseCoordinateFrame` subclass instance
             An object of class matching ``fromsys`` that is to be transformed.
         toframe : object
             An object that has the attributes necessary to fully specify the
@@ -804,7 +803,7 @@ class CoordinateTransform(metaclass=ABCMeta):
 
         Returns
         -------
-        tocoord : tosys object
+        tocoord : `BaseCoordinateFrame` subclass instance
             The new coordinate after the transform has been applied.
         """
 
@@ -825,10 +824,10 @@ class FunctionTransform(CoordinateTransform):
         The coordinate frame class to start from.
     tosys : class
         The coordinate frame class to transform into.
-    priority : number
+    priority : float or int
         The priority if this transform when finding the shortest
         coordinate transform path - large numbers are lower priorities.
-    register_graph : `TransformGraph` or `None`
+    register_graph : `TransformGraph` or None
         A graph to register this transformation with on creation, or
         `None` to leave it unregistered.
 
@@ -895,7 +894,7 @@ class FunctionTransformWithFiniteDifference(FunctionTransform):
         attribute, but only one needs to have it. If None, no velocity
         component induced from the frame itself will be included - only the
         re-orientation of any existing differential.
-    finite_difference_dt : `~astropy.units.Quantity` or callable
+    finite_difference_dt : `~astropy.units.Quantity` ['time'] or callable
         If a quantity, this is the size of the differential used to do the
         finite difference.  If a callable, should accept
         ``(fromcoord, toframe)`` and return the ``dt`` value.
@@ -1212,10 +1211,10 @@ class AffineTransform(BaseAffineTransform):
         The coordinate frame class to start from.
     tosys : class
         The coordinate frame class to transform into.
-    priority : number
+    priority : float or int
         The priority if this transform when finding the shortest
         coordinate transform path - large numbers are lower priorities.
-    register_graph : `TransformGraph` or `None`
+    register_graph : `TransformGraph` or None
         A graph to register this transformation with on creation, or
         `None` to leave it unregistered.
 
@@ -1255,7 +1254,7 @@ class StaticMatrixTransform(BaseAffineTransform):
 
     Parameters
     ----------
-    matrix : array_like or callable
+    matrix : array-like or callable
         A 3 x 3 matrix for transforming 3-vectors. In most cases will
         be unitary (although this is not strictly required). If a callable,
         will be called *with no arguments* to get the matrix.
@@ -1263,10 +1262,10 @@ class StaticMatrixTransform(BaseAffineTransform):
         The coordinate frame class to start from.
     tosys : class
         The coordinate frame class to transform into.
-    priority : number
+    priority : float or int
         The priority if this transform when finding the shortest
         coordinate transform path - large numbers are lower priorities.
-    register_graph : `TransformGraph` or `None`
+    register_graph : `TransformGraph` or None
         A graph to register this transformation with on creation, or
         `None` to leave it unregistered.
 
@@ -1311,10 +1310,10 @@ class DynamicMatrixTransform(BaseAffineTransform):
         The coordinate frame class to start from.
     tosys : class
         The coordinate frame class to transform into.
-    priority : number
+    priority : float or int
         The priority if this transform when finding the shortest
         coordinate transform path - large numbers are lower priorities.
-    register_graph : `TransformGraph` or `None`
+    register_graph : `TransformGraph` or None
         A graph to register this transformation with on creation, or
         `None` to leave it unregistered.
 
@@ -1355,16 +1354,16 @@ class CompositeTransform(CoordinateTransform):
 
     Parameters
     ----------
-    transforms : sequence of `CoordinateTransform` objects
+    transforms : sequence of `CoordinateTransform` object
         The sequence of transformations to apply.
     fromsys : class
         The coordinate frame class to start from.
     tosys : class
         The coordinate frame class to transform into.
-    priority : number
+    priority : float or int
         The priority if this transform when finding the shortest
         coordinate transform path - large numbers are lower priorities.
-    register_graph : `TransformGraph` or `None`
+    register_graph : `TransformGraph` or None
         A graph to register this transformation with on creation, or
         `None` to leave it unregistered.
     collapse_static_mats : bool
@@ -1428,7 +1427,7 @@ class CompositeTransform(CoordinateTransform):
 
 
 # map class names to colorblind-safe colors
-trans_to_color = OrderedDict()
+trans_to_color = {}
 trans_to_color[AffineTransform] = '#555555'  # gray
 trans_to_color[FunctionTransform] = '#783001'  # dark red-ish/brown
 trans_to_color[FunctionTransformWithFiniteDifference] = '#d95f02'  # red-ish
