@@ -5,17 +5,14 @@ from io import StringIO
 import pytest
 import numpy as np
 
-from astropy.cosmology import core, funcs
+from astropy.cosmology import core, funcs, realizations
+from astropy.cosmology.realizations import Planck13, Planck15, Planck18, WMAP5, WMAP7, WMAP9
 from astropy.units import allclose
 from astropy import constants as const
 from astropy import units as u
-
-try:
-    import scipy  # pylint: disable=W0611  # noqa
-except ImportError:
-    HAS_SCIPY = False
-else:
-    HAS_SCIPY = True
+from astropy.utils.exceptions import (AstropyDeprecationWarning,
+                                      AstropyUserWarning)
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 
 def test_init():
@@ -30,8 +27,6 @@ def test_init():
     with pytest.raises(ValueError):
         h0bad = u.Quantity([70, 100], u.km / u.s / u.Mpc)
         cosmo = core.FlatLambdaCDM(H0=h0bad, Om0=0.27)
-    with pytest.raises(ValueError):
-        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, Tcmb0=3, m_nu=0.5)
     with pytest.raises(ValueError):
         bad_mnu = u.Quantity([-0.3, 0.2, 0.1], u.eV)
         cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, Tcmb0=3, m_nu=bad_mnu)
@@ -52,7 +47,14 @@ def test_init():
         cosmo = core.FlatLambdaCDM(H0=70, Om0=0.27)
         cosmo.Odm(1)
     with pytest.raises(TypeError):
-        core.default_cosmology.validate(4)
+        realizations.default_cosmology.validate(4)
+
+
+def test_immutability():
+    """Test immutability of cosmologies."""
+    cosmo = core.Cosmology()
+    with pytest.raises(AttributeError):
+        cosmo.name = "new name"
 
 
 def test_basic():
@@ -281,7 +283,7 @@ def test_clone():
                          Om0=0.27, Ode0=0.5, wa=0.1, Tcmb0=4.0 * u.K)
     newclone = cosmo.clone(w0=-1.1, wa=0.2)
     assert newclone.__class__ == cosmo.__class__
-    assert newclone.name == cosmo.name
+    assert newclone.name == cosmo.name + " (modified)"
     assert allclose(newclone.H0, cosmo.H0)
     assert allclose(newclone.Om0, cosmo.Om0)
     assert allclose(newclone.Ode0, cosmo.Ode0)
@@ -293,7 +295,8 @@ def test_clone():
 
     # Now test exception if user passes non-parameter
     with pytest.raises(AttributeError):
-        newclone = cosmo.clone(not_an_arg=4)
+        with pytest.warns(AstropyDeprecationWarning, match="Astropy v5.0"):
+            newclone = cosmo.clone(not_an_arg=4)
 
 
 def test_xtfuncs():
@@ -1233,11 +1236,16 @@ def test_angular_diameter_distance_z1z2():
     assert allclose(tcos.angular_diameter_distance_z1z2(1, 2),
                     646.22968662822018 * u.Mpc)
 
-    z1 = 0, 0, 2, 0.5, 1
-    z2 = 2, 1, 1, 2.5, 1.1
+    z1 = 2  # Separate test for z2<z1, returns negative value with warning
+    z2 = 1
+    results = -969.34452994 * u.Mpc
+    with pytest.warns(AstropyUserWarning, match='less than first redshift'):
+        assert allclose(tcos.angular_diameter_distance_z1z2(z1, z2), results)
+
+    z1 = 0, 0, 0.5, 1
+    z2 = 2, 1, 2.5, 1.1
     results = (1760.0628637762106,
                1670.7497657219858,
-               -969.34452994,
                1159.0970895962193,
                115.72768186186921) * u.Mpc
 
@@ -1279,7 +1287,7 @@ def test_absorption_distance():
 def test_massivenu_basic():
     # Test no neutrinos case
     tcos = core.FlatLambdaCDM(70.4, 0.272, Neff=4.05,
-                              Tcmb0=2.725 * u.K, m_nu=u.Quantity(0, u.eV))
+                              Tcmb0=2.725 * u.K, m_nu=0)
     assert allclose(tcos.Neff, 4.05)
     assert not tcos.has_massive_nu
     mnu = tcos.m_nu
@@ -1293,7 +1301,7 @@ def test_massivenu_basic():
 
     # Alternative no neutrinos case
     tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=0 * u.K,
-                              m_nu=u.Quantity(0.4, u.eV))
+                              m_nu=str((0.4 * u.eV).to(u.g, u.mass_energy())))
     assert not tcos.has_massive_nu
     assert tcos.m_nu is None
 
@@ -1554,54 +1562,129 @@ def test_z_at_value():
     # there we are checking internal consistency on the same architecture
     # and so can be more demanding
     z_at_value = funcs.z_at_value
-    cosmo = core.Planck13
-    d = cosmo.luminosity_distance(3)
-    assert allclose(z_at_value(cosmo.luminosity_distance, d), 3,
-                    rtol=1e-8)
-    assert allclose(z_at_value(cosmo.age, 2 * u.Gyr), 3.198122684356,
-                    rtol=1e-6)
-    assert allclose(z_at_value(cosmo.luminosity_distance, 1e4 * u.Mpc),
-                    1.3685790653802761, rtol=1e-6)
-    assert allclose(z_at_value(cosmo.lookback_time, 7 * u.Gyr),
-                    0.7951983674601507, rtol=1e-6)
-    assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc,
-                               zmax=2), 0.68127769625288614, rtol=1e-6)
-    assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc,
-                               zmin=2.5), 3.7914908028272083, rtol=1e-6)
-    assert allclose(z_at_value(cosmo.distmod, 46 * u.mag),
-                    1.9913891680278133, rtol=1e-6)
+    cosmo = Planck13
+    assert allclose(z_at_value(cosmo.age, 2 * u.Gyr), 3.19812268, rtol=1e-6)
+    assert allclose(z_at_value(cosmo.lookback_time, 7 * u.Gyr), 0.795198375, rtol=1e-6)
+    assert allclose(z_at_value(cosmo.distmod, 46 * u.mag), 1.991389168, rtol=1e-6)
+    assert allclose(z_at_value(cosmo.luminosity_distance, 1e4 * u.Mpc), 1.36857907, rtol=1e-6)
+    assert allclose(z_at_value(cosmo.luminosity_distance, 26.037193804 * u.Gpc, ztol=1e-10),
+                    3, rtol=1e-9)
+    assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, zmax=2),
+                    0.681277696, rtol=1e-6)
+    assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, zmin=2.5),
+                    3.7914908, rtol=1e-6)
 
     # test behavior when the solution is outside z limits (should
     # raise a CosmologyError)
     with pytest.raises(core.CosmologyError):
-        with pytest.warns(UserWarning, match=r'fval is not bracketed'):
+        with pytest.warns(AstropyUserWarning, match=r'fval is not bracketed'):
             z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, zmax=0.5)
 
     with pytest.raises(core.CosmologyError):
-        with pytest.warns(UserWarning, match=r'fval is not bracketed'):
+        with pytest.warns(AstropyUserWarning, match=r'fval is not bracketed'):
             z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, zmin=4.)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
-def test_z_at_value_roundtrip():
+@pytest.mark.parametrize('method', ['Brent', 'Golden', 'Bounded'])
+def test_z_at_value_bracketed(method):
+    """
+    Test 2 solutions for angular diameter distance by not constraining zmin, zmax,
+    but setting `bracket` on the appropriate side of the turning point z.
+    Setting zmin / zmax should override `bracket`.
+    """
+    z_at_value = funcs.z_at_value
+    cosmo = Planck13
+
+    if method == 'Bounded':
+        with pytest.warns(AstropyUserWarning, match=r'fval is not bracketed'):
+            z = z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method)
+        if z > 1.6:
+            z = 3.7914908
+            bracket = (0.9, 1.5)
+        else:
+            z = 0.6812777
+            bracket = (1.6, 2.0)
+        with pytest.warns(UserWarning, match=r"Option 'bracket' is ignored"):
+            assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                       bracket=bracket), z, rtol=1e-6)
+    else:
+        assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                   bracket=(0.3, 1.0)), 0.6812777, rtol=1e-6)
+        assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                   bracket=(2.0, 4.0)), 3.7914908, rtol=1e-6)
+        assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                   bracket=(0.1, 1.5)), 0.6812777, rtol=1e-6)
+        assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                   bracket=(0.1, 1.0, 2.0)), 0.6812777, rtol=1e-6)
+        with pytest.warns(AstropyUserWarning, match=r'fval is not bracketed'):
+            assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                       bracket=(0.9, 1.5)), 0.6812777, rtol=1e-6)
+            assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                       bracket=(1.6, 2.0)), 3.7914908, rtol=1e-6)
+        assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                   bracket=(1.6, 2.0), zmax=1.6), 0.6812777, rtol=1e-6)
+        assert allclose(z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                                   bracket=(0.9, 1.5), zmin=1.5), 3.7914908, rtol=1e-6)
+
+    with pytest.raises(core.CosmologyError):
+        with pytest.warns(AstropyUserWarning, match=r'fval is not bracketed'):
+            z_at_value(cosmo.angular_diameter_distance, 1500*u.Mpc, method=method,
+                       bracket=(3.9, 5.0), zmin=4.)
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.parametrize('method', ['Brent', 'Golden', 'Bounded'])
+def test_z_at_value_unconverged(method):
+    """
+    Test warnings on non-converged solution when setting `maxfun` to too small iteration number -
+    only 'Bounded' returns status value and specific message.
+    """
+    z_at_value = funcs.z_at_value
+    cosmo = Planck18
+    ztol = {'Brent': [1e-4, 1e-4], 'Golden': [1e-3, 1e-2], 'Bounded': [1e-3, 1e-1]}
+
+    if method == 'Bounded':
+        status, message = 1, 'Maximum number of function calls reached.'
+    else:
+        status, message = None, 'Unsuccessful'
+    diag = rf'Solver returned {status}: {message}'
+
+    with pytest.warns(AstropyUserWarning, match=diag):
+        z0 = z_at_value(cosmo.angular_diameter_distance, 1*u.Gpc, zmax=2, maxfun=13, method=method)
+    with pytest.warns(AstropyUserWarning, match=diag):
+        z1 = z_at_value(cosmo.angular_diameter_distance, 1*u.Gpc, zmin=2, maxfun=13, method=method)
+
+    assert allclose(z0, 0.32442, rtol=ztol[method][0])
+    assert allclose(z1, 8.18551, rtol=ztol[method][1])
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.parametrize('cosmo', [Planck13, Planck15, Planck18, WMAP5, WMAP7, WMAP9,
+                                   core.LambdaCDM, core.FlatLambdaCDM, core.wpwaCDM, core.w0wzCDM,
+                                   core.wCDM, core.FlatwCDM, core.w0waCDM, core.Flatw0waCDM])
+def test_z_at_value_roundtrip(cosmo):
     """
     Calculate values from a known redshift, and then check that
     z_at_value returns the right answer.
     """
     z = 0.5
 
-    # Skip Ok, w, de_density_scale because in the Planck13 cosmology
+    # Skip Ok, w, de_density_scale because in the Planck cosmologies
     # they are redshift independent and hence uninvertable,
     # *_distance_z1z2 methods take multiple arguments, so require
     # special handling
-    # clone isn't a redshift-dependent method
+    # clone is not a redshift-dependent method
+    # nu_relative_density is not redshift-dependent in the WMAP cosmologies
     skip = ('Ok',
             'angular_diameter_distance_z1z2',
             'clone',
             'de_density_scale', 'w')
+    if str(cosmo.name).startswith('WMAP'):
+        skip += ('nu_relative_density', )
 
     import inspect
-    methods = inspect.getmembers(core.Planck13, predicate=inspect.ismethod)
+    methods = inspect.getmembers(cosmo, predicate=inspect.ismethod)
 
     for name, func in methods:
         if name.startswith('_') or name in skip:
@@ -1612,20 +1695,19 @@ def test_z_at_value_roundtrip():
         # angular_diameter_distance and related methods.
         # Be slightly more generous with rtol than the default 1e-8
         # used in z_at_value
-        assert allclose(z, funcs.z_at_value(func, fval, zmax=1.5),
-                        rtol=2e-8)
+        assert allclose(z, funcs.z_at_value(func, fval, bracket=[0.3, 1.0], ztol=1e-12), rtol=2e-11)
 
-    # Test distance functions between two redshifts
-    z2 = 2.0
-    func_z1z2 = [lambda z1: core.Planck13._comoving_distance_z1z2(z1, z2),
-                 lambda z1:
-                 core.Planck13._comoving_transverse_distance_z1z2(z1, z2),
-                 lambda z1:
-                 core.Planck13.angular_diameter_distance_z1z2(z1, z2)]
-    for func in func_z1z2:
-        fval = func(z)
-        assert allclose(z, funcs.z_at_value(func, fval, zmax=1.5),
-                        rtol=2e-8)
+    # Test distance functions between two redshifts; only for realizations
+    if isinstance(cosmo.name, str):
+        z2 = 2.0
+        func_z1z2 = [
+            lambda z1: cosmo._comoving_distance_z1z2(z1, z2),
+            lambda z1: cosmo._comoving_transverse_distance_z1z2(z1, z2),
+            lambda z1: cosmo.angular_diameter_distance_z1z2(z1, z2)
+        ]
+        for func in func_z1z2:
+            fval = func(z)
+            assert allclose(z, funcs.z_at_value(func, fval, zmax=1.5, ztol=1e-12), rtol=2e-11)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -1637,3 +1719,52 @@ def test_elliptic_comoving_distance_z1z2():
                     cosmo._integral_comoving_distance_z1z2(0., z))
     assert allclose(cosmo._elliptic_comoving_distance_z1z2(0., z),
                     cosmo._integral_comoving_distance_z1z2(0., z))
+
+
+SPECIALIZED_COMOVING_DISTANCE_COSMOLOGIES = [
+    core.FlatLambdaCDM(H0=70, Om0=0.0, Tcmb0=0.0),  # de Sitter
+    core.FlatLambdaCDM(H0=70, Om0=1.0, Tcmb0=0.0),  # Einstein - de Sitter
+    core.FlatLambdaCDM(H0=70, Om0=0.3, Tcmb0=0.0),  # Hypergeometric
+    core.LambdaCDM(H0=70, Om0=0.3, Ode0=0.6, Tcmb0=0.0),  # Elliptic
+]
+
+
+ITERABLE_REDSHIFTS = [
+    (0, 1, 2, 3, 4),  # tuple
+    [0, 1, 2, 3, 4],  # list
+    np.array([0, 1, 2, 3, 4]),  # array
+]
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.parametrize('cosmo', SPECIALIZED_COMOVING_DISTANCE_COSMOLOGIES)
+@pytest.mark.parametrize('z', ITERABLE_REDSHIFTS)
+def test_comoving_distance_iterable_argument(cosmo, z):
+    """
+    Regression test for #10980
+    Test that specialized comoving distance methods handle iterable arguments.
+    """
+
+    assert allclose(cosmo.comoving_distance(z),
+                    cosmo._integral_comoving_distance_z1z2(0., z))
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+@pytest.mark.parametrize('cosmo', SPECIALIZED_COMOVING_DISTANCE_COSMOLOGIES)
+def test_comoving_distance_broadcast(cosmo):
+    """
+    Regression test for #10980
+    Test that specialized comoving distance methods broadcast array arguments.
+    """
+
+    z1 = np.zeros((2, 5))
+    z2 = np.ones((3, 1, 5))
+    z3 = np.ones((7, 5))
+    output_shape = np.broadcast(z1, z2).shape
+
+    # Check compatible array arguments return an array with the correct shape
+    assert cosmo._comoving_distance_z1z2(z1, z2).shape == output_shape
+
+    # Check incompatible array arguments raise an error
+    with pytest.raises(ValueError, match='z1 and z2 have different shapes'):
+        cosmo._comoving_distance_z1z2(z1, z3)
