@@ -473,6 +473,9 @@ class Ipac(basic.Basic):
 
         core._apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
 
+        # Check that table has only 1-d columns.
+        self._check_multidim_table(table)
+
         # Now use altered columns
         new_cols = list(table.columns.values())
         # link information about the columns to the writer object (i.e. self)
@@ -494,7 +497,7 @@ class Ipac(basic.Basic):
             for keyword in keydict:
                 try:
                     val = keydict[keyword]['value']
-                    lines.append('\\{}={!r}'.format(keyword.strip(), val))
+                    lines.append(f'\\{keyword.strip()}={val!r}')
                     # meta is not standardized: Catch some common Errors.
                 except TypeError:
                     warn("Table metadata keyword {0} has been skipped.  "
