@@ -13,16 +13,13 @@ from astropy import table
 from astropy.table.table_helpers import simple_table
 from astropy.utils.compat.context import nullcontext
 from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
+from astropy.utils.compat.optional_deps import HAS_BS4
 from astropy import units as u
 
 from .common import setup_function, teardown_function  # noqa
 
-# Check to see if the BeautifulSoup dependency is present.
-try:
+if HAS_BS4:
     from bs4 import BeautifulSoup, FeatureNotFound  # noqa
-    HAS_BEAUTIFUL_SOUP = True
-except ImportError:
-    HAS_BEAUTIFUL_SOUP = False
 
 test_defs = [
     dict(kwargs=dict(),
@@ -400,8 +397,8 @@ def check_write_table(test_def, table, fast_writer):
         if 'not in the list of formats with fast writers' not in str(e.value):
             raise e
         return
-    print('Expected:\n{}'.format(test_def['out']))
-    print('Actual:\n{}'.format(out.getvalue()))
+    print(f"Expected:\n{test_def['out']}")
+    print(f'Actual:\n{out.getvalue()}')
     assert [x.strip() for x in out.getvalue().strip().splitlines()] == [
         x.strip() for x in test_def['out'].strip().splitlines()]
 
@@ -411,7 +408,7 @@ def check_write_table_via_table(test_def, table, fast_writer):
 
     test_def = copy.deepcopy(test_def)
     if 'Writer' in test_def['kwargs']:
-        format = 'ascii.{}'.format(test_def['kwargs']['Writer']._format_name)
+        format = f"ascii.{test_def['kwargs']['Writer']._format_name}"
         del test_def['kwargs']['Writer']
     else:
         format = 'ascii'
@@ -422,8 +419,8 @@ def check_write_table_via_table(test_def, table, fast_writer):
         if 'not in the list of formats with fast writers' not in str(e.value):
             raise e
         return
-    print('Expected:\n{}'.format(test_def['out']))
-    print('Actual:\n{}'.format(out.getvalue()))
+    print(f"Expected:\n{test_def['out']}")
+    print(f'Actual:\n{out.getvalue()}')
     assert [x.strip() for x in out.getvalue().strip().splitlines()] == [
         x.strip() for x in test_def['out'].strip().splitlines()]
 
@@ -737,11 +734,15 @@ def test_roundtrip_masked(fmt_name_class):
         return
 
     # Skip tests for fixed_width or HTML without bs4
-    if ((fmt_name == 'html' and not HAS_BEAUTIFUL_SOUP)
+    if ((fmt_name == 'html' and not HAS_BS4)
             or fmt_name == 'fixed_width'):
         return
 
-    t = simple_table(masked=True)
+    if 'qdp' in fmt_name:
+        # QDP tables are for numeric values only
+        t = simple_table(masked=True, kinds=['f', 'i'])
+    else:
+        t = simple_table(masked=True)
 
     out = StringIO()
     fast = fmt_name in ascii.core.FAST_CLASSES
@@ -752,10 +753,12 @@ def test_roundtrip_masked(fmt_name_class):
 
     # No-header formats need to be told the column names
     kwargs = {'names': t.colnames} if 'no_header' in fmt_name else {}
+    if 'qdp' in fmt_name:
+        kwargs.update({'table_id': 0, 'names': t.colnames})
 
     t2 = ascii.read(out.getvalue(), format=fmt_name, fast_reader=fast, guess=False, **kwargs)
-
     assert t.colnames == t2.colnames
+
     for col, col2 in zip(t.itercols(), t2.itercols()):
         assert col.dtype.kind == col2.dtype.kind
         assert np.all(col == col2)
@@ -779,6 +782,24 @@ def test_write_newlines(fast_writer, tmpdir):
     assert content == os.linesep.join(['col', 'a', 'b', 'c']) + os.linesep
 
 
+@pytest.mark.parametrize("fast_writer", [True, False])
+def test_write_csv_with_comments(fast_writer):
+    """
+    Test fix for #7357 where writing a Table with comments to 'csv' fails with
+    a cryptic message. The comments are dropped by default, but when comment='#'
+    is supplied they are still written.
+    """
+    out = StringIO()
+    t = table.Table([[1, 2], [3, 4]], names=['a', 'b'])
+    t.meta['comments'] = ['hello']
+    ascii.write(t, out, format='csv', fast_writer=fast_writer)
+    assert out.getvalue().splitlines() == ['a,b', '1,3', '2,4']
+
+    out = StringIO()
+    ascii.write(t, out, format='csv', fast_writer=fast_writer, comment='#')
+    assert out.getvalue().splitlines() == ['#hello', 'a,b', '1,3', '2,4']
+
+
 @pytest.mark.parametrize("fast_writer", [True, False])
 def test_write_formatted_mixin(fast_writer):
     """
@@ -801,3 +822,29 @@ def test_validate_write_kwargs():
                        r"\(<class 'bool'>, <class 'str'>\) object, "
                        r"got <class 'int'> instead"):
         ascii.write(t, out, fast_writer=12)
+
+
+@pytest.mark.parametrize("fmt_name_class", fmt_name_classes)
+def test_multidim_column_error(fmt_name_class):
+    """
+    Test that trying to write a multidim column fails in every format except
+    ECSV.
+    """
+    fmt_name, fmt_cls = fmt_name_class
+
+    if not getattr(fmt_cls, '_io_registry_can_write', True):
+        return
+
+    # Skip tests for ecsv or HTML without bs4. See the comment in latex.py
+    # Latex class where max_ndim = None is defined regarding latex and aastex.
+    if ((fmt_name == 'html' and not HAS_BS4)
+            or fmt_name in ('ecsv', 'latex', 'aastex')):
+        return
+
+    out = StringIO()
+    t = table.Table()
+    t['a'] = np.arange(16).reshape(2, 2, 2, 2)
+    t['b'] = [1, 2]
+    fast = fmt_name in ascii.core.FAST_CLASSES
+    with pytest.raises(ValueError, match=r'column\(s\) with dimension'):
+        ascii.write(t, out, format=fmt_name, fast_writer=fast)
