@@ -42,12 +42,6 @@ from astropy.utils.exceptions import AstropyWarning, AstropyDeprecationWarning
 
 _read_trace = []
 
-try:
-    import yaml  # noqa
-    HAS_YAML = True
-except ImportError:
-    HAS_YAML = False
-
 # Default setting for guess parameter in read()
 _GUESS = True
 
@@ -362,8 +356,7 @@ def read(table, guess=None, **kwargs):
                 # special testing value to avoid falling back on the slow reader
                 if fast_reader['enable'] == 'force':
                     raise core.InconsistentTableError(
-                        'fast reader {} exception: {}'
-                        .format(fast_reader_rdr.__class__, err))
+                        f'fast reader {fast_reader_rdr.__class__} exception: {err}')
                 # If the fast reader doesn't work, try the slow version
                 reader = get_reader(**new_kwargs)
                 dat = reader.read(table)
@@ -438,7 +431,7 @@ def _guess(table, read_kwargs, format, fast_reader):
             _read_trace.append({'kwargs': copy.deepcopy(guess_kwargs),
                                 'Reader': guess_kwargs['Reader'].__class__,
                                 'status': 'Disabled: reader only available in fast version',
-                                'dt': '{:.3f} ms'.format(0.0)})
+                                'dt': f'{0.0:.3f} ms'})
             continue
 
         # If user required a fast reader then skip all non-fast readers
@@ -447,7 +440,7 @@ def _guess(table, read_kwargs, format, fast_reader):
             _read_trace.append({'kwargs': copy.deepcopy(guess_kwargs),
                                 'Reader': guess_kwargs['Reader'].__class__,
                                 'status': 'Disabled: no fast version of reader available',
-                                'dt': '{:.3f} ms'.format(0.0)})
+                                'dt': f'{0.0:.3f} ms'})
             continue
 
         guess_kwargs_ok = True  # guess_kwargs are consistent with user_kwargs?
@@ -501,14 +494,13 @@ def _guess(table, read_kwargs, format, fast_reader):
             _read_trace.append({'kwargs': copy.deepcopy(guess_kwargs),
                                 'Reader': reader.__class__,
                                 'status': 'Success (guessing)',
-                                'dt': '{:.3f} ms'.format((time.time() - t0) * 1000)})
+                                'dt': f'{(time.time() - t0) * 1000:.3f} ms'})
             return dat
 
         except guess_exception_classes as err:
             _read_trace.append({'kwargs': copy.deepcopy(guess_kwargs),
-                                'status': '{}: {}'.format(err.__class__.__name__,
-                                                          str(err)),
-                                'dt': '{:.3f} ms'.format((time.time() - t0) * 1000)})
+                                'status': f'{err.__class__.__name__}: {str(err)}',
+                                'dt': f'{(time.time() - t0) * 1000:.3f} ms'})
             failed_kwargs.append(guess_kwargs)
     else:
         # Failed all guesses, try the original read_kwargs without column requirements
@@ -523,8 +515,7 @@ def _guess(table, read_kwargs, format, fast_reader):
 
         except guess_exception_classes as err:
             _read_trace.append({'kwargs': copy.deepcopy(guess_kwargs),
-                                'status': '{}: {}'.format(err.__class__.__name__,
-                                                          str(err))})
+                                'status': f'{err.__class__.__name__}: {str(err)}'})
             failed_kwargs.append(read_kwargs)
             lines = ['\nERROR: Unable to guess table format with the guesses listed below:']
             for kwargs in failed_kwargs:
@@ -670,7 +661,7 @@ def _read_in_chunks_generator(table, chunk_size, **kwargs):
     @contextlib.contextmanager
     def passthrough_fileobj(fileobj, encoding=None):
         """Stub for get_readable_fileobj, which does not seem to work in Py3
-        for input File-like object, see #6460"""
+        for input file-like object, see #6460"""
         yield fileobj
 
     # Set up to coerce `table` input into a readable file object by selecting
@@ -883,7 +874,7 @@ def get_read_trace():
 
     Returns
     -------
-    trace : list of dicts
+    trace : list of dict
        Ordered list of format guesses and status
     """
 
