@@ -1379,7 +1379,7 @@ class ColDefs(NotifierMixin):
         Parameters
         ----------
 
-        input : sequence of `Column`, `ColDefs`, other
+        input : sequence of `Column` or `ColDefs` or ndarray or `~numpy.recarray`
             An existing table HDU, an existing `ColDefs`, or any multi-field
             Numpy array or `numpy.recarray`.
 
@@ -1425,8 +1425,7 @@ class ColDefs(NotifierMixin):
     def _init_from_sequence(self, columns):
         for idx, col in enumerate(columns):
             if not isinstance(col, Column):
-                raise TypeError('Element {} in the ColDefs input is not a '
-                                'Column.'.format(idx))
+                raise TypeError(f'Element {idx} in the ColDefs input is not a Column.')
 
         self._init_from_coldefs(columns)
 
@@ -1494,7 +1493,7 @@ class ColDefs(NotifierMixin):
             valid_kwargs, invalid_kwargs = Column._verify_keywords(**kwargs)
             for val in invalid_kwargs.values():
                 warnings.warn(
-                    'Invalid keyword for column {}: {}'.format(idx + 1, val[1]),
+                    f'Invalid keyword for column {idx + 1}: {val[1]}',
                     VerifyWarning)
             # Special cases for recformat and dim
             # TODO: Try to eliminate the need for these special cases
@@ -1722,6 +1721,9 @@ class ColDefs(NotifierMixin):
         if not isinstance(column, Column):
             raise AssertionError
 
+        # Ask the HDU object to load the data before we modify our columns
+        self._notify('load_data')
+
         self._arrays.append(column.array)
         # Obliterate caches of certain things
         del self.dtype
@@ -1748,6 +1750,9 @@ class ColDefs(NotifierMixin):
             The column's name or index
         """
 
+        # Ask the HDU object to load the data before we modify our columns
+        self._notify('load_data')
+
         indx = _get_index(self.names, col_name)
         col = self.columns[indx]
 
@@ -1834,7 +1839,7 @@ class ColDefs(NotifierMixin):
             and blanks.  If there are two or more attribute names, they must be
             separated by comma(s).
 
-        output : file, optional
+        output : file-like, optional
             File-like object to output to.  Outputs to stdout by default.
             If `False`, returns the attributes as a `dict` instead.
 
@@ -1865,7 +1870,7 @@ class ColDefs(NotifierMixin):
                                  "definitions.\n".format(attr))
                     continue
                 output.write(f"{attr}:\n")
-                output.write('    {}\n'.format(getattr(self, attr + 's')))
+                output.write(f"    {getattr(self, attr + 's')}\n")
             else:
                 ret[attr] = getattr(self, attr + 's')
 
