@@ -5,7 +5,6 @@ import errno
 import http.client
 import mmap
 import operator
-import pathlib
 import io
 import os
 import sys
@@ -26,12 +25,9 @@ from astropy.utils.decorators import classproperty, deprecated_renamed_argument
 from astropy.utils.exceptions import AstropyUserWarning
 
 # NOTE: Python can be built without bz2.
-try:
+from astropy.utils.compat.optional_deps import HAS_BZ2
+if HAS_BZ2:
     import bz2
-except ImportError:
-    HAS_BZ2 = False
-else:
-    HAS_BZ2 = True
 
 
 # Maps astropy.io.fits-specific file mode names to the appropriate file
@@ -134,9 +130,8 @@ class _File:
             return
         else:
             self.simulateonly = False
-            # If fileobj is of type pathlib.Path
-            if isinstance(fileobj, pathlib.Path):
-                fileobj = str(fileobj)
+            if isinstance(fileobj, os.PathLike):
+                fileobj = os.fspath(fileobj)
 
         if mode is not None and mode not in IO_FITS_MODES:
             raise ValueError(f"Mode '{mode}' not recognized")
@@ -215,8 +210,7 @@ class _File:
                 self.memmap = False
 
     def __repr__(self):
-        return '<{}.{} {}>'.format(self.__module__, self.__class__.__name__,
-                                   self._file)
+        return f'<{self.__module__}.{self.__class__.__name__} {self._file}>'
 
     # Support the 'with' statement
     def __enter__(self):
