@@ -20,6 +20,9 @@ except ImportError:
         pass
 
 
+__all__ = ["Section", "PrimaryHDU", "ImageHDU"]
+
+
 class _ImageBaseHDU(_ValidHDU):
     """FITS image HDU base class.
 
@@ -522,7 +525,7 @@ class _ImageBaseHDU(_ValidHDU):
                 self.data = self.data - _zero
             else:
                 # 0.9.6.3 to avoid out of range error for BZERO = +32768
-                # We have to explcitly cast _zero to prevent numpy from raising an
+                # We have to explicitly cast _zero to prevent numpy from raising an
                 # error when doing self.data -= zero, and we do this instead of
                 # self.data = self.data - zero to avoid doubling memory usage.
                 np.add(self.data, -_zero, out=self.data, casting='unsafe')
@@ -1028,7 +1031,7 @@ class PrimaryHDU(_ImageBaseHDU):
 
         Parameters
         ----------
-        data : array or DELAYED, optional
+        data : array or ``astropy.io.fits.hdu.base.DELAYED``, optional
             The data in the HDU.
 
         header : `~astropy.io.fits.Header`, optional
