@@ -3,6 +3,7 @@
 import collections
 import copy
 import itertools
+import numbers
 import re
 import warnings
 
@@ -86,7 +87,7 @@ class Header:
 
         Parameters
         ----------
-        cards : A list of `Card` objects, optional
+        cards : list of `Card`, optional
             The cards to initialize the header with. Also allowed are other
             `Header` (or `dict`-like) objects.
 
@@ -189,7 +190,7 @@ class Header:
             comment = None
 
         card = None
-        if isinstance(key, int):
+        if isinstance(key, numbers.Integral):
             card = self._cards[key]
         elif isinstance(key, tuple):
             card = self._cards[self._cardindex(key)]
@@ -387,7 +388,7 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new `Header` instance.
         """
 
@@ -486,7 +487,7 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new `Header` instance.
         """
 
@@ -599,15 +600,15 @@ class Header:
             if not is_binary:
                 block = encode_ascii(block)
 
+        header_str = ''.join(read_blocks)
+        _check_padding(header_str, actual_block_size, is_eof,
+                       check_block_size=padding)
+
         if not end_found and is_eof and endcard:
             # TODO: Pass this error to validation framework as an ERROR,
             # rather than raising an exception
             raise OSError('Header missing END card.')
 
-        header_str = ''.join(read_blocks)
-        _check_padding(header_str, actual_block_size, is_eof,
-                       check_block_size=padding)
-
         return header_str, cls.fromstring(header_str, sep=sep)
 
     @classmethod
@@ -680,7 +681,7 @@ class Header:
 
         Returns
         -------
-        s : str
+        str
             A string representing a FITS header.
         """
 
@@ -712,9 +713,9 @@ class Header:
 
         Parameters
         ----------
-        fileobj : str, file, optional
+        fileobj : path-like or file-like, optional
             Either the pathname of a file, or an open file handle or file-like
-            object
+            object.
 
         sep : str, optional
             The character or string with which to separate cards.  By default
@@ -824,13 +825,13 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new :class:`Header` instance.
         """
 
         tmp = self.__class__((copy.copy(card) for card in self._cards))
         if strip:
-            tmp._strip()
+            tmp.strip()
         return tmp
 
     def __copy__(self):
@@ -859,7 +860,7 @@ class Header:
 
         Returns
         -------
-        header
+        `Header`
             A new `Header` instance.
         """
 
@@ -886,7 +887,7 @@ class Header:
 
         Returns
         -------
-        value
+        value: str, number, complex, bool, or ``astropy.io.fits.card.Undefined``
             The value associated with the given keyword, or the default value
             if the keyword is not in the header.
         """
@@ -1012,8 +1013,7 @@ class Header:
         """
 
         if len(args) > 2:
-            raise TypeError('Header.pop expected at most 2 arguments, got '
-                            '{}'.format(len(args)))
+            raise TypeError(f'Header.pop expected at most 2 arguments, got {len(args)}')
 
         if len(args) == 0:
             key = -1
@@ -1300,7 +1300,7 @@ class Header:
 
         temp = self.__class__(cards)
         if strip:
-            temp._strip()
+            temp.strip()
 
         if len(self):
             first = self._cards[0].keyword
@@ -1409,8 +1409,7 @@ class Header:
             if self._cards[idx].keyword.upper() == norm_keyword:
                 return idx
         else:
-            raise ValueError('The keyword {!r} is not in the '
-                             ' header.'.format(keyword))
+            raise ValueError(f'The keyword {keyword!r} is not in the  header.')
 
     def insert(self, key, card, useblanks=True, after=False):
         """
@@ -1439,7 +1438,7 @@ class Header:
             rather than before it.  Defaults to `False`.
         """
 
-        if not isinstance(key, int):
+        if not isinstance(key, numbers.Integral):
             # Don't pass through ints to _cardindex because it will not take
             # kindly to indices outside the existing number of cards in the
             # header, which insert needs to be able to support (for example
@@ -1565,8 +1564,7 @@ class Header:
                 raise ValueError('Regular and commentary keys can not be '
                                  'renamed to each other.')
         elif not force and newkeyword in self:
-            raise ValueError('Intended keyword {} already exists in header.'
-                             .format(newkeyword))
+            raise ValueError(f'Intended keyword {newkeyword} already exists in header.')
 
         idx = self.index(oldkeyword)
         card = self._cards[idx]
@@ -1627,6 +1625,38 @@ class Header:
 
         self._add_commentary('', value, before=before, after=after)
 
+    def strip(self):
+        """
+        Strip cards specific to a certain kind of header.
+
+        Strip cards like ``SIMPLE``, ``BITPIX``, etc. so the rest of
+        the header can be used to reconstruct another kind of header.
+        """
+
+        # TODO: Previously this only deleted some cards specific to an HDU if
+        # _hdutype matched that type.  But it seemed simple enough to just
+        # delete all desired cards anyways, and just ignore the KeyErrors if
+        # they don't exist.
+        # However, it might be desirable to make this extendable somehow--have
+        # a way for HDU classes to specify some headers that are specific only
+        # to that type, and should be removed otherwise.
+
+        naxis = self.get('NAXIS', 0)
+        tfields = self.get('TFIELDS', 0)
+
+        for idx in range(naxis):
+            self.remove('NAXIS' + str(idx + 1), ignore_missing=True)
+
+        for name in ('TFORM', 'TSCAL', 'TZERO', 'TNULL', 'TTYPE',
+                     'TUNIT', 'TDISP', 'TDIM', 'THEAP', 'TBCOL'):
+            for idx in range(tfields):
+                self.remove(name + str(idx + 1), ignore_missing=True)
+
+        for name in ('SIMPLE', 'XTENSION', 'BITPIX', 'NAXIS', 'EXTEND',
+                     'PCOUNT', 'GCOUNT', 'GROUPS', 'BSCALE', 'BZERO',
+                     'TFIELDS'):
+            self.remove(name, ignore_missing=True)
+
     def _update(self, card):
         """
         The real update code.  If keyword already exists, its value and/or
@@ -1679,7 +1709,7 @@ class Header:
         if isinstance(key, str):
             keyword = key
             n = 0
-        elif isinstance(key, int):
+        elif isinstance(key, numbers.Integral):
             # If < 0, determine the actual index
             if key < 0:
                 key += len(self._cards)
@@ -1690,7 +1720,7 @@ class Header:
             return key
         elif isinstance(key, tuple):
             if (len(key) != 2 or not isinstance(key[0], str) or
-                    not isinstance(key[1], int)):
+                    not isinstance(key[1], numbers.Integral)):
                 raise ValueError(
                     'Tuple indices must be 2-tuples consisting of a '
                     'keyword string and an integer index.')
@@ -1755,7 +1785,7 @@ class Header:
             insertionkey = before
 
         def get_insertion_idx():
-            if not (isinstance(insertionkey, int) and
+            if not (isinstance(insertionkey, numbers.Integral) and
                     insertionkey >= len(self._cards)):
                 idx = self._cardindex(insertionkey)
             else:
@@ -1902,54 +1932,6 @@ class Header:
                 idx += maxlen
         return cards
 
-    def _strip(self):
-        """
-        Strip cards specific to a certain kind of header.
-
-        Strip cards like ``SIMPLE``, ``BITPIX``, etc. so the rest of
-        the header can be used to reconstruct another kind of header.
-        """
-
-        # TODO: Previously this only deleted some cards specific to an HDU if
-        # _hdutype matched that type.  But it seemed simple enough to just
-        # delete all desired cards anyways, and just ignore the KeyErrors if
-        # they don't exist.
-        # However, it might be desirable to make this extendable somehow--have
-        # a way for HDU classes to specify some headers that are specific only
-        # to that type, and should be removed otherwise.
-
-        if 'NAXIS' in self:
-            naxis = self['NAXIS']
-        else:
-            naxis = 0
-
-        if 'TFIELDS' in self:
-            tfields = self['TFIELDS']
-        else:
-            tfields = 0
-
-        for idx in range(naxis):
-            try:
-                del self['NAXIS' + str(idx + 1)]
-            except KeyError:
-                pass
-
-        for name in ('TFORM', 'TSCAL', 'TZERO', 'TNULL', 'TTYPE',
-                     'TUNIT', 'TDISP', 'TDIM', 'THEAP', 'TBCOL'):
-            for idx in range(tfields):
-                try:
-                    del self[name + str(idx + 1)]
-                except KeyError:
-                    pass
-
-        for name in ('SIMPLE', 'XTENSION', 'BITPIX', 'NAXIS', 'EXTEND',
-                     'PCOUNT', 'GCOUNT', 'GROUPS', 'BSCALE', 'BZERO',
-                     'TFIELDS'):
-            try:
-                del self[name]
-            except KeyError:
-                pass
-
     def _add_commentary(self, key, value, before=None, after=None):
         """
         Add a commentary card.
@@ -2051,7 +2033,7 @@ class _BasicHeader(collections.abc.Mapping):
         self._modified = False
 
     def __getitem__(self, key):
-        if isinstance(key, int):
+        if isinstance(key, numbers.Integral):
             key = self._keys[key]
 
         try:
@@ -2245,7 +2227,7 @@ class _HeaderCommentaryCards(_CardAccessor):
             n = self.__class__(self._header, self._keyword)
             n._indices = idx.indices(self._count)
             return n
-        elif not isinstance(idx, int):
+        elif not isinstance(idx, numbers.Integral):
             raise ValueError(f'{self._keyword} index must be an integer')
 
         idx = list(range(*self._indices))[idx]
@@ -2306,5 +2288,4 @@ def _check_padding(header_str, block_size, is_eof, check_block_size=True):
         # now, but maybe it shouldn't?
         actual_len = len(header_str) - block_size + BLOCK_SIZE
         # TODO: Pass this error to validation framework
-        raise ValueError('Header size is not multiple of {}: {}'
-                         .format(BLOCK_SIZE, actual_len))
+        raise ValueError(f'Header size is not multiple of {BLOCK_SIZE}: {actual_len}')
