@@ -1,4 +1,3 @@
-import os
 import gc
 import sys
 import pathlib
@@ -6,12 +5,12 @@ import warnings
 
 import pytest
 import numpy as np
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose, assert_array_equal
 
 from astropy.io.fits.column import (_parse_tdisp_format, _fortran_to_python_format,
                                     python_to_tdisp)
 
-from astropy.io.fits import HDUList, PrimaryHDU, BinTableHDU, ImageHDU
+from astropy.io.fits import HDUList, PrimaryHDU, BinTableHDU, ImageHDU, table_to_hdu
 
 from astropy.io import fits
 
@@ -19,6 +18,7 @@ from astropy import units as u
 from astropy.table import Table, QTable, NdarrayMixin, Column
 from astropy.table.table_helpers import simple_table
 from astropy.units.format.fits import UnitScaleError
+from astropy.utils.data import get_pkg_data_filename
 from astropy.utils.exceptions import (AstropyUserWarning,
                                       AstropyDeprecationWarning)
 
@@ -27,14 +27,7 @@ from astropy.coordinates import (SkyCoord, Latitude, Longitude, Angle, EarthLoca
                                  SphericalCosLatDifferential)
 from astropy.time import Time, TimeDelta
 from astropy.units.quantity import QuantityInfo
-
-try:
-    import yaml  # pylint: disable=W0611  # noqa
-    HAS_YAML = True
-except ImportError:
-    HAS_YAML = False
-
-DATA = os.path.join(os.path.dirname(__file__), 'data')
+from astropy.utils.compat.optional_deps import HAS_YAML  # noqa
 
 
 def equal_data(a, b):
@@ -147,6 +140,22 @@ class TestSingleTable:
             with pytest.warns(AstropyUserWarning):
                 t3.write(filename, overwrite=True)
 
+        # It should also be possible to read the file in using a unit alias,
+        # even to a unit that may not be the same.
+        with u.set_enabled_aliases({'bandpass_sol_lum': u.Lsun}):
+            t3 = QTable.read(filename)
+            assert t3['l'].unit is u.Lsun
+
+    @pytest.mark.parametrize('table_type', (Table, QTable))
+    def test_read_with_unit_aliases(self, table_type):
+        hdu = BinTableHDU(self.data)
+        hdu.columns[0].unit = 'Angstroms'
+        hdu.columns[2].unit = 'ergs/(cm.s.Angstroms)'
+        with u.set_enabled_aliases(dict(Angstroms=u.AA, ergs=u.erg)):
+            t = table_type.read(hdu)
+        assert t['a'].unit == u.AA
+        assert t['c'].unit == u.erg/(u.cm*u.s*u.AA)
+
     @pytest.mark.parametrize('table_type', (Table, QTable))
     def test_with_format(self, table_type, tmpdir):
         filename = str(tmpdir.join('test_with_format.fits'))
@@ -169,31 +178,35 @@ class TestSingleTable:
         t1.mask['c'] = [0, 1, 1, 0]
         t1.write(filename, overwrite=True)
         t2 = Table.read(filename)
-        assert t2.masked
         assert equal_data(t1, t2)
         assert np.all(t1['a'].mask == t2['a'].mask)
-        # Disabled for now, as there is no obvious way to handle masking of
-        # non-integer columns in FITS
-        # TODO: Re-enable these tests if some workaround for this can be found
-        # assert np.all(t1['b'].mask == t2['b'].mask)
-        # assert np.all(t1['c'].mask == t2['c'].mask)
+        assert np.all(t1['b'].mask == t2['b'].mask)
+        assert np.all(t1['c'].mask == t2['c'].mask)
 
-    def test_masked_nan(self, tmpdir):
+    @pytest.mark.parametrize('masked', [True, False])
+    def test_masked_nan(self, masked, tmpdir):
         filename = str(tmpdir.join('test_masked_nan.fits'))
-        data = np.array(list(zip([5.2, 8.4, 3.9, 6.3],
-                                 [2.3, 4.5, 6.7, 8.9])),
-                        dtype=[('a', np.float64), ('b', np.float32)])
-        t1 = Table(data, masked=True)
-        t1.mask['a'] = [1, 0, 1, 0]
-        t1.mask['b'] = [1, 0, 0, 1]
+        a = np.ma.MaskedArray([5.25, 8.5, 3.75, 6.25], mask=[1, 0, 1, 0])
+        b = np.ma.MaskedArray([2.5, 4.5, 6.75, 8.875], mask=[1, 0, 0, 1], dtype='f4')
+        t1 = Table([a, b], names=['a', 'b'], masked=masked)
         t1.write(filename, overwrite=True)
         t2 = Table.read(filename)
-        np.testing.assert_array_almost_equal(t2['a'], [np.nan, 8.4, np.nan, 6.3])
-        np.testing.assert_array_almost_equal(t2['b'], [np.nan, 4.5, 6.7, np.nan])
-        # assert t2.masked
-        # t2.masked = false currently, as the only way to determine whether a table is masked
-        # while reading is to check whether col.null is present. For float columns, col.null
-        # is not initialized
+        assert_array_equal(t2['a'].data, [np.nan, 8.5, np.nan, 6.25])
+        assert_array_equal(t2['b'].data, [np.nan, 4.5, 6.75, np.nan])
+        assert np.all(t1['a'].mask == t2['a'].mask)
+        assert np.all(t1['b'].mask == t2['b'].mask)
+
+    def test_masked_serialize_data_mask(self, tmpdir):
+        filename = str(tmpdir.join('test_masked_nan.fits'))
+        a = np.ma.MaskedArray([5.25, 8.5, 3.75, 6.25], mask=[1, 0, 1, 0])
+        b = np.ma.MaskedArray([2.5, 4.5, 6.75, 8.875], mask=[1, 0, 0, 1])
+        t1 = Table([a, b], names=['a', 'b'])
+        t1.write(filename, overwrite=True, serialize_method='data_mask')
+        t2 = Table.read(filename)
+        assert_array_equal(t2['a'].data, [5.25, 8.5, 3.75, 6.25])
+        assert_array_equal(t2['b'].data, [2.5, 4.5, 6.75, 8.875])
+        assert np.all(t1['a'].mask == t2['a'].mask)
+        assert np.all(t1['b'].mask == t2['b'].mask)
 
     def test_read_from_fileobj(self, tmpdir):
         filename = str(tmpdir.join('test_read_from_fileobj.fits'))
@@ -268,6 +281,60 @@ class TestSingleTable:
         assert read['x'].shape == (2, 1)
         assert len(read['x'][0]) == 1
 
+    def test_write_append(self, tmpdir):
+
+        t = Table(self.data)
+        hdu = table_to_hdu(t)
+
+        def check_equal(filename, expected, start_from=1):
+            with fits.open(filename) as hdu_list:
+                assert len(hdu_list) == expected
+                for hdu_table in hdu_list[start_from:]:
+                    assert hdu_table.header == hdu.header
+                    assert np.all(hdu_table.data == hdu.data)
+
+        filename = str(tmpdir.join('test_write_append.fits'))
+        t.write(filename, append=True)
+        t.write(filename, append=True)
+        check_equal(filename, 3)
+
+        # Check the overwrite works correctly.
+        t.write(filename, append=True, overwrite=True)
+        t.write(filename, append=True)
+        check_equal(filename, 3)
+
+        # Normal write, check it's not appending.
+        t.write(filename, overwrite=True)
+        t.write(filename, overwrite=True)
+        check_equal(filename, 2)
+
+        # Now write followed by append, with different shaped tables.
+        t2 = Table(np.array([1, 2]))
+        t2.write(filename, overwrite=True)
+        t.write(filename, append=True)
+        check_equal(filename, 3, start_from=2)
+        assert equal_data(t2, Table.read(filename, hdu=1))
+
+    def test_mask_nans_on_read(self, tmpdir):
+        filename = str(tmpdir.join('test_inexact_format_parse_on_read.fits'))
+        c1 = fits.Column(name='a', array=np.array([1, 2, np.nan]), format='E')
+        table_hdu = fits.TableHDU.from_columns([c1])
+        table_hdu.writeto(filename)
+
+        tab = Table.read(filename)
+        assert any(tab.mask)
+        assert tab.mask[2]
+
+    def test_mask_null_on_read(self, tmpdir):
+        filename = str(tmpdir.join('test_null_format_parse_on_read.fits'))
+        col = fits.Column(name='a', array=np.array([1, 2, 99, 60000], dtype='u2'), format='I', null=99, bzero=32768)
+        bin_table_hdu = fits.BinTableHDU.from_columns([col])
+        bin_table_hdu.writeto(filename, overwrite=True)
+
+        tab = Table.read(filename)
+        assert any(tab.mask)
+        assert tab.mask[2]
+
 
 class TestMultipleHDU:
 
@@ -458,11 +525,11 @@ def test_masking_regression_1795():
     Regression test for #1795 - this bug originally caused columns where TNULL
     was not defined to have their first element masked.
     """
-    t = Table.read(os.path.join(DATA, 'tb.fits'))
+    t = Table.read(get_pkg_data_filename('data/tb.fits'))
     assert np.all(t['c1'].mask == np.array([False, False]))
-    assert np.all(t['c2'].mask == np.array([False, False]))
-    assert np.all(t['c3'].mask == np.array([False, False]))
-    assert np.all(t['c4'].mask == np.array([False, False]))
+    assert not hasattr(t['c2'], 'mask')
+    assert not hasattr(t['c3'], 'mask')
+    assert not hasattr(t['c4'], 'mask')
     assert np.all(t['c1'].data == np.array([1, 2]))
     assert np.all(t['c2'].data == np.array([b'abc', b'xy ']))
     assert_allclose(t['c3'].data, np.array([3.70000007153, 6.6999997139]))
@@ -577,7 +644,7 @@ def test_convert_comment_convention(tmpdir):
     """
     Regression test for https://github.com/astropy/astropy/issues/6079
     """
-    filename = os.path.join(DATA, 'stddata.fits')
+    filename = get_pkg_data_filename('data/stddata.fits')
     with pytest.warns(AstropyUserWarning, match=r'hdu= was not specified but '
                       r'multiple tables are present'):
         t = Table.read(filename)
