@@ -25,12 +25,9 @@ from astropy.utils.exceptions import AstropyUserWarning
 from astropy.utils import data
 
 # NOTE: Python can be built without bz2.
-try:
+from astropy.utils.compat.optional_deps import HAS_BZ2
+if HAS_BZ2:
     import bz2
-except ImportError:
-    HAS_BZ2 = False
-else:
-    HAS_BZ2 = True
 
 
 class TestCore(FitsTestCase):
@@ -80,6 +77,25 @@ class TestCore(FitsTestCase):
             with fits.open(self.data('tdim.fits')) as hdulist2:
                 assert FITSDiff(hdulist2, hdulist).identical is True
 
+    def test_fits_pathlike_object(self):
+        """
+        Testing when fits file is passed as os.PathLike object #11579.
+        """
+        class TPath(os.PathLike):
+            def __init__(self, path):
+                self.path = path
+
+            def __fspath__(self):
+                return str(self.path)
+
+        fpath = TPath(self.data('tdim.fits'))
+        with fits.open(fpath) as hdulist:
+            assert hdulist[0].filebytes() == 2880
+            assert hdulist[1].filebytes() == 5760
+
+            with fits.open(self.data('tdim.fits')) as hdulist2:
+                assert FITSDiff(hdulist2, hdulist).identical is True
+
     def test_fits_file_bytes_object(self):
         """
         Testing when fits file is passed as bytes.
@@ -663,7 +679,7 @@ class TestFileFunctions(FitsTestCase):
     @pytest.mark.remote_data(source='astropy')
     def test_open_from_remote_url(self):
         for dataurl in (conf.dataurl, conf.dataurl_mirror):
-            remote_url = '{}/{}'.format(dataurl, 'allsky/allsky_rosat.fits')
+            remote_url = f"{dataurl}/allsky/allsky_rosat.fits"
             try:
                 with urllib.request.urlopen(remote_url) as urlobj:
                     with fits.open(urlobj) as fits_handle:
@@ -1097,7 +1113,7 @@ class TestFileFunctions(FitsTestCase):
         """
         Regression test for https://github.com/astropy/astropy/issues/2356
 
-        Demonstrates that FITS files can still be read from "file-like" objects
+        Demonstrates that FITS files can still be read from file-like objects
         that don't have an obvious "open" or "closed" state.
         """
 
