@@ -12,17 +12,12 @@ from numpy.testing import assert_equal
 from astropy.io import fits
 from astropy.io.fits.hdu.compressed import SUBTRACTIVE_DITHER_1, DITHER_SEED_CHECKSUM
 from astropy.utils.exceptions import AstropyUserWarning
+from astropy.utils.data import get_pkg_data_filename
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 from .test_table import comparerecords
 
 from . import FitsTestCase
 
-try:
-    import scipy  # noqa
-except ImportError:
-    HAS_SCIPY = False
-else:
-    HAS_SCIPY = True
-
 
 class TestImageFunctions(FitsTestCase):
     def test_constructor_name_arg(self):
@@ -1403,7 +1398,7 @@ class TestCompressedImage(FitsTestCase):
     def test_lossless_gzip_compression(self):
         """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/198"""
 
-        rng = np.random.RandomState(seed=42)
+        rng = np.random.default_rng(42)
         noise = rng.normal(size=(20, 20))
 
         chdu1 = fits.CompImageHDU(data=noise, compression_type='GZIP_1')
@@ -1731,6 +1726,68 @@ class TestCompressedImage(FitsTestCase):
         with pytest.raises(KeyError):
             comp_hdu.compressed_data
 
+    def test_compressed_header_double_extname(self):
+        """Test that a double EXTNAME with one default value does not
+        mask the non-default value."""
+        with fits.open(self.data('double_ext.fits')) as hdul:
+            hdu = hdul[1]
+
+            # Raw header has 2 EXTNAME entries
+            indices = hdu._header._keyword_indices['EXTNAME']
+            assert len(indices) == 2
+
+            # The non-default name should be returned.
+            assert hdu.name == 'ccd00'
+            assert 'EXTNAME' in hdu.header
+            assert hdu.name == hdu.header['EXTNAME']
+
+            # There should be 1 non-default EXTNAME entries.
+            indices = hdu.header._keyword_indices['EXTNAME']
+            assert len(indices) == 1
+
+            # Test header sync from property set.
+            new_name = 'NEW_NAME'
+            hdu.name = new_name
+            assert hdu.name == new_name
+            assert hdu.header['EXTNAME'] == new_name
+            assert hdu._header['EXTNAME'] == new_name
+            assert hdu._image_header['EXTNAME'] == new_name
+
+            # Check that setting the header will change the name property.
+            hdu.header['EXTNAME'] = 'NEW2'
+            assert hdu.name == 'NEW2'
+
+            hdul.writeto(self.temp('tmp.fits'), overwrite=True)
+            with fits.open(self.temp('tmp.fits')) as hdul1:
+                hdu1 = hdul1[1]
+                assert len(hdu1._header._keyword_indices['EXTNAME']) == 1
+                assert hdu1.name == 'NEW2'
+
+            # Check that deleting EXTNAME will and setting the name will
+            # work properly.
+            del hdu.header['EXTNAME']
+            hdu.name = 'RE-ADDED'
+            assert hdu.name == 'RE-ADDED'
+
+            with pytest.raises(TypeError):
+                hdu.name = 42
+
+    def test_compressed_header_extname(self):
+        """Test consistent EXTNAME / hdu name interaction."""
+        name = 'FOO'
+        hdu = fits.CompImageHDU(data=np.arange(10), name=name)
+        assert hdu._header['EXTNAME'] == name
+        assert hdu.header['EXTNAME'] == name
+        assert hdu.name == name
+
+        name = 'BAR'
+        hdu.name = name
+        assert hdu._header['EXTNAME'] == name
+        assert hdu.header['EXTNAME'] == name
+        assert hdu.name == name
+
+        assert len(hdu._header._keyword_indices['EXTNAME']) == 1
+
     @pytest.mark.parametrize(
         ('keyword', 'dtype', 'expected'),
         [('BSCALE', np.uint8, np.float32), ('BSCALE', np.int16, np.float32),
@@ -1855,8 +1912,7 @@ def test_bzero_implicit_casting_compressed():
     # case BZERO should be 32768. But if the keyword is stored as 32768.0, then
     # it was possible to trigger the implicit casting error.
 
-    filename = os.path.join(os.path.dirname(__file__),
-                            'data', 'compressed_float_bzero.fits')
+    filename = get_pkg_data_filename('data/compressed_float_bzero.fits')
 
     with fits.open(filename) as hdul:
         hdu = hdul[1]
