@@ -6,7 +6,6 @@ import io
 import mmap
 import operator
 import os
-import pathlib
 import platform
 import signal
 import sys
@@ -26,8 +25,7 @@ import numpy as np
 
 from astropy.utils.exceptions import AstropyUserWarning
 
-from os import PathLike
-path_like = (str, PathLike)
+path_like = (str, os.PathLike)
 
 cmp = lambda a, b: (a > b) - (a < b)
 
@@ -394,7 +392,7 @@ def fileobj_open(filename, mode):
 
 def fileobj_name(f):
     """
-    Returns the 'name' of file-like object f, if it has anything that could be
+    Returns the 'name' of file-like object *f*, if it has anything that could be
     called its name.  Otherwise f's class or type is returned.  If f is a
     string f itself is returned.
     """
@@ -422,14 +420,14 @@ def fileobj_name(f):
 
 def fileobj_closed(f):
     """
-    Returns True if the given file-like object is closed or if f is a string
+    Returns True if the given file-like object is closed or if *f* is a string
     (and assumed to be a pathname).
 
     Returns False for all other types of objects, under the assumption that
     they are file-like objects with no sense of a 'closed' state.
     """
 
-    if isinstance(f, (str, pathlib.Path)):
+    if isinstance(f, path_like):
         return True
 
     if hasattr(f, 'closed'):
@@ -603,7 +601,7 @@ def _array_to_file(arr, outfile):
 
     Parameters
     ----------
-    arr : `~numpy.ndarray`
+    arr : ndarray
         The Numpy array to write.
     outfile : file-like
         A file-like object such as a Python file object, an `io.BytesIO`, or
@@ -910,7 +908,7 @@ def _rstrip_inplace(array):
     # View the array as appropriate integers. The last dimension will
     # equal the number of characters in each string.
     bpc = 1 if dt.kind == 'S' else 4
-    dt_int = "({},){}u{}".format(dt.itemsize // bpc, dt.byteorder, bpc)
+    dt_int = f"({dt.itemsize // bpc},){dt.byteorder}u{bpc}"
     b = array.view(dt_int, np.ndarray)
     # For optimal speed, work in chunks of the internal ufunc buffer size.
     bufsize = np.getbufsize()
