@@ -179,9 +179,9 @@ def _skycoord_constructor(loader, node):
 # Straight from yaml's Representer
 def _complex_representer(self, data):
     if data.imag == 0.0:
-        data = '%r' % data.real
+        data = f'{data.real!r}'
     elif data.real == 0.0:
-        data = '%rj' % data.imag
+        data = f'{data.imag!r}j'
     elif data.imag > 0:
         data = f'{data.real!r}+{data.imag!r}j'
     else:
@@ -293,7 +293,7 @@ def load(stream):
 
     Parameters
     ----------
-    stream : str or file-like object
+    stream : str or file-like
         YAML input
 
     Returns
@@ -310,7 +310,7 @@ def load_all(stream):
 
     Parameters
     ----------
-    stream : str or file-like object
+    stream : str or file-like
         YAML input
 
     Returns
@@ -330,7 +330,7 @@ def dump(data, stream=None, **kwargs):
     ----------
     data: object
         Object to serialize to YAML
-    stream : file-like object, optional
+    stream : file-like, optional
         YAML output (if not supplied a string is returned)
     **kwargs
         Other keyword arguments that get passed to yaml.dump()
