@@ -14,6 +14,8 @@ from astropy.utils.data import get_pkg_data_filename, get_pkg_data_fileobj
 from astropy.io.votable.table import parse, writeto
 from astropy.io.votable import tree, conf
 from astropy.io.votable.exceptions import VOWarning, W39, E25
+from astropy.table import Column, Table
+from astropy.units import Unit
 from astropy.utils.exceptions import AstropyDeprecationWarning
 
 
@@ -72,8 +74,6 @@ def test_table(tmpdir):
 
 
 def test_read_through_table_interface(tmpdir):
-    from astropy.table import Table
-
     with get_pkg_data_fileobj('data/regression.xml', encoding='binary') as fd:
         t = Table.read(fd, format='votable', table_id='main_table')
 
@@ -95,14 +95,19 @@ def test_read_through_table_interface(tmpdir):
 
 
 def test_read_through_table_interface2():
-    from astropy.table import Table
-
     with get_pkg_data_fileobj('data/regression.xml', encoding='binary') as fd:
         t = Table.read(fd, format='votable', table_id='last_table')
 
     assert len(t) == 0
 
 
+def test_pass_kwargs_through_table_interface():
+    # Table.read() should pass on keyword arguments meant for parse()
+    filename = get_pkg_data_filename('data/nonstandard_units.xml')
+    t = Table.read(filename, format='votable', unit_format='generic')
+    assert t['Flux1'].unit == Unit("erg / (Angstrom cm2 s)")
+
+
 def test_names_over_ids():
     with get_pkg_data_fileobj('data/names.xml', encoding='binary') as fd:
         votable = parse(fd)
@@ -130,8 +135,6 @@ def test_table_read_with_unnamed_tables():
     """
     Issue #927
     """
-    from astropy.table import Table
-
     with get_pkg_data_fileobj('data/names.xml', encoding='binary') as fd:
         t = Table.read(fd, format='votable')
 
@@ -150,7 +153,6 @@ def test_votable_path_object():
 
 
 def test_from_table_without_mask():
-    from astropy.table import Table, Column
     t = Table()
     c = Column(data=[1, 2, 3], name='a')
     t.add_column(c)
@@ -159,7 +161,6 @@ def test_from_table_without_mask():
 
 
 def test_write_with_format():
-    from astropy.table import Table, Column
     t = Table()
     c = Column(data=[1, 2, 3], name='a')
     t.add_column(c)
