@@ -27,6 +27,11 @@ import inspect
 import operator
 import warnings
 
+try:
+    from importlib.metadata import entry_points
+except ImportError:
+    from importlib_metadata import entry_points
+
 from functools import reduce, wraps
 
 import numpy as np
@@ -38,16 +43,9 @@ from .optimizers import (SLSQP, Simplex)
 from .statistic import (leastsquare)
 from .optimizers import (DEFAULT_MAXITER, DEFAULT_EPS, DEFAULT_ACC)
 
-# Check pkg_resources exists
-try:
-    from pkg_resources import iter_entry_points
-    HAS_PKG = True
-except ImportError:
-    HAS_PKG = False
-
-
 __all__ = ['LinearLSQFitter', 'LevMarLSQFitter', 'FittingWithOutlierRemoval',
-           'SLSQPLSQFitter', 'SimplexLSQFitter', 'JointFitter', 'Fitter']
+           'SLSQPLSQFitter', 'SimplexLSQFitter', 'JointFitter', 'Fitter',
+           "ModelLinearityError", "ModelsError"]
 
 
 # Statistic functions implemented in `astropy.modeling.statistic.py
@@ -493,9 +491,9 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             model to fit to x, y, z
         x : array
             Input coordinates
-        y : array_like
+        y : array-like
             Input coordinates
-        z : array_like, optional
+        z : array-like, optional
             Input coordinates.
             If the dependent (``y`` or ``z``) co-ordinate values are provided
             as a `numpy.ma.MaskedArray`, any masked points are ignored when
@@ -511,7 +509,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             Cut-off ratio for small singular values of ``a``.
             Singular values are set to zero if they are smaller than ``rcond``
             times the largest singular value of ``a``.
-        equivalencies : list or None, optional and keyword-only argument
+        equivalencies : list or None, optional, keyword-only
             List of *additional* equivalencies that are should be applied in
             case x, y and/or z have units. Default is None.
 
@@ -535,6 +533,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
         _validate_constraints(self.supported_constraints, model)
 
         model_copy = model.copy()
+        model_copy.sync_constraints = False
         _, fitparam_indices = _model_to_fit_params(model_copy)
 
         if model_copy.n_inputs == 2 and z is None:
@@ -785,7 +784,7 @@ class LinearLSQFitter(metaclass=_FitterMeta):
             if len(y) > len(lacoef):
                 self._add_fitting_uncertainties(model_copy, a*scl,
                                                len(lacoef), x, y, z, resids)
-
+        model_copy.sync_constraints = True
         return model_copy
 
 
@@ -798,12 +797,12 @@ class FittingWithOutlierRemoval:
 
     Parameters
     ----------
-    fitter : An Astropy fitter
+    fitter : `Fitter`
         An instance of any Astropy fitter, i.e., LinearLSQFitter,
         LevMarLSQFitter, SLSQPLSQFitter, SimplexLSQFitter, JointFitter. For
         model set fitting, this must understand masked input data (as
         indicated by the fitter class attribute ``supports_masked_input``).
-    outlier_func : function
+    outlier_func : callable
         A function for outlier removal.
         If this accepts an ``axis`` parameter like the `numpy` functions, the
         appropriate value will be supplied automatically when fitting model
@@ -854,13 +853,13 @@ class FittingWithOutlierRemoval:
             An analytic model which will be fit to the provided data.
             This also contains the initial guess for an optimization
             algorithm.
-        x : array_like
+        x : array-like
             Input coordinates.
-        y : array_like
+        y : array-like
             Data measurements (1D case) or input coordinates (2D case).
-        z : array_like, optional
+        z : array-like, optional
             Data measurements (2D case).
-        weights : array_like, optional
+        weights : array-like, optional
             Weights to be passed to the fitter.
         kwargs : dict, optional
             Keyword arguments to be passed to the fitter.
@@ -1133,7 +1132,7 @@ class LevMarLSQFitter(metaclass=_FitterMeta):
             If False (default) and if the model has a fit_deriv method,
             it will be used. Otherwise the Jacobian will be estimated.
             If True, the Jacobian will be estimated in any case.
-        equivalencies : list or None, optional and keyword-only argument
+        equivalencies : list or None, optional, keyword-only
             List of *additional* equivalencies that are should be applied in
             case x, y and/or z have units. Default is None.
 
@@ -1147,6 +1146,7 @@ class LevMarLSQFitter(metaclass=_FitterMeta):
         from scipy import optimize
 
         model_copy = _validate_model(model, self.supported_constraints)
+        model_copy.sync_constraints = False
         farg = (model_copy, weights, ) + _convert_input(x, y, z)
         if model_copy.fit_deriv is None or estimate_jacobian:
             dfunc = None
@@ -1180,6 +1180,7 @@ class LevMarLSQFitter(metaclass=_FitterMeta):
                 self._add_fitting_uncertainties(model_copy,
                                                self.fit_info['param_cov'])
 
+        model_copy.sync_constraints = True
         return model_copy
 
     @staticmethod
@@ -1228,8 +1229,14 @@ class LevMarLSQFitter(metaclass=_FitterMeta):
             return [np.ravel(_) for _ in residues]
         else:
             if z is None:
-                return [np.ravel(_) for _ in np.ravel(weights) *
-                        np.array(model.fit_deriv(x, *params))]
+                try:
+                    return np.array([np.ravel(_) for _ in np.array(weights) *
+                                     np.array(model.fit_deriv(x, *params))])
+                except ValueError:
+                    return np.array([np.ravel(_) for _ in np.array(weights) *
+                                     np.moveaxis(
+                                         np.array(model.fit_deriv(x, *params)),
+                                         -1, 0)]).transpose()
             else:
                 if not model.col_fit_deriv:
                     return [np.ravel(_) for _ in
@@ -1290,7 +1297,7 @@ class SLSQPLSQFitter(Fitter):
             the step size for finite-difference derivative estimates
         acc : float
             Requested accuracy
-        equivalencies : list or None, optional and keyword-only argument
+        equivalencies : list or None, optional, keyword-only
             List of *additional* equivalencies that are should be applied in
             case x, y and/or z have units. Default is None.
 
@@ -1302,6 +1309,7 @@ class SLSQPLSQFitter(Fitter):
         """
 
         model_copy = _validate_model(model, self._opt_method.supported_constraints)
+        model_copy.sync_constraints = False
         farg = _convert_input(x, y, z)
         farg = (model_copy, weights, ) + farg
         init_values, _ = _model_to_fit_params(model_copy)
@@ -1309,6 +1317,7 @@ class SLSQPLSQFitter(Fitter):
             self.objective_function, init_values, farg, **kwargs)
         _fitter_to_model_params(model_copy, fitparams)
 
+        model_copy.sync_constraints = True
         return model_copy
 
 
@@ -1318,7 +1327,7 @@ class SimplexLSQFitter(Fitter):
 
     Raises
     ------
-    ModelLinearityError
+    `ModelLinearityError`
         A linear model is passed to a nonlinear fitter
 
     """
@@ -1354,7 +1363,7 @@ class SimplexLSQFitter(Fitter):
             maximum number of iterations
         acc : float
             Relative error in approximate solution
-        equivalencies : list or None, optional and keyword-only argument
+        equivalencies : list or None, optional, keyword-only
             List of *additional* equivalencies that are should be applied in
             case x, y and/or z have units. Default is None.
 
@@ -1367,6 +1376,7 @@ class SimplexLSQFitter(Fitter):
 
         model_copy = _validate_model(model,
                                      self._opt_method.supported_constraints)
+        model_copy.sync_constraints = False
         farg = _convert_input(x, y, z)
         farg = (model_copy, weights, ) + farg
 
@@ -1375,6 +1385,7 @@ class SimplexLSQFitter(Fitter):
         fitparams, self.fit_info = self._opt_method(
             self.objective_function, init_values, farg, **kwargs)
         _fitter_to_model_params(model_copy, fitparams)
+        model_copy.sync_constraints = True
         return model_copy
 
 
@@ -1472,8 +1483,7 @@ class JointFitter(metaclass=_FitterMeta):
 
     def _verify_input(self):
         if len(self.models) <= 1:
-            raise TypeError("Expected >1 models, {} is given".format(
-                len(self.models)))
+            raise TypeError(f"Expected >1 models, {len(self.models)} is given")
         if len(self.jointparams.keys()) < 2:
             raise TypeError("At least two parameters are expected, "
                             "{} is given".format(len(self.jointparams.keys())))
@@ -1723,12 +1733,13 @@ def populate_entry_points(entry_points):
     This injects entry points into the `astropy.modeling.fitting` namespace.
     This provides a means of inserting a fitting routine without requirement
     of it being merged into astropy's core.
+
     Parameters
     ----------
-    entry_points : a list of `~pkg_resources.EntryPoint`
-                  entry_points are objects which encapsulate
-                  importable objects and are defined on the
-                  installation of a package.
+    entry_points : list of `~importlib.metadata.EntryPoint`
+        entry_points are objects which encapsulate importable objects and
+        are defined on the installation of a package.
+
     Notes
     -----
     An explanation of entry points can be found `here <http://setuptools.readthedocs.io/en/latest/setuptools.html#dynamic-discovery-of-services-and-plugins>`
@@ -1757,6 +1768,13 @@ def populate_entry_points(entry_points):
                         'astropy.modeling.Fitter' .format(name)))
 
 
-# this is so fitting doesn't choke if pkg_resources doesn't exist
-if HAS_PKG:
-    populate_entry_points(iter_entry_points(group='astropy.modeling', name=None))
+def _populate_ep():
+    # TODO: Exclusively use select when Python minversion is 3.10
+    ep = entry_points()
+    if hasattr(ep, 'select'):
+        populate_entry_points(ep.select(group='astropy.modeling'))
+    else:
+        populate_entry_points(ep.get('astropy.modeling', []))
+
+
+_populate_ep()
