@@ -16,7 +16,7 @@ import operator
 import numpy as np
 
 from astropy.units import Quantity
-from astropy.utils import isiterable, OrderedDescriptor
+from astropy.utils import isiterable
 from .utils import array_repr_oneline
 from .utils import get_inputs_and_params
 
@@ -46,8 +46,7 @@ def _tofloat(value):
             # catch arrays with strings or user errors like different
             # types of parameters in a parameter set
             raise InputParameterError(
-                "Parameter of {} could not be converted to "
-                "float".format(type(value)))
+                f"Parameter of {type(value)} could not be converted to float")
     elif isinstance(value, Quantity):
         # Quantities are fine as is
         pass
@@ -61,8 +60,7 @@ def _tofloat(value):
             "Expected parameter to be of numerical type, not boolean")
     else:
         raise InputParameterError(
-            "Don't know how to convert parameter of {} to "
-            "float".format(type(value)))
+            f"Don't know how to convert parameter of {type(value)} to float")
     return value
 
 
@@ -113,14 +111,13 @@ def _unary_arithmetic_operation(op):
     return wrapper
 
 
-class Parameter(OrderedDescriptor):
+class Parameter:
     """
     Wraps individual parameters.
 
-    Since 4.0 Parameters are no longer descriptors (despite the fact that
-    it inherits from OrderedDescriptor) and are based on a new implementation
-    of the Parameter class. Parameters now  (as of 4.0) store values locally
-    (as instead previously in the associated model)
+    Since 4.0 Parameters are no longer descriptors and are based on a new
+    implementation of the Parameter class. Parameters now  (as of 4.0) store
+    values locally (as instead previously in the associated model)
 
     This class represents a model's parameter (in a somewhat broad sense). It
     serves a number of purposes:
@@ -193,10 +190,6 @@ class Parameter(OrderedDescriptor):
     fitters as of this writing.
     """
 
-    # Settings for OrderedDescriptor
-    _class_attribute_ = '_parameters_'
-    _name_attribute_ = '_name'
-
     def __init__(self, name='', description='', default=None, unit=None,
                  getter=None, setter=None, fixed=False, tied=False, min=None,
                  max=None, bounds=None):
@@ -253,6 +246,9 @@ class Parameter(OrderedDescriptor):
 
         self._std = None
 
+    def __set_name__(self, owner, name):
+        self._name = name
+
     def __len__(self):
         val = self.value
         if val.shape == ():
