@@ -464,8 +464,8 @@ class Chebyshev1D(_PolyDomainWindow1D):
         ----------
         x : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -571,8 +571,8 @@ class Hermite1D(_PolyDomainWindow1D):
         ----------
         x : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -713,8 +713,8 @@ class Hermite2D(OrthoPolynomialBase):
             input
         y : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -823,8 +823,8 @@ class Legendre1D(_PolyDomainWindow1D):
         ----------
         x : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -926,8 +926,8 @@ class Polynomial1D(_PolyDomainWindow1D):
         ----------
         x : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -1037,8 +1037,6 @@ class Polynomial2D(PolynomialModel):
         inputs, format_info = super().prepare_inputs(x, y, **kwargs)
 
         x, y = inputs
-        if x.shape != y.shape:
-            raise ValueError("Expected input arrays to have the same shape")
         return (x, y), format_info
 
     def evaluate(self, x, y, *coeffs):
@@ -1087,8 +1085,8 @@ class Polynomial2D(PolynomialModel):
             input
         y : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -1136,7 +1134,8 @@ class Polynomial2D(PolynomialModel):
         Parameters
         ----------
         x, y : array
-        coeffs : array of coefficients in inverse lexical order
+        coeffs : array
+            Coefficients in inverse lexical order.
         """
 
         alpha = self._invlex()
@@ -1296,8 +1295,8 @@ class Chebyshev2D(OrthoPolynomialBase):
             input
         y : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -1434,8 +1433,8 @@ class Legendre2D(OrthoPolynomialBase):
             input
         y : ndarray
             input
-        params : throw away parameter
-            parameter list returned by non-linear fitters
+        *params
+            throw-away parameter list returned by non-linear fitters
 
         Returns
         -------
@@ -1579,7 +1578,7 @@ class SIP(Model):
 
     Parameters
     ----------
-    crpix : list or ndarray of length(2)
+    crpix : list or (2,) ndarray
         CRPIX values
     a_order : int
         SIP polynomial order for first axis
@@ -1707,8 +1706,7 @@ class InverseSIP(Model):
                          name=name, meta=meta)
 
     def __repr__(self):
-        return '<{}({!r})>'.format(self.__class__.__name__,
-                                   [self.sip1d_ap, self.sip1d_bp])
+        return f'<{self.__class__.__name__}({[self.sip1d_ap, self.sip1d_bp]!r})>'
 
     def __str__(self):
         parts = [f'Model: {self.__class__.__name__}']
