@@ -69,7 +69,7 @@ class RotationSequence3D(Model):
 
     Parameters
     ----------
-    angles : array_like
+    angles : array-like
         Angles of rotation in deg in the order of axes_order.
     axes_order : str
         A sequence of 'x', 'y', 'z' corresponding to axis of rotation.
@@ -169,7 +169,7 @@ class _EulerRotation:
 
     def evaluate(self, alpha, delta, phi, theta, psi, axes_order):
         shape = None
-        if isinstance(alpha, np.ndarray) and alpha.ndim == 2:
+        if isinstance(alpha, np.ndarray):
             alpha = alpha.flatten()
             delta = delta.flatten()
             shape = alpha.shape
@@ -209,7 +209,7 @@ class EulerAngleRotation(_EulerRotation, Model):
 
     Parameters
     ----------
-    phi, theta, psi : float or `~astropy.units.Quantity`
+    phi, theta, psi : float or `~astropy.units.Quantity` ['angle']
         "proper" Euler angles in deg.
         If floats, they should be in deg.
     axes_order : str
@@ -288,11 +288,11 @@ class RotateNative2Celestial(_SkyRotation):
 
     Parameters
     ----------
-    lon : float or or `~astropy.units.Quantity`
+    lon : float or `~astropy.units.Quantity` ['angle']
         Celestial longitude of the fiducial point.
-    lat : float or or `~astropy.units.Quantity`
+    lat : float or `~astropy.units.Quantity` ['angle']
         Celestial latitude of the fiducial point.
-    lon_pole : float or or `~astropy.units.Quantity`
+    lon_pole : float or `~astropy.units.Quantity` ['angle']
         Longitude of the celestial pole in the native system.
 
     Notes
@@ -325,15 +325,19 @@ class RotateNative2Celestial(_SkyRotation):
         """
         Parameters
         ----------
-        phi_N, theta_N : float (deg) or `~astropy.units.Quantity`
+        phi_N, theta_N : float or `~astropy.units.Quantity` ['angle']
             Angles in the Native coordinate system.
-        lon, lat, lon_pole : float (in deg) or `~astropy.units.Quantity`
+            it is assumed that numerical only inputs are in degrees.
+            If float, assumed in degrees.
+        lon, lat, lon_pole : float or `~astropy.units.Quantity` ['angle']
             Parameter values when the model was initialized.
+            If float, assumed in degrees.
 
         Returns
         -------
-        alpha_C, delta_C : float (deg) or `~astropy.units.Quantity`
+        alpha_C, delta_C : float or `~astropy.units.Quantity` ['angle']
             Angles on the Celestial sphere.
+            If float, in degrees.
         """
         # The values are in radians since they have already been through the setter.
         if isinstance(lon, u.Quantity):
@@ -359,11 +363,11 @@ class RotateCelestial2Native(_SkyRotation):
 
     Parameters
     ----------
-    lon : float or or `~astropy.units.Quantity`
+    lon : float or `~astropy.units.Quantity` ['angle']
         Celestial longitude of the fiducial point.
-    lat : float or or `~astropy.units.Quantity`
+    lat : float or `~astropy.units.Quantity` ['angle']
         Celestial latitude of the fiducial point.
-    lon_pole : float or or `~astropy.units.Quantity`
+    lon_pole : float or `~astropy.units.Quantity` ['angle']
         Longitude of the celestial pole in the native system.
 
     Notes
@@ -399,15 +403,18 @@ class RotateCelestial2Native(_SkyRotation):
         """
         Parameters
         ----------
-        alpha_C, delta_C : float (deg) or `~astropy.units.Quantity`
+        alpha_C, delta_C : float or `~astropy.units.Quantity` ['angle']
             Angles in the Celestial coordinate frame.
-        lon, lat, lon_pole : float (deg) or `~astropy.units.Quantity`
+            If float, assumed in degrees.
+        lon, lat, lon_pole : float or `~astropy.units.Quantity` ['angle']
             Parameter values when the model was initialized.
+            If float, assumed in degrees.
 
         Returns
         -------
-        phi_N, theta_N : float (deg) or `~astropy.units.Quantity`
+        phi_N, theta_N : float or `~astropy.units.Quantity` ['angle']
             Angles on the Native sphere.
+            If float, in degrees.
 
         """
         if isinstance(lon, u.Quantity):
@@ -435,7 +442,7 @@ class Rotation2D(Model):
 
     Parameters
     ----------
-    angle : float or `~astropy.units.Quantity`
+    angle : float or `~astropy.units.Quantity` ['angle']
         Angle of rotation (if float it should be in deg).
     """
     n_inputs = 2
@@ -463,10 +470,11 @@ class Rotation2D(Model):
 
         Parameters
         ----------
-        x, y : array_like
+        x, y : array-like
             Input quantities
-        angle : float (deg) or `~astropy.units.Quantity`
+        angle : float or `~astropy.units.Quantity` ['angle']
             Angle of rotations.
+            If float, assumed in degrees.
 
         """
 
