@@ -7,6 +7,11 @@ import os.path
 import warnings
 from unittest import mock
 
+try:
+    from importlib.metadata import EntryPoint
+except ImportError:
+    from importlib_metadata import EntryPoint
+
 import pytest
 import numpy as np
 from numpy import linalg
@@ -16,26 +21,18 @@ from astropy.modeling import models
 from astropy.modeling.core import Fittable2DModel, Parameter
 from astropy.modeling.fitting import (
     SimplexLSQFitter, SLSQPLSQFitter, LinearLSQFitter, LevMarLSQFitter,
-    JointFitter, Fitter, FittingWithOutlierRemoval, Covariance)
+    JointFitter, Fitter, FittingWithOutlierRemoval)
 from astropy.utils import NumpyRNGContext
 from astropy.utils.data import get_pkg_data_filename
 from astropy.stats import sigma_clip
 
+from astropy.utils.compat.optional_deps import HAS_SCIPY
 from astropy.utils.exceptions import AstropyUserWarning
 from astropy.modeling.fitting import populate_entry_points
 from . import irafutil
 
-try:
+if HAS_SCIPY:
     from scipy import optimize
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
-
-try:
-    from pkg_resources import EntryPoint
-    HAS_PKG = True
-except ImportError:
-    HAS_PKG = False
 
 
 fitters = [SimplexLSQFitter, SLSQPLSQFitter]
@@ -441,6 +438,7 @@ class TestNonLinearFitters:
         assert_allclose(model.parameters, withw.parameters, rtol=10 ** (-4))
 
     @pytest.mark.filterwarnings(r'ignore:.* Maximum number of iterations reached')
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
     @pytest.mark.parametrize('fitter_class', fitters)
     def test_fitter_against_LevMar(self, fitter_class):
         """Tests results from non-linear fitters against `LevMarLSQFitter`."""
@@ -454,6 +452,7 @@ class TestNonLinearFitters:
         assert_allclose(model.parameters, new_model.parameters,
                         rtol=10 ** (-4))
 
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
     def test_LSQ_SLSQP_with_constraints(self):
         """
         Runs `LevMarLSQFitter` and `SLSQPLSQFitter` on a model with
@@ -469,6 +468,57 @@ class TestNonLinearFitters:
         assert_allclose(model.parameters, slsqp_model.parameters,
                         rtol=10 ** (-4))
 
+    def test_LevMar_with_weights(self):
+        """
+        Tests that issue #11581 has been solved.
+        """
+
+        np.random.seed(42)
+        norder = 2
+
+        fitter1 = LevMarLSQFitter()
+        fitter2 = LinearLSQFitter()
+
+        model = models.Polynomial1D(norder)
+        npts = 10000
+        c = [2.0, -10.0, 7.0]
+        tw = np.random.uniform(0.0, 10.0, npts)
+        tx = np.random.uniform(0.0, 10.0, npts)
+        ty = c[0] + c[1] * tx + c[2] * (tx ** 2)
+        ty += np.random.normal(0.0, 1.5, npts)
+
+        with pytest.warns(AstropyUserWarning, match=r'Model is linear in parameters'):
+            tf1 = fitter1(model, tx, ty, weights=tw)
+        tf2 = fitter2(model, tx, ty, weights=tw)
+
+        assert_allclose(tf1.parameters, tf2.parameters,
+                        atol=10 ** (-16))
+        assert_allclose(tf1.parameters, c,
+                        rtol=10 ** (-2), atol=10 ** (-2))
+
+        model = models.Gaussian1D()
+        fitter1(model, tx, ty, weights=tw)
+
+        model = models.Polynomial2D(norder)
+        nxpts = 100
+        nypts = 150
+        npts = nxpts * nypts
+        c = [1.0, 4.0, 7.0, -8.0, -9.0, -3.0]
+        tw = np.random.uniform(0.0, 10.0, npts).reshape(nxpts, nypts)
+        tx = np.random.uniform(0.0, 10.0, npts).reshape(nxpts, nypts)
+        ty = np.random.uniform(0.0, 10.0, npts).reshape(nxpts, nypts)
+        tz = c[0] + c[1] * tx + c[2] * (tx ** 2) + c[3] * ty + c[4] * (ty ** 2) + c[5] * tx * ty
+        tz += np.random.normal(0.0, 1.5, npts).reshape(nxpts, nypts)
+
+        with pytest.warns(AstropyUserWarning, match=r'Model is linear in parameters'):
+            tf1 = fitter1(model, tx, ty, tz, weights=tw)
+        tf2 = fitter2(model, tx, ty, tz, weights=tw)
+
+        assert_allclose(tf1.parameters, tf2.parameters,
+                        atol=10 ** (-16))
+        assert_allclose(tf1.parameters, c,
+                        rtol=10 ** (-2), atol=10 ** (-2))
+
     def test_simplex_lsq_fitter(self):
         """A basic test for the `SimplexLSQ` fitter."""
 
@@ -524,7 +574,6 @@ class TestNonLinearFitters:
         assert_allclose(olscov, fitter.fit_info['param_cov'])
 
 
-@pytest.mark.skipif('not HAS_PKG')
 class TestEntryPoint:
     """Tests population of fitting with entry point fitters"""
 
@@ -624,6 +673,7 @@ class Test1DFittingWithOutlierRemoval:
         self.y = func(self.model_params, self.x)
 
     @pytest.mark.filterwarnings('ignore:The fit may be unsuccessful')
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
     def test_with_fitters_and_sigma_clip(self):
         import scipy.stats as stats
 
@@ -685,6 +735,7 @@ class Test2DFittingWithOutlierRemoval:
         return amplitude, x_mean, y_mean
 
     @pytest.mark.filterwarnings('ignore:The fit may be unsuccessful')
+    @pytest.mark.filterwarnings(r'ignore:Values in x were outside bounds during a minimize step, clipping to bounds')
     def test_with_fitters_and_sigma_clip(self):
         import scipy.stats as stats
 
