@@ -1,12 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 # pylint: disable=invalid-name, no-member
-from collections import OrderedDict
 
 import pytest
 import numpy as np
 
 from astropy import units as u
 from astropy.tests.helper import assert_quantity_allclose
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 from astropy.modeling.functional_models import (
     Gaussian1D,
@@ -28,12 +28,6 @@ from astropy.modeling.polynomial import Polynomial1D, Polynomial2D
 
 from astropy.modeling.fitting import LevMarLSQFitter
 
-try:
-    from scipy import optimize  # noqa
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
-
 FUNC_MODELS_1D = [
 {'class': Gaussian1D,
  'parameters': {'amplitude': 3 * u.Jy, 'mean': 2 * u.m, 'stddev': 30 * u.cm},
@@ -217,9 +211,9 @@ def test_models_evaluate_without_units(model):
     m = model['class'](**model['parameters'])
     for args in model['evaluation']:
         if len(args) == 2:
-            kwargs = OrderedDict(zip(('x', 'y'), args))
+            kwargs = dict(zip(('x', 'y'), args))
         else:
-            kwargs = OrderedDict(zip(('x', 'y', 'z'), args))
+            kwargs = dict(zip(('x', 'y', 'z'), args))
             if kwargs['x'].unit.is_equivalent(kwargs['y'].unit):
                 kwargs['x'] = kwargs['x'].to(kwargs['y'].unit)
         mnu = m.without_units_for_data(**kwargs)
