@@ -55,12 +55,12 @@ def biweight_location(data, c=6.0, M=None, axis=None, *, ignore_nan=False):
 
     Parameters
     ----------
-    data : array_like
+    data : array-like
         Input array or object that can be converted to an array.
         ``data`` can be a `~numpy.ma.MaskedArray`.
     c : float, optional
         Tuning constant for the biweight estimator (default = 6.0).
-    M : float or array_like, optional
+    M : float or array-like, optional
         Initial guess for the location.  If ``M`` is a scalar value,
         then its value will be used for the entire array (or along each
         ``axis``, if specified).  If ``M`` is an array, then its must be
@@ -68,7 +68,7 @@ def biweight_location(data, c=6.0, M=None, axis=None, *, ignore_nan=False):
         ``axis`` of the input array.  If `None` (default), then the
         median of the input array will be used (or along each ``axis``,
         if specified).
-    axis : `None`, int, or tuple of ints, optional
+    axis : None, int, or tuple of int, optional
         The axis or axes along which the biweight locations are
         computed.  If `None` (default), then the biweight location of
         the flattened input array will be computed.
@@ -99,10 +99,10 @@ def biweight_location(data, c=6.0, M=None, axis=None, *, ignore_nan=False):
 
     >>> import numpy as np
     >>> from astropy.stats import biweight_location
-    >>> rand = np.random.RandomState(12345)
-    >>> biloc = biweight_location(rand.randn(1000))
+    >>> rand = np.random.default_rng(12345)
+    >>> biloc = biweight_location(rand.standard_normal(1000))
     >>> print(biloc)    # doctest: +FLOAT_CMP
-    -0.0175741540445
+    0.01535330525461019
     """
 
     median_func, sum_func = _stat_functions(data, ignore_nan=ignore_nan)
@@ -206,19 +206,19 @@ def biweight_scale(data, c=9.0, M=None, axis=None, modify_sample_size=False,
 
     Parameters
     ----------
-    data : array_like
+    data : array-like
         Input array or object that can be converted to an array.
         ``data`` can be a `~numpy.ma.MaskedArray`.
     c : float, optional
         Tuning constant for the biweight estimator (default = 9.0).
-    M : float or array_like, optional
+    M : float or array-like, optional
         The location estimate.  If ``M`` is a scalar value, then its
         value will be used for the entire array (or along each ``axis``,
         if specified).  If ``M`` is an array, then its must be an array
         containing the location estimate along each ``axis`` of the
         input array.  If `None` (default), then the median of the input
         array will be used (or along each ``axis``, if specified).
-    axis : `None`, int, or tuple of ints, optional
+    axis : None, int, or tuple of int, optional
         The axis or axes along which the biweight scales are computed.
         If `None` (default), then the biweight scale of the flattened
         input array will be computed.
@@ -258,10 +258,10 @@ def biweight_scale(data, c=9.0, M=None, axis=None, modify_sample_size=False,
 
     >>> import numpy as np
     >>> from astropy.stats import biweight_scale
-    >>> rand = np.random.RandomState(12345)
-    >>> biscl = biweight_scale(rand.randn(1000))
+    >>> rand = np.random.default_rng(12345)
+    >>> biscl = biweight_scale(rand.standard_normal(1000))
     >>> print(biscl)    # doctest: +FLOAT_CMP
-    0.986726249291
+    1.0239311812635818
     """
 
     return np.sqrt(
@@ -319,19 +319,19 @@ def biweight_midvariance(data, c=9.0, M=None, axis=None,
 
     Parameters
     ----------
-    data : array_like
+    data : array-like
         Input array or object that can be converted to an array.
         ``data`` can be a `~numpy.ma.MaskedArray`.
     c : float, optional
         Tuning constant for the biweight estimator (default = 9.0).
-    M : float or array_like, optional
+    M : float or array-like, optional
         The location estimate.  If ``M`` is a scalar value, then its
         value will be used for the entire array (or along each ``axis``,
         if specified).  If ``M`` is an array, then its must be an array
         containing the location estimate along each ``axis`` of the
         input array.  If `None` (default), then the median of the input
         array will be used (or along each ``axis``, if specified).
-    axis : `None`, int, or tuple of ints, optional
+    axis : None, int, or tuple of int, optional
         The axis or axes along which the biweight midvariances are
         computed.  If `None` (default), then the biweight midvariance of
         the flattened input array will be computed.
@@ -371,10 +371,10 @@ def biweight_midvariance(data, c=9.0, M=None, axis=None,
 
     >>> import numpy as np
     >>> from astropy.stats import biweight_midvariance
-    >>> rand = np.random.RandomState(12345)
-    >>> bivar = biweight_midvariance(rand.randn(1000))
+    >>> rand = np.random.default_rng(12345)
+    >>> bivar = biweight_midvariance(rand.standard_normal(1000))
     >>> print(bivar)    # doctest: +FLOAT_CMP
-    0.97362869104
+    1.0484350639638342
     """
 
     median_func, sum_func = _stat_functions(data, ignore_nan=ignore_nan)
@@ -537,7 +537,7 @@ def biweight_midcovariance(data, c=9.0, M=None, modify_sample_size=False):
 
     Parameters
     ----------
-    data : 2D or 1D array_like
+    data : 2D or 1D array-like
         Input data either as a 2D or 1D array.  For a 2D array, it
         should have a shape (N_variables, N_observations).  A 1D array
         may be input for observations of a single variable, in which
@@ -549,7 +549,7 @@ def biweight_midcovariance(data, c=9.0, M=None, modify_sample_size=False):
     c : float, optional
         Tuning constant for the biweight estimator (default = 9.0).
 
-    M : float or 1D array_like, optional
+    M : float or 1D array-like, optional
         The location estimate of each variable, either as a scalar or
         array.  If ``M`` is an array, then its must be a 1D array
         containing the location estimate of each row (i.e. ``a.ndim``
@@ -568,7 +568,7 @@ def biweight_midcovariance(data, c=9.0, M=None, modify_sample_size=False):
 
     Returns
     -------
-    biweight_midcovariance : `~numpy.ndarray`
+    biweight_midcovariance : ndarray
         A 2D array representing the biweight midcovariances between each
         pair of the variables (rows) in the input array.  The output
         array will have a shape of (N_variables, N_variables).  The
@@ -591,19 +591,19 @@ def biweight_midcovariance(data, c=9.0, M=None, modify_sample_size=False):
     >>> import numpy as np
     >>> from astropy.stats import biweight_midcovariance
     >>> # Generate two random variables x and y
-    >>> rng = np.random.RandomState(1)
+    >>> rng = np.random.default_rng(1)
     >>> x = rng.normal(0, 1, 200)
     >>> y = rng.normal(0, 3, 200)
     >>> # Introduce an obvious outlier
     >>> x[0] = 30.0
     >>> # Calculate the biweight midcovariances between x and y
     >>> bicov = biweight_midcovariance([x, y])
-    >>> print(bicov)    # doctest: +FLOAT_CMP
-    [[ 0.82483155 -0.18961219]
-     [-0.18961219 9.80265764]]
+    >>> print(bicov)  # doctest: +FLOAT_CMP
+    [[0.83435568 0.02379316]
+     [0.02379316 7.15665769]]
     >>> # Print standard deviation estimates
-    >>> print(np.sqrt(bicov.diagonal()))    # doctest: +FLOAT_CMP
-    [ 0.90820237  3.13091961]
+    >>> print(np.sqrt(bicov.diagonal()))  # doctest: +FLOAT_CMP
+    [0.91343072 2.67519302]
     """
 
     data = np.asanyarray(data).astype(np.float64)
@@ -678,13 +678,13 @@ def biweight_midcorrelation(x, y, c=9.0, M=None, modify_sample_size=False):
 
     Parameters
     ----------
-    x, y : 1D array_like
+    x, y : 1D array-like
         Input arrays for the two variables.  ``x`` and ``y`` must be 1D
         arrays and have the same number of elements.
     c : float, optional
         Tuning constant for the biweight estimator (default = 9.0).  See
         `biweight_midcovariance` for more details.
-    M : float or array_like, optional
+    M : float or array-like, optional
         The location estimate.  If ``M`` is a scalar value, then its
         value will be used for the entire array (or along each ``axis``,
         if specified).  If ``M`` is an array, then its must be an array
@@ -722,14 +722,14 @@ def biweight_midcorrelation(x, y, c=9.0, M=None, modify_sample_size=False):
 
     >>> import numpy as np
     >>> from astropy.stats import biweight_midcorrelation
-    >>> rng = np.random.RandomState(12345)
+    >>> rng = np.random.default_rng(12345)
     >>> x = rng.normal(0, 1, 200)
     >>> y = rng.normal(0, 3, 200)
     >>> # Introduce an obvious outlier
     >>> x[0] = 30.0
     >>> bicorr = biweight_midcorrelation(x, y)
-    >>> print(bicorr)    # doctest: +FLOAT_CMP
-    -0.0495780713907
+    >>> print(bicorr)  # doctest: +FLOAT_CMP
+    -0.09203238319481295
     """
 
     x = np.asanyarray(x)
