@@ -4,17 +4,12 @@ import pytest
 import numpy as np
 from numpy.testing import assert_equal, assert_allclose
 
-try:
-    from scipy import stats  # used in testing
-except ImportError:
-    HAS_SCIPY = False
-else:
-    HAS_SCIPY = True
-
 from astropy import units as u
+from astropy.stats import mad_std
 from astropy.stats.sigma_clipping import sigma_clip, SigmaClip, sigma_clipped_stats
 from astropy.utils.exceptions import AstropyUserWarning
 from astropy.utils.misc import NumpyRNGContext
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 
 def test_sigma_clip():
@@ -64,11 +59,24 @@ def test_sigma_clip():
         assert filtered_data.count() == 25
 
 
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_axis_none():
+    """
+    For masked=False and axis=None, masked elements should be removed
+    from the result.
+    """
+    data = np.arange(10.)
+    data[0] = 100
+    result = sigma_clip(data, masked=False, axis=None)
+    assert_equal(result, data[1:])
+
+
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_compare_to_scipy_sigmaclip():
+    from scipy import stats
+
     # need to seed the numpy RNG to make sure we don't get some
     # amazingly flukey random number that breaks one of the tests
-
     with NumpyRNGContext(12345):
 
         randvar = np.random.randn(10000)
@@ -169,9 +177,17 @@ def test_invalid_sigma_clip():
     data[3, 4] = np.nan
     data[1, 1] = np.inf
 
+    data_ma = np.ma.MaskedArray(data)
+
     with pytest.warns(AstropyUserWarning,
                       match=r'Input data contains invalid values'):
         result = sigma_clip(data)
+    with pytest.warns(AstropyUserWarning,
+                      match=r'Input data contains invalid values'):
+        result_ma = sigma_clip(data_ma)
+
+    assert_equal(result.data, result_ma.data)
+    assert_equal(result.mask, result_ma.mask)
 
     # Pre #4051 if data contains any NaN or infs sigma_clip returns the
     # mask containing `False` only or TypeError if data also contains a
@@ -196,8 +212,8 @@ def test_invalid_sigma_clip():
     data[0, :] = np.nan     # row of all nans
     with pytest.warns(AstropyUserWarning,
                       match=r'Input data contains invalid values'):
-        result4, minarr, maxarr = sigma_clip(data, axis=1, masked=False,
-                                             return_bounds=True)
+        _, minarr, maxarr = sigma_clip(data, axis=1, masked=False,
+                                       return_bounds=True)
     assert np.isnan(minarr[0])
     assert np.isnan(maxarr[0])
 
@@ -217,7 +233,7 @@ def test_sigmaclip_fully_masked():
     data = np.ma.MaskedArray(data=[[1., 0.], [0., 1.]],
                              mask=[[True, True], [True, True]])
     clipped_data = sigma_clip(data)
-    np.ma.allequal(data, clipped_data)
+    assert np.ma.allequal(data, clipped_data)
 
     clipped_data = sigma_clip(data, masked=False)
     assert not isinstance(clipped_data, np.ma.MaskedArray)
@@ -231,7 +247,7 @@ def test_sigmaclip_empty_masked():
 
     data = np.ma.MaskedArray(data=[], mask=[])
     clipped_data = sigma_clip(data)
-    np.ma.allequal(data, clipped_data)
+    assert np.ma.allequal(data, clipped_data)
 
 
 def test_sigmaclip_empty():
@@ -271,16 +287,14 @@ def test_sigma_clip_axis_tuple_3D():
 def test_sigmaclip_repr():
 
     sigclip = SigmaClip()
-    median_str = str(sigclip._parse_cenfunc('median'))
-    std_str = str(sigclip._parse_stdfunc('std'))
 
     sigclip_repr = ('SigmaClip(sigma=3.0, sigma_lower=3.0, sigma_upper=3.0,'
-                    ' maxiters=5, cenfunc={}, stdfunc={}, '
-                    'grow=False)'.format(median_str, std_str))
+                    ' maxiters=5, cenfunc=median, stdfunc=std, '
+                    'grow=False)')
     sigclip_str = ('<SigmaClip>\n    sigma: 3.0\n    sigma_lower: 3.0\n'
                    '    sigma_upper: 3.0\n    maxiters: 5\n'
-                   '    cenfunc: {}\n    stdfunc: {}\n'
-                   '    grow: False'.format(median_str, std_str))
+                   '    cenfunc: median\n    stdfunc: std\n'
+                   '    grow: False')
 
     assert repr(sigclip) == sigclip_repr
     assert str(sigclip) == sigclip_str
@@ -412,3 +426,91 @@ def test_sigma_clip_grow():
     )
 
     assert np.array_equal(np.where(filtered_data.mask), expected)
+
+
+@pytest.mark.parametrize(('axis', 'bounds_shape'), [(0, (4, 5, 6, 7)),
+                                                    (1, (3, 5, 6, 7)),
+                                                    (-1, (3, 4, 5, 6)),
+                                                    ((1, 3), (3, 5, 7)),
+                                                    ((3, 1), (3, 5, 7)),
+                                                    ((1, 2, 4), (3, 6))])
+def test_sigma_clip_axis_shapes(axis, bounds_shape):
+
+    # Check the shapes of the output for different use cases
+
+    with NumpyRNGContext(12345):
+        array = np.random.random((3, 4, 5, 6, 7))
+
+    result1 = sigma_clip(array, axis=axis)
+    assert result1.shape == array.shape
+
+    result2, bound1, bound2 = sigma_clip(array, axis=axis, return_bounds=True)
+    assert result2.shape == array.shape
+    assert bound1.shape == bounds_shape
+    assert bound2.shape == bounds_shape
+
+
+@pytest.mark.parametrize('dtype', ['>f2', '<f2', '>f4', '<f4', '>f8', '<f8', '<i4', '>i8'])
+def test_sigma_clip_dtypes(dtype):
+
+    # Check the shapes of the output for different use cases
+
+    with NumpyRNGContext(12345):
+        array = np.random.randint(-5, 5, 1000).astype(float)
+    array[30] = 100
+
+    reference = sigma_clip(array, copy=True, masked=False)
+
+    actual = sigma_clip(array.astype(dtype), copy=True, masked=False)
+
+    assert_equal(reference, actual)
+
+
+def test_mad_std():
+
+    # Check with a small array where we know how the result should differ from std
+
+    # Choose an array with few elements and a high proportion of outliers since
+    # in this case std and mad_std will be very different.
+    array = np.array([1, 10000, 4, 3, 10000])
+
+    # First check with regular std, which shouldn't remove any values
+    result_std = sigma_clip(array, cenfunc='median', stdfunc='std',
+                            maxiters=1, sigma=5, masked=False)
+    assert_equal(result_std, array)
+
+    # Whereas using mad_std should result in the high values being removed
+    result_mad_std = sigma_clip(array, cenfunc='median', stdfunc='mad_std',
+                                maxiters=1, sigma=5, masked=False)
+    assert_equal(result_mad_std, [1, 4, 3])
+
+    # We now check this again but with the axis= keyword set since at the time
+    # of writing this test this relies on a fast C implementation in which we
+    # have re-inplemented mad_std.
+
+    result_std = sigma_clip(array, cenfunc='median', stdfunc='std',
+                            maxiters=1, sigma=5, masked=False, axis=0)
+    assert_equal(result_std, array)
+
+    result_mad_std = sigma_clip(array, cenfunc='median', stdfunc='mad_std',
+                                maxiters=1, sigma=5, masked=False, axis=0)
+    assert_equal(result_mad_std, [1, np.nan, 4, 3, np.nan])
+
+
+def test_mad_std_large():
+
+    # And now test with a larger array and compare with Python mad_std function
+
+    with NumpyRNGContext(12345):
+        array = np.random.uniform(-1, 2, (30, 40))
+
+    def nan_mad_std(data, axis=None):
+        return mad_std(data, axis=axis, ignore_nan=True)
+
+    result1 = sigma_clip(array, sigma=2, maxiters=None,
+                         stdfunc=nan_mad_std, axis=0, masked=False)
+
+    result2 = sigma_clip(array, sigma=2, maxiters=None,
+                         stdfunc='mad_std', axis=0, masked=False)
+
+    assert_allclose(result1, result2)
