@@ -133,8 +133,7 @@ def column_group_by(column, keys):
         keys = keys.as_array()
 
     if not isinstance(keys, np.ndarray):
-        raise TypeError('Keys input must be numpy array, but got {}'
-                        .format(type(keys)))
+        raise TypeError(f'Keys input must be numpy array, but got {type(keys)}')
 
     if len(keys) != len(column):
         raise ValueError('Input keys array length {} does not match column length {}'
