@@ -18,6 +18,7 @@ from collections.abc import Mapping, Sequence
 import numpy as np
 
 from astropy.utils import metadata
+from astropy.utils.masked import Masked
 from .table import Table, QTable, Row, Column, MaskedColumn
 from astropy.units import Quantity
 
@@ -64,8 +65,7 @@ def _get_list_of_tables(tables):
             try:
                 tables[ii] = Table([val])
             except (ValueError, TypeError):
-                raise TypeError('cannot convert {} to table column.'
-                                .format(val))
+                raise TypeError(f'cannot convert {val} to table column.')
 
     return tables
 
@@ -111,8 +111,9 @@ def join_skycoord(distance, distance_func='search_around_sky'):
 
     Parameters
     ----------
-    distance : Quantity (angle or length)
-        Maximum distance between points to be considered a join match
+    distance : `~astropy.units.Quantity` ['angle', 'length']
+        Maximum distance between points to be considered a join match.
+        Must have angular or distance units.
     distance_func : str or function
         Specifies the function for performing the cross-match based on
         ``distance``. If supplied as a string this specifies the name of a
@@ -222,7 +223,7 @@ def join_distance(distance, kdtree_args=None, query_args=None):
 
     Parameters
     ----------
-    distance : float, Quantity
+    distance : float or `~astropy.units.Quantity` ['length']
         Maximum distance between points to be considered a join match
     kdtree_args : dict, None
         Optional extra args for `~scipy.spatial.cKDTree`
@@ -344,10 +345,10 @@ def join(left, right, keys=None, join_type='inner',
 
     Parameters
     ----------
-    left : Table object or a value that will initialize a Table object
-        Left side table in the join
-    right : Table object or a value that will initialize a Table object
-        Right side table in the join
+    left : `~astropy.table.Table`-like object
+        Left side table in the join. If not a Table, will call ``Table(left)``
+    right : `~astropy.table.Table`-like object
+        Right side table in the join. If not a Table, will call ``Table(right)``
     keys : str or list of str
         Name(s) of column(s) used to match rows of left and right tables.
         Default is to use all columns which are common to both tables.
@@ -499,7 +500,7 @@ def dstack(tables, join_type='outer', metadata_conflicts='warn'):
 
     Parameters
     ----------
-    tables : Table or list of Table or Row objects
+    tables : `~astropy.table.Table` or `~astropy.table.Row` or list thereof
         Table(s) to stack along depth-wise with the current table
         Table columns should have same shape and name for depth-wise stacking
     join_type : str
@@ -592,8 +593,8 @@ def vstack(tables, join_type='outer', metadata_conflicts='warn'):
 
     Parameters
     ----------
-    tables : Table or list of Table or Row objects
-        Table(s) to stack along rows (vertically)
+    tables : `~astropy.table.Table` or `~astropy.table.Row` or list thereof
+        Table(s) to stack along rows (vertically) with the current table
     join_type : str
         Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     metadata_conflicts : str
@@ -661,8 +662,8 @@ def hstack(tables, join_type='outer',
 
     Parameters
     ----------
-    tables : Table or list of Table or Row objects
-        Tables to stack along columns (horizontally)
+    tables : `~astropy.table.Table` or `~astropy.table.Row` or list thereof
+        Tables to stack along columns (horizontally) with the current table
     join_type : str
         Join type ('inner' | 'exact' | 'outer'), default is 'outer'
     uniq_col_name : str or None
@@ -674,7 +675,8 @@ def hstack(tables, join_type='outer',
     metadata_conflicts : str
         How to proceed with metadata conflicts. This should be one of:
             * ``'silent'``: silently pick the last conflicting meta-data value
-            * ``'warn'``: pick the last conflicting meta-data value, but emit a warning (default)
+            * ``'warn'``: pick the last conflicting meta-data value,
+              but emit a warning (default)
             * ``'error'``: raise an exception.
 
     Returns
@@ -726,13 +728,11 @@ def unique(input_table, keys=None, silent=False, keep='first'):
 
     Parameters
     ----------
-
-    input_table : `~astropy.table.Table` object or a value that
-        will initialize a `~astropy.table.Table` object
+    input_table : table-like
     keys : str or list of str
         Name(s) of column(s) used to create unique rows.
         Default is to use all columns.
-    keep : one of 'first', 'last' or 'none'
+    keep : {'first', 'last', 'none'}
         Whether to keep the first or last row for each set of
         duplicates. If 'none', all rows that are duplicate are
         removed, leaving only rows that are already unique in
@@ -954,8 +954,7 @@ def common_dtype(cols):
     try:
         return metadata.common_dtype(cols)
     except metadata.MergeConflictError as err:
-        tme = TableMergeError('Columns have incompatible types {}'
-                              .format(err._incompat_types))
+        tme = TableMergeError(f'Columns have incompatible types {err._incompat_types}')
         tme._incompat_types = err._incompat_types
         raise tme
 
@@ -1192,6 +1191,9 @@ def _join(left, right, keys=None, join_type='inner',
             if isinstance(col, Column) and not isinstance(col, MaskedColumn):
                 col = out.MaskedColumn(col, copy=False)
 
+            if isinstance(col, Quantity) and not isinstance(col, Masked):
+                col = Masked(col, copy=False)
+
             # array_mask is 1-d corresponding to length of output column.  We need
             # make it have the correct shape for broadcasting, i.e. (length, 1, 1, ..).
             # Mixin columns might not have ndim attribute so use len(col.shape).
@@ -1322,6 +1324,9 @@ def _vstack(arrays, join_type='outer', col_name_map=None, metadata_conflicts='wa
                 if isinstance(col, Column) and not isinstance(col, MaskedColumn):
                     col = out.MaskedColumn(col, copy=False)
 
+                if isinstance(col, Quantity) and not isinstance(col, Masked):
+                    col = Masked(col, copy=False)
+
                 try:
                     col[idx0:idx1] = col.info.mask_val
                 except Exception:
@@ -1374,7 +1379,7 @@ def _hstack(arrays, join_type='outer', uniq_col_name='{col_name}_{table_name}',
     _col_name_map = col_name_map
 
     if table_names is None:
-        table_names = ['{}'.format(ii + 1) for ii in range(len(arrays))]
+        table_names = [f'{ii + 1}' for ii in range(len(arrays))]
     if len(arrays) != len(table_names):
         raise ValueError('Number of arrays must match number of table_names')
 
@@ -1422,6 +1427,9 @@ def _hstack(arrays, join_type='outer', uniq_col_name='{col_name}_{table_name}',
                 if isinstance(col, Column) and not isinstance(col, MaskedColumn):
                     col = out.MaskedColumn(col, copy=False)
 
+                if isinstance(col, Quantity) and not isinstance(col, Masked):
+                    col = Masked(col, copy=False)
+
                 try:
                     col[arr_len:] = col.info.mask_val
                 except Exception:
