@@ -9,18 +9,7 @@ from astropy.time import Time
 from astropy.table.table import Table
 from astropy import extern
 
-try:
-    import bleach  # noqa
-    HAS_BLEACH = True
-except ImportError:
-    HAS_BLEACH = False
-
-try:
-    import IPython  # noqa
-except ImportError:
-    HAS_IPYTHON = False
-else:
-    HAS_IPYTHON = True
+from astropy.utils.compat.optional_deps import HAS_BLEACH, HAS_IPYTHON  # noqa
 
 EXTERN_DIR = abspath(join(dirname(extern.__file__), 'jquery', 'data'))
 
@@ -106,7 +95,7 @@ def test_write_jsviewer_default(tmpdir):
     ref = REFERENCE % dict(
         lines=format_lines(t['a'], t['b']),
         table_class='display compact',
-        table_id='table%s' % id(t),
+        table_id=f'table{id(t)}',
         length='50',
         display_length='10, 25, 50, 100, 500, 1000',
         datatables_css_url='https://cdn.datatables.net/1.10.12/css/jquery.dataTables.css',
@@ -133,7 +122,7 @@ def test_write_jsviewer_mixin(tmpdir, mixin):
     ref = REFERENCE % dict(
         lines=format_lines(t['a'], t['b']),
         table_class='display compact',
-        table_id='table%s' % id(t),
+        table_id=f'table{id(t)}',
         length='50',
         display_length='10, 25, 50, 100, 500, 1000',
         datatables_css_url='https://cdn.datatables.net/1.10.12/css/jquery.dataTables.css',
