@@ -1,18 +1,6 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-try:
-    import h5py  # pylint: disable=W0611 # noqa
-except ImportError:
-    HAS_H5PY = False
-else:
-    HAS_H5PY = True
-
-try:
-    import yaml  # pylint: disable=W0611 # noqa
-    HAS_YAML = True
-except ImportError:
-    HAS_YAML = False
 
 import copy
 import pickle
@@ -21,6 +9,9 @@ from io import StringIO
 import pytest
 import numpy as np
 
+from astropy.table.serialize import represent_mixins_as_columns
+from astropy.utils.data_info import ParentDtypeInfo
+from astropy.table.table_helpers import ArrayWrapper
 from astropy.coordinates import EarthLocation, SkyCoord
 from astropy.table import Table, QTable, join, hstack, vstack, Column, NdarrayMixin
 from astropy.table import serialize
@@ -33,6 +24,7 @@ from astropy.utils.exceptions import AstropyUserWarning
 from astropy.utils.metadata import MergeConflictWarning
 from astropy.coordinates.tests.test_representation import representation_equal
 from astropy.io.misc.asdf.tags.helpers import skycoord_equal
+from astropy.utils.compat.optional_deps import HAS_YAML
 
 from .conftest import MIXIN_COLS
 
@@ -49,7 +41,7 @@ def test_attributes(mixin_cols):
     assert m.info.description == 'a'
 
     # Cannot set unit for these classes
-    if isinstance(m, (u.Quantity, coordinates.SkyCoord, time.Time,
+    if isinstance(m, (u.Quantity, coordinates.SkyCoord, time.Time, time.TimeDelta,
                       coordinates.BaseRepresentationOrDifferential)):
         with pytest.raises(AttributeError):
             m.info.unit = u.m
@@ -124,13 +116,13 @@ def test_votable_quantity_write(tmpdir):
     (io/fits/tests/test_connect.py and io/misc/tests/test_hdf5.py).
     """
     t = QTable()
-    t['a'] = u.Quantity([1, 2, 4], unit='Angstrom')
+    t['a'] = u.Quantity([1, 2, 4], unit='nm')
 
     filename = str(tmpdir.join('table-tmp'))
     t.write(filename, format='votable', overwrite=True)
     qt = QTable.read(filename, format='votable')
     assert isinstance(qt['a'], u.Quantity)
-    assert qt['a'].unit == 'Angstrom'
+    assert qt['a'].unit == 'nm'
 
 
 @pytest.mark.remote_data
@@ -440,13 +432,15 @@ def test_info_preserved_pickle_copy_init(mixin_cols):
         for func in (copy.copy, copy.deepcopy, pickle_roundtrip, init_from_class):
             m2 = func(m)
             for attr in attrs:
+                # non-native byteorder not preserved by last 2 func, _except_ for structured dtype
                 if (attr != 'dtype'
                         or getattr(m.info.dtype, 'isnative', True)
+                        or m.info.dtype.name.startswith('void')
                         or func in (copy.copy, copy.deepcopy)):
                     original = getattr(m.info, attr)
                 else:
-                    # func does not preserve byteorder, check against (native) base type.
-                    original = m.info.dtype.name
+                    # func does not preserve byteorder, check against (native) type.
+                    original = m.info.dtype.newbyteorder('=')
                 assert getattr(m2.info, attr) == original
 
 
@@ -521,7 +515,7 @@ def test_insert_row(mixin_cols):
     t0 = t.copy()
     t['m'].info.description = 'd'
     idxs = [0, -1, 1, 2, 3]
-    if isinstance(t['m'], (u.Quantity, Column, time.Time, coordinates.SkyCoord)):
+    if isinstance(t['m'], (u.Quantity, Column, time.Time, time.TimeDelta, coordinates.SkyCoord)):
         t.insert_row(1, t[-1])
 
         for name in t.colnames:
@@ -545,7 +539,7 @@ def test_insert_row_bad_unit():
     t = QTable([[1] * u.m])
     with pytest.raises(ValueError) as exc:
         t.insert_row(0, (2 * u.m / u.s,))
-    assert "'m / s' (speed) and 'm' (length) are not convertible" in str(exc.value)
+    assert "'m / s' (speed/velocity) and 'm' (length) are not convertible" in str(exc.value)
 
 
 def test_convert_np_array(mixin_cols):
@@ -836,3 +830,39 @@ def test_skycoord_with_velocity():
     s.seek(0)
     t2 = Table.read(s.read(), format='ascii.ecsv')
     assert skycoord_equal(t2['col0'], sc)
+
+
+@pytest.mark.parametrize('table_cls', [Table, QTable])
+def test_ensure_input_info_is_unchanged(table_cls):
+    """If a mixin input to a table has no info, it should stay that way.
+
+    This since having 'info' slows down slicing, etc.
+    See gh-11066.
+    """
+    q = [1, 2] * u.m
+    assert 'info' not in q.__dict__
+    t = table_cls([q], names=['q'])
+    assert 'info' not in q.__dict__
+    t = table_cls([q])
+    assert 'info' not in q.__dict__
+    t = table_cls({'q': q})
+    assert 'info' not in q.__dict__
+    t['q2'] = q
+    assert 'info' not in q.__dict__
+    sc = SkyCoord([1, 2], [2, 3], unit='deg')
+    t['sc'] = sc
+    assert 'info' not in sc.__dict__
+
+
+def test_bad_info_class():
+    """Make a mixin column class that does not trigger the machinery to generate
+    a pure column representation"""
+    class MyArrayWrapper(ArrayWrapper):
+        info = ParentDtypeInfo()
+
+    t = Table()
+    t['tm'] = MyArrayWrapper([0, 1, 2])
+    out = StringIO()
+    match = r"failed to represent column 'tm' \(MyArrayWrapper\) as one or more Column subclasses"
+    with pytest.raises(TypeError, match=match):
+        represent_mixins_as_columns(t)
