@@ -1,18 +1,21 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+from astropy.table.table_helpers import ArrayWrapper
+from astropy.coordinates.earth import EarthLocation
+from astropy.units.quantity import Quantity
 from collections import OrderedDict
 
 import pytest
 import numpy as np
 
-from astropy.table import Table, QTable, TableMergeError, Column, MaskedColumn
+from astropy.table import Table, QTable, TableMergeError, Column, MaskedColumn, NdarrayMixin
 from astropy.table.operations import _get_out_class, join_skycoord, join_distance
 from astropy import units as u
 from astropy.utils import metadata
 from astropy.utils.metadata import MergeConflictError
 from astropy.utils.compat.context import nullcontext
 from astropy import table
-from astropy.time import Time
+from astropy.time import Time, TimeDelta
 from astropy.coordinates import (SkyCoord, SphericalRepresentation,
                                  UnitSphericalRepresentation,
                                  CartesianRepresentation,
@@ -20,18 +23,29 @@ from astropy.coordinates import (SkyCoord, SphericalRepresentation,
                                  search_around_3d)
 from astropy.coordinates.tests.test_representation import representation_equal
 from astropy.io.misc.asdf.tags.helpers import skycoord_equal
-
-try:
-    import scipy  # noqa
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 
 def sort_eq(list1, list2):
     return sorted(list1) == sorted(list2)
 
 
+def check_mask(col, exp_mask):
+    """Check that col.mask == exp_mask"""
+    if hasattr(col, 'mask'):
+        # Coerce expected mask into dtype of col.mask. In particular this is
+        # needed for types like EarthLocation where the mask is a structured
+        # array.
+        exp_mask = np.array(exp_mask).astype(col.mask.dtype)
+        out = np.all(col.mask == exp_mask)
+    else:
+        # With no mask the check is OK if all the expected mask values
+        # are False (i.e. no auto-conversion to MaskedQuantity if it was
+        # not required by the join).
+        out = np.all(exp_mask == False)
+    return out
+
+
 class TestJoin():
 
     def _setup(self, t_cls=Table):
@@ -523,37 +537,37 @@ class TestJoin():
             assert np.all(out['m1'] == col[[0, 3]])
             assert np.all(out['m2'] == col[[0, 3]])
 
-        # Check for left, right, outer join which requires masking.  Only Time
-        # supports this currently.
-        if cls_name == 'Time':
+        # Check for left, right, outer join which requires masking. Works for
+        # the listed mixins classes.
+        if isinstance(col, (Quantity, Time, TimeDelta)):
             out = table.join(t1, t2, join_type='left')
             assert len(out) == 3
             assert np.all(out['idx'] == [0, 1, 3])
             assert np.all(out['m1'] == t1['m1'])
             assert np.all(out['m2'] == t2['m2'])
-            assert np.all(out['m1'].mask == [False, False, False])
-            assert np.all(out['m2'].mask == [False, True, False])
+            check_mask(out['m1'], [False, False, False])
+            check_mask(out['m2'], [False, True, False])
 
             out = table.join(t1, t2, join_type='right')
             assert len(out) == 3
             assert np.all(out['idx'] == [0, 2, 3])
             assert np.all(out['m1'] == t1['m1'])
             assert np.all(out['m2'] == t2['m2'])
-            assert np.all(out['m1'].mask == [False, True, False])
-            assert np.all(out['m2'].mask == [False, False, False])
+            check_mask(out['m1'], [False, True, False])
+            check_mask(out['m2'], [False, False, False])
 
             out = table.join(t1, t2, join_type='outer')
             assert len(out) == 4
             assert np.all(out['idx'] == [0, 1, 2, 3])
             assert np.all(out['m1'] == col)
             assert np.all(out['m2'] == col)
-            assert np.all(out['m1'].mask == [False, False, True, False])
-            assert np.all(out['m2'].mask == [False, True, False, False])
+            assert check_mask(out['m1'], [False, False, True, False])
+            assert check_mask(out['m2'], [False, True, False, False])
         else:
             # Otherwise make sure it fails with the right exception message
             for join_type in ('outer', 'left', 'right'):
                 with pytest.raises(NotImplementedError) as err:
-                    table.join(t1, t2, join_type='outer')
+                    table.join(t1, t2, join_type=join_type)
                 assert ('join requires masking' in str(err.value)
                         or 'join unavailable' in str(err.value))
 
@@ -1121,7 +1135,7 @@ class TestVStack():
         cls_name = type(col).__name__
 
         # Vstack works for these classes:
-        if isinstance(col, (u.Quantity, Time, SkyCoord,
+        if isinstance(col, (u.Quantity, Time, TimeDelta, SkyCoord, EarthLocation,
                             BaseRepresentationOrDifferential)):
             out = table.vstack([t, t])
             assert len(out) == len_col * 2
@@ -1144,13 +1158,13 @@ class TestVStack():
         # Check for outer stack which requires masking.  Only Time supports
         # this currently.
         t2 = table.QTable([col], names=['b'])  # different from col name for t
-        if cls_name == 'Time':
+        if isinstance(col, (Time, TimeDelta, Quantity)):
             out = table.vstack([t, t2], join_type='outer')
             assert len(out) == len_col * 2
             assert np.all(out['a'][:len_col] == col)
             assert np.all(out['b'][len_col:] == col)
-            assert np.all(out['a'].mask == [False] * len_col + [True] * len_col)
-            assert np.all(out['b'].mask == [True] * len_col + [False] * len_col)
+            assert check_mask(out['a'], [False] * len_col + [True] * len_col)
+            assert check_mask(out['b'], [True] * len_col + [False] * len_col)
             # check directly stacking mixin columns:
             out2 = table.vstack([t, t2['b']])
             assert np.all(out['a'] == out2['a'])
@@ -1562,12 +1576,12 @@ class TestHStack():
             assert np.all(out['col0_2'] == col2)
 
         # Time class supports masking, all other mixins do not
-        if cls_name == 'Time':
+        if isinstance(col1, (Time, TimeDelta, Quantity)):
             out = table.hstack([t1, t2], join_type='outer')
             assert len(out) == len(t1)
             assert np.all(out['col0_1'] == col1)
             assert np.all(out['col0_2'][:len(col2)] == col2)
-            assert np.all(out['col0_2'].mask == [False, False, True, True])
+            assert check_mask(out['col0_2'], [False, False, True, True])
 
             # check directly stacking mixin columns:
             out2 = table.hstack([t1, t2['col0']], join_type='outer')
@@ -1834,13 +1848,13 @@ def test_masking_required_exception():
     Test that outer join, hstack and vstack fail for a mixin column which
     does not support masking.
     """
-    col = [1, 2, 3, 4] * u.m
+    col = table.NdarrayMixin([0, 1, 2, 3])
     t1 = table.QTable([[1, 2, 3, 4], col], names=['a', 'b'])
     t2 = table.QTable([[1, 2], col[:2]], names=['a', 'c'])
 
     with pytest.raises(NotImplementedError) as err:
         table.vstack([t1, t2], join_type='outer')
-    assert 'vstack requires masking' in str(err.value)
+    assert 'vstack unavailable' in str(err.value)
 
     with pytest.raises(NotImplementedError) as err:
         table.hstack([t1, t2], join_type='outer')
