@@ -1,7 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
 This module provides the tools used to internally run the astropy test suite
-from the installed astropy.  It makes use of the `pytest` testing framework.
+from the installed astropy.  It makes use of the `pytest`_ testing framework.
 """
 import os
 import sys
@@ -12,14 +12,6 @@ import functools
 
 import pytest
 
-try:
-    # Import pkg_resources to prevent it from issuing warnings upon being
-    # imported from within pytest.  See
-    # https://github.com/astropy/astropy/pull/537 for a detailed explanation.
-    import pkg_resources  # pylint: disable=W0611  # noqa
-except ImportError:
-    pass
-
 from astropy.units import allclose as quantity_allclose  # noqa
 from astropy.utils.exceptions import (AstropyDeprecationWarning,
                                       AstropyPendingDeprecationWarning)
