@@ -199,11 +199,6 @@ class TestRunnerBase:
             for egg in glob.glob(os.path.join('.eggs', '*.egg')):
                 sys.path.insert(0, egg)
 
-            # We now need to force reload pkg_resources in case any pytest
-            # plugins were added above, so that their entry points are picked up
-            import pkg_resources
-            importlib.reload(pkg_resources)
-
         self._has_test_dependencies()  # pragma: no cover
 
         # The docstring for this method is defined as a class variable.
@@ -217,7 +212,7 @@ class TestRunnerBase:
         passed_kwargs = set(kwargs.keys())
         if not passed_kwargs.issubset(allowed_kwargs):
             wrong_kwargs = list(passed_kwargs.difference(allowed_kwargs))
-            raise TypeError("run_tests() got an unexpected keyword argument {}".format(wrong_kwargs[0]))
+            raise TypeError(f"run_tests() got an unexpected keyword argument {wrong_kwargs[0]}")
 
         args = self._generate_args(**kwargs)
 
