@@ -348,10 +348,10 @@ class TimeFormat(metaclass=TimeFormatMeta):
 
         Parameters
         ----------
-        parent : obj
+        parent : object
             Parent `~astropy.time.Time` object associated with this
             `~astropy.time.TimeFormat` object
-        out_subfmt : str or `None`
+        out_subfmt : str or None
             Output subformt (use existing self.out_subfmt if `None`)
 
         Returns
@@ -1186,7 +1186,7 @@ class TimezoneInfo(datetime.tzinfo):
         dst : `~astropy.units.Quantity`, optional
             Daylight Savings Time offset in days. Defaults to zero
             (no daylight savings).
-        tzname : str or `None`, optional
+        tzname : str or None, optional
             Name of timezone
 
         Examples
@@ -1299,8 +1299,7 @@ class TimeString(TimeUnique):
 
     def _check_val_type(self, val1, val2):
         if val1.dtype.kind not in ('S', 'U') and val1.size:
-            raise TypeError('Input values for {} class must be strings'
-                            .format(self.name))
+            raise TypeError(f'Input values for {self.name} class must be strings')
         if val2 is not None:
             raise ValueError(
                 f'{self.name} objects do not accept a val2 but you provided {val2}')
@@ -1344,8 +1343,7 @@ class TimeString(TimeUnique):
             vals[-1] = vals[-1] + fracsec
             return vals
         else:
-            raise ValueError('Time {} does not match {} format'
-                             .format(timestr, self.name))
+            raise ValueError(f'Time {timestr} does not match {self.name} format')
 
     def set_jds(self, val1, val2):
         """Parse the time strings contained in val1 and set jd1, jd2"""
@@ -1740,8 +1738,7 @@ class TimeFITS(TimeString):
             if tm:
                 break
         else:
-            raise ValueError('Time {} does not match {} format'
-                             .format(timestr, self.name))
+            raise ValueError(f'Time {timestr} does not match {self.name} format')
         tm = tm.groupdict()
         # Scale and realization are deprecated and strings in this form
         # are no longer created.  We issue a warning but still use the value.
@@ -1847,8 +1844,7 @@ class TimeEpochDateString(TimeString):
                 if epoch_type.upper() != epoch_prefix:
                     raise ValueError
             except (IndexError, ValueError, UnicodeEncodeError):
-                raise ValueError('Time {} does not match {} format'
-                                 .format(time_str, self.name))
+                raise ValueError(f'Time {time_str} does not match {self.name} format')
             else:
                 years[...] = year
 
