@@ -14,6 +14,7 @@ from astropy.utils.misc import isiterable
 from . import si
 from . import cgs
 from . import astrophys
+from . import misc
 from .function import units as function_units
 from . import dimensionless_unscaled
 from .core import UnitsError, Unit
@@ -24,7 +25,7 @@ __all__ = ['parallax', 'spectral', 'spectral_density', 'doppler_radio',
            'brightness_temperature', 'thermodynamic_temperature',
            'beam_angular_area', 'dimensionless_angles', 'logarithmic',
            'temperature', 'temperature_energy', 'molar_mass_amu',
-           'pixel_scale', 'plate_scale', 'with_H0']
+           'pixel_scale', 'plate_scale', 'with_H0', "Equivalency"]
 
 
 class Equivalency(UserList):
@@ -105,7 +106,7 @@ def parallax():
 def spectral():
     """
     Returns a list of equivalence pairs that handle spectral
-    wavelength, wave number, frequency, and energy equivalences.
+    wavelength, wave number, frequency, and energy equivalencies.
 
     Allows conversions between wavelength units, wave number units,
     frequency units, and energy units as they relate to light.
@@ -183,7 +184,7 @@ def spectral_density(wav, factor=None):
     phot_L_la = astrophys.photon / (si.s * si.AA)
     phot_L_nu = astrophys.photon / (si.s * si.Hz)
 
-    # surface brigthness (flux equiv)
+    # surface brightness (flux equiv)
     S_la = cgs.erg / si.angstrom / si.cm ** 2 / si.s / si.sr
     S_nu = cgs.erg / si.Hz / si.cm ** 2 / si.s / si.sr
     nu_S_nu = cgs.erg / si.cm ** 2 / si.s / si.sr
@@ -512,7 +513,7 @@ def molar_mass_amu():
     Returns the equivalence between amu and molar mass.
     """
     return Equivalency([
-        (si.g/si.mol, astrophys.u)
+        (si.g/si.mol, misc.u)
     ], "molar_mass_amu")
 
 
@@ -552,14 +553,14 @@ def brightness_temperature(frequency, beam_area=None):
 
     Parameters
     ----------
-    frequency : `~astropy.units.Quantity` with spectral units
+    frequency : `~astropy.units.Quantity`
         The observed ``spectral`` equivalent `~astropy.units.Unit` (e.g.,
         frequency or wavelength).  The variable is named 'frequency' because it
         is more commonly used in radio astronomy.
         BACKWARD COMPATIBILITY NOTE: previous versions of the brightness
         temperature equivalency used the keyword ``disp``, which is no longer
         supported.
-    beam_area : angular area equivalent
+    beam_area : `~astropy.units.Quantity` ['solid angle']
         Beam area in angular units, i.e. steradian equivalent
 
     Examples
@@ -640,8 +641,9 @@ def beam_angular_area(beam_area):
 
     Parameters
     ----------
-    beam_area : angular area equivalent
+    beam_area : unit-like
         The area of the beam in angular area units (e.g., steradians)
+        Must have angular area equivalent units.
     """
     return Equivalency([(astrophys.beam, Unit(beam_area)),
                         (astrophys.beam**-1, Unit(beam_area)**-1),
@@ -661,12 +663,12 @@ def thermodynamic_temperature(frequency, T_cmb=None):
 
     Parameters
     ----------
-    frequency : `~astropy.units.Quantity` with spectral units
+    frequency : `~astropy.units.Quantity`
         The observed `spectral` equivalent `~astropy.units.Unit` (e.g.,
-        frequency or wavelength)
-    T_cmb :  `~astropy.units.Quantity` with temperature units or None
+        frequency or wavelength). Must have spectral units.
+    T_cmb :  `~astropy.units.Quantity` ['temperature'] or None
         The CMB temperature at z=0.  If `None`, the default cosmology will be
-        used to get this temperature.
+        used to get this temperature. Must have units of temperature.
 
     Notes
     -----
@@ -755,18 +757,18 @@ def pixel_scale(pixscale):
 
     decomposed = pixscale.unit.decompose()
     dimensions = dict(zip(decomposed.bases, decomposed.powers))
-    pix_power = dimensions.get(astrophys.pix, 0)
+    pix_power = dimensions.get(misc.pix, 0)
 
     if pix_power == -1:
-        physical_unit = Unit(pixscale * astrophys.pix)
+        physical_unit = Unit(pixscale * misc.pix)
     elif pix_power == 1:
-        physical_unit = Unit(astrophys.pix / pixscale)
+        physical_unit = Unit(misc.pix / pixscale)
     else:
         raise UnitsError(
                 "The pixel scale unit must have"
                 " pixel dimensionality of 1 or -1.")
 
-    return Equivalency([(astrophys.pix, physical_unit)],
+    return Equivalency([(misc.pix, physical_unit)],
                        "pixel_scale", {'pixscale': pixscale})
 
 
@@ -798,7 +800,7 @@ def with_H0(H0=None):
 
     Parameters
     ----------
-    H0 : `None` or `~astropy.units.Quantity`
+    H0 : None or `~astropy.units.Quantity` ['frequency']
         The value of the Hubble constant to assume. If a `~astropy.units.Quantity`,
         will assume the quantity *is* ``H0``.  If `None` (default), use the
         ``H0`` attribute from the default `astropy.cosmology` cosmology.
