@@ -2,6 +2,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Converters for Quantity."""
 
+import threading
+
 import numpy as np
 
 from astropy.units.core import (UnitsError, UnitConversionError, UnitTypeError,
@@ -19,7 +21,12 @@ class UfuncHelpers(dict):
 
     Such modules should be registered using ``register_module``.
     """
-    UNSUPPORTED = set()
+
+    def __init__(self, *args, **kwargs):
+        self.modules = {}
+        self.UNSUPPORTED = set()   # Upper-case for backwards compatibility
+        self._lock = threading.RLock()
+        super().__init__(*args, **kwargs)
 
     def register_module(self, module, names, importer):
         """Register (but do not import) a set of ufunc helpers.
@@ -35,15 +42,9 @@ class UfuncHelpers(dict):
             keyed by those ufuncs.  If the value is `None`, the ufunc is
             explicitly *not* supported.
         """
-        self.modules[module] = {'names': names,
-                                'importer': importer}
-
-    @property
-    def modules(self):
-        """Modules for which helpers are available (but not yet loaded)."""
-        if not hasattr(self, '_modules'):
-            self._modules = {}
-        return self._modules
+        with self._lock:
+            self.modules[module] = {'names': names,
+                                    'importer': importer}
 
     def import_module(self, module):
         """Import the helpers from the given module using its helper function.
@@ -53,8 +54,9 @@ class UfuncHelpers(dict):
         module : str
             Name of the module. Has to have been registered beforehand.
         """
-        module_info = self.modules.pop(module)
-        self.update(module_info['importer']())
+        with self._lock:
+            module_info = self.modules.pop(module)
+            self.update(module_info['importer']())
 
     def __missing__(self, ufunc):
         """Called if a ufunc is not found.
@@ -62,26 +64,30 @@ class UfuncHelpers(dict):
         Check if the ufunc is in any of the available modules, and, if so,
         import the helpers for that module.
         """
-        if ufunc in self.UNSUPPORTED:
-            raise TypeError("Cannot use ufunc '{}' with quantities"
-                            .format(ufunc.__name__))
+        with self._lock:
+            # Check if it was loaded while we waited for the lock
+            if ufunc in self:
+                return self[ufunc]
 
-        for module, module_info in list(self.modules.items()):
-            if ufunc.__name__ in module_info['names']:
-                # A ufunc with the same name is supported by this module.
-                # Of course, this doesn't necessarily mean it is the
-                # right module. So, we try let the importer do its work.
-                # If it fails (e.g., for `scipy.special`), then that's
-                # fine, just raise the TypeError.  If it succeeds, but
-                # the ufunc is not found, that is also fine: we will
-                # enter __missing__ again and either find another
-                # module or get the TypeError there.
-                try:
-                    self.import_module(module)
-                except ImportError:
-                    pass
-                else:
-                    return self[ufunc]
+            if ufunc in self.UNSUPPORTED:
+                raise TypeError(f"Cannot use ufunc '{ufunc.__name__}' with quantities")
+
+            for module, module_info in list(self.modules.items()):
+                if ufunc.__name__ in module_info['names']:
+                    # A ufunc with the same name is supported by this module.
+                    # Of course, this doesn't necessarily mean it is the
+                    # right module. So, we try let the importer do its work.
+                    # If it fails (e.g., for `scipy.special`), then that's
+                    # fine, just raise the TypeError.  If it succeeds, but
+                    # the ufunc is not found, that is also fine: we will
+                    # enter __missing__ again and either find another
+                    # module or get the TypeError there.
+                    try:
+                        self.import_module(module)
+                    except ImportError:  # pragma: no cover
+                        pass
+                    else:
+                        return self[ufunc]
 
         raise TypeError("unknown ufunc {}.  If you believe this ufunc "
                         "should be supported, please raise an issue on "
@@ -93,12 +99,13 @@ class UfuncHelpers(dict):
         # mean that something is not implemented, but this means an
         # extra if clause for the output, slowing down the common
         # path where a ufunc is supported.
-        if value is None:
-            self.UNSUPPORTED |= {key}
-            self.pop(key, None)
-        else:
-            super().__setitem__(key, value)
-            self.UNSUPPORTED -= {key}
+        with self._lock:
+            if value is None:
+                self.UNSUPPORTED |= {key}
+                self.pop(key, None)
+            else:
+                super().__setitem__(key, value)
+                self.UNSUPPORTED -= {key}
 
 
 UFUNC_HELPERS = UfuncHelpers()
@@ -117,7 +124,8 @@ def can_have_arbitrary_unit(value):
 
     Returns
     -------
-    `True` if each member is either zero or not finite, `False` otherwise
+    bool
+        `True` if each member is either zero or not finite, `False` otherwise
     """
     return np.all(np.logical_or(np.equal(value, 0.), ~np.isfinite(value)))
 
@@ -137,7 +145,7 @@ def converters_and_unit(function, method, *args):
     method : str
         Method with which the function is evaluated, e.g.,
         '__call__', 'reduce', etc.
-    *args : Quantity or other ndarray subclass
+    *args :  `~astropy.units.Quantity` or ndarray subclass
         Input arguments to the function
 
     Raises
@@ -245,8 +253,7 @@ def converters_and_unit(function, method, *args):
         else:
             if method in {'reduce', 'accumulate',
                           'reduceat', 'outer'} and nin != 2:
-                raise ValueError("{} only supported for binary functions"
-                                 .format(method))
+                raise ValueError(f"{method} only supported for binary functions")
 
             raise TypeError("Unexpected ufunc method {}.  If this should "
                             "work, please raise an issue on"
@@ -291,7 +298,8 @@ def check_output(output, unit, inputs, function=None):
 
     Returns
     -------
-    arrays : `~numpy.ndarray` view of ``output`` (or tuple of such views).
+    arrays : ndarray view or tuple thereof
+        The view(s) is of ``output``.
 
     Raises
     ------
@@ -318,13 +326,14 @@ def check_output(output, unit, inputs, function=None):
                                  if function is not None else ""),
                                 type(output)))
 
-        if output.__quantity_subclass__(unit)[0] is not type(output):
+        q_cls, subok = output.__quantity_subclass__(unit)
+        if not (subok or q_cls is type(output)):
             raise UnitTypeError(
                 "Cannot store output with unit '{}'{} "
                 "in {} instance.  Use {} instance instead."
                 .format(unit, (f" from {function.__name__} function"
-                               if function is not None else ""), type(output),
-                        output.__quantity_subclass__(unit)[0]))
+                               if function is not None else ""),
+                        type(output), q_cls))
 
         # check we can handle the dtype (e.g., that we are not int
         # when float is required).  Note that we only do this for Quantity
