@@ -1,6 +1,8 @@
 # The purpose of these tests are to ensure that calling ufuncs with quantities
 # returns quantities with the right units, or raises exceptions.
 
+import concurrent.futures
+import sys
 import warnings
 from collections import namedtuple
 
@@ -11,14 +13,9 @@ from erfa import ufunc as erfa_ufunc
 
 from astropy import units as u
 from astropy.units import quantity_helper as qh
-
-
-try:
-    import scipy  # pylint: disable=W0611 # noqa
-except ImportError:
-    HAS_SCIPY = False
-else:
-    HAS_SCIPY = True
+from astropy.units.quantity_helper.converters import UfuncHelpers
+from astropy.units.quantity_helper.helpers import helper_sqrt
+from astropy.utils.compat.optional_deps import HAS_SCIPY  # noqa
 
 
 testcase = namedtuple('testcase', ['f', 'q_in', 'q_out'])
@@ -88,6 +85,26 @@ class TestUfuncHelpers:
         assert np.add in qh.UFUNC_HELPERS
         assert np.add not in qh.UNSUPPORTED_UFUNCS
 
+    def test_thread_safety(self, fast_thread_switching):
+        def dummy_ufunc(*args, **kwargs):
+            return np.sqrt(*args, **kwargs)
+
+        def register():
+            return {dummy_ufunc: helper_sqrt}
+
+        workers = 8
+        with concurrent.futures.ThreadPoolExecutor(max_workers=workers) as executor:
+            for p in range(10000):
+                helpers = UfuncHelpers()
+                helpers.register_module(
+                    'astropy.units.tests.test_quantity_ufuncs',
+                    ['dummy_ufunc'],
+                    register
+                )
+                futures = [executor.submit(lambda: helpers[dummy_ufunc]) for i in range(workers)]
+                values = [future.result() for future in futures]
+                assert values == [helper_sqrt] * workers
+
 
 class TestQuantityTrigonometricFuncs:
     """
