@@ -13,20 +13,6 @@ from astropy import constants as c
 from astropy.units import utils
 
 
-def test_getting_started():
-    """
-    Corresponds to "Getting Started" section in the docs.
-    """
-    from astropy.units import imperial
-    with imperial.enable():
-        speed_unit = u.cm / u.s
-        x = speed_unit.to(imperial.mile / u.hour, 1)
-        assert_allclose(x, 0.02236936292054402)
-        speed_converter = speed_unit._get_converter("mile hour^-1")
-        x = speed_converter([1., 1000., 5000.])
-        assert_allclose(x, [2.23693629e-02, 2.23693629e+01, 1.11846815e+02])
-
-
 def test_initialisation():
     assert u.Unit(u.m) is u.m
 
@@ -45,6 +31,8 @@ def test_initialisation():
     assert u.Unit('10 m') == ten_meter
     assert u.Unit(10.) == u.CompositeUnit(10., [], [])
 
+    assert u.Unit() == u.dimensionless_unscaled
+
 
 def test_invalid_power():
     x = u.m ** Fraction(1, 3)
@@ -232,11 +220,6 @@ def test_unrecognized_equivalency():
     assert u.m.is_equivalent('pc') is True
 
 
-def test_unit_noarg():
-    with pytest.raises(TypeError):
-        u.Unit()
-
-
 def test_convertible_exception():
     with pytest.raises(u.UnitsError, match=r'length.+ are not convertible'):
         u.AA.to(u.h * u.s ** 2)
