@@ -14,6 +14,7 @@ import struct
 import sys
 import threading
 import time
+from concurrent.futures import ProcessPoolExecutor, as_completed
 
 try:
     import fcntl
@@ -87,13 +88,6 @@ class _IPython:
                 cls._ipyio = io
         return cls._ipyio
 
-    @classproperty
-    def IOStream(cls):
-        if cls.ipyio is None:
-            return None
-        else:
-            return cls.ipyio.IOStream
-
     @classmethod
     def get_stream(cls, stream):
         return getattr(cls.ipyio, stream)
@@ -115,19 +109,7 @@ def _get_stdout(stderr=False):
         stream = 'stdout'
 
     sys_stream = getattr(sys, stream)
-    if not isatty(sys_stream) or _IPython.OutStream is None:
-        return sys_stream
-
-    # Our system stream is an atty and we're in ipython.
-    ipyio_stream = _IPython.get_stream(stream)
-
-    if ipyio_stream is not None and isatty(ipyio_stream):
-        # Use the IPython console output stream
-        return ipyio_stream
-    else:
-        # sys.stdout was set to some other non-TTY stream (a file perhaps)
-        # so just use it directly
-        return sys_stream
+    return sys_stream
 
 
 def isatty(file):
@@ -139,25 +121,24 @@ def isatty(file):
     ttys.
     """
     if (multiprocessing.current_process().name != 'MainProcess' or
-        threading.current_thread().getName() != 'MainThread'):
+            threading.current_thread().getName() != 'MainThread'):
         return False
 
     if hasattr(file, 'isatty'):
         return file.isatty()
 
-    # Use two isinstance calls to only evaluate IOStream when necessary.
-    if (_IPython.OutStream is None or
-        (not isinstance(file, _IPython.OutStream) and
-         not isinstance(file, _IPython.IOStream))):
+    if _IPython.OutStream is None or (not isinstance(file, _IPython.OutStream)):
         return False
 
-    # File is an IPython OutStream or IOStream.  Check whether:
+    # File is an IPython OutStream. Check whether:
     # - File name is 'stdout'; or
     # - File wraps a Console
     if getattr(file, 'name', None) == 'stdout':
         return True
 
     if hasattr(file, 'stream'):
+        # FIXME: pyreadline has no had new release since 2015, drop it when
+        #        IPython minversion is 5.x.
         # On Windows, in IPython 2 the standard I/O streams will wrap
         # pyreadline.Console objects if pyreadline is available; this should
         # be considered a TTY.
@@ -185,9 +166,9 @@ def terminal_size(file=None):
         file = _get_stdout()
 
     try:
-        s = struct.pack(str("HHHH"), 0, 0, 0, 0)
+        s = struct.pack("HHHH", 0, 0, 0, 0)
         x = fcntl.ioctl(file, termios.TIOCGWINSZ, s)
-        (lines, width, xpixels, ypixels) = struct.unpack(str("HHHH"), x)
+        (lines, width, xpixels, ypixels) = struct.unpack("HHHH", x)
         if lines > 12:
             lines -= 6
         if width > 10:
@@ -256,7 +237,7 @@ def _color_text(text, color):
         return text
 
     color_code = color_mapping.get(color, '0;39')
-    return '\033[{0}m{1}\033[0m'.format(color_code, text)
+    return f'\033[{color_code}m{text}\033[0m'
 
 
 def _decode_preferred_encoding(s):
@@ -283,15 +264,6 @@ def _write_with_fallback(s, write, fileobj):
     of a UnicodeEncodeError.  Failing that attempt to write with 'utf-8' or
     'latin-1'.
     """
-    if (_IPython.IOStream is not None and
-        isinstance(fileobj, _IPython.IOStream)):
-        # If the output stream is an IPython.utils.io.IOStream object that's
-        # not going to be very helpful to us since it doesn't raise any
-        # exceptions when an error occurs writing to its underlying stream.
-        # There's no advantage to us using IOStream.write directly though;
-        # instead just write directly to its underlying stream:
-        write = fileobj.stream.write
-
     try:
         write(s)
         return write
@@ -342,7 +314,7 @@ def color_print(*args, end='\n', **kwargs):
         default, darkgrey, lightred, lightgreen, yellow, lightblue,
         lightmagenta, lightcyan, white, or '' (the empty string).
 
-    file : writeable file-like object, optional
+    file : writable file-like, optional
         Where to write to.  Defaults to `sys.stdout`.  If file is not
         a tty (as determined by calling its `isatty` member, if one
         exists), no coloring will be included.
@@ -425,12 +397,12 @@ def human_time(seconds):
     seconds = int(seconds)
 
     if seconds < 60:
-        return '   {0:2d}s'.format(seconds)
+        return f'   {seconds:2d}s'
     for i in range(len(units) - 1):
         unit1, limit1 = units[i]
         unit2, limit2 = units[i + 1]
         if seconds >= limit1:
-            return '{0:2d}{1}{2:2d}{3}'.format(
+            return '{:2d}{}{:2d}{}'.format(
                 seconds // limit1, unit1,
                 (seconds % limit1) // limit2, unit2)
     return '  ~inf'
@@ -482,7 +454,7 @@ def human_file_size(size):
         str_value = str_value[:2]
     else:
         str_value = str_value[:3]
-    return "{0:>3s}{1}".format(str_value, suffix)
+    return f"{str_value:>3s}{suffix}"
 
 
 class _mapfunc(object):
@@ -526,7 +498,7 @@ class ProgressBar:
             If `True`, the progress bar will display as an IPython
             notebook widget.
 
-        file : writable file-like object, optional
+        file : writable file-like, optional
             The file to write the progress bar to.  Defaults to
             `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any, or special case hacks
@@ -648,10 +620,8 @@ class ProgressBar:
         else:
             t = ((time.time() - self._start_time) * (1.0 - frac)) / frac
             prefix = ' ETA '
-        write(' {0:>4s}/{1:>4s}'.format(
-            human_file_size(value),
-            self._human_total))
-        write(' ({:>6.2%})'.format(frac))
+        write(f' {human_file_size(value):>4s}/{self._human_total:>4s}')
+        write(f' ({frac:>6.2%})')
         write(prefix)
         if t is not None:
             write(human_time(t))
@@ -685,18 +655,18 @@ class ProgressBar:
         # Calculate percent completion, and update progress bar
         frac = (value/self._total)
         self._widget.value = frac * 100
-        self._widget.description = ' ({:>6.2%})'.format(frac)
+        self._widget.description = f' ({frac:>6.2%})'
 
     def _silent_update(self, value=None):
         pass
 
     @classmethod
     def map(cls, function, items, multiprocess=False, file=None, step=100,
-            ipython_widget=False):
-        """
-        Does a `map` operation while displaying a progress bar with
-        percentage complete. The map operation may run on arbitrary order
-        on the items, but the results are returned in sequential order.
+            ipython_widget=False, multiprocessing_start_method=None):
+        """Map function over items while displaying a progress bar with percentage complete.
+
+        The map operation may run in arbitrary order on the items, but the results are
+        returned in sequential order.
 
         ::
 
@@ -723,7 +693,7 @@ class ProgressBar:
             If `True`, the progress bar will display as an IPython
             notebook widget.
 
-        file : writeable file-like object, optional
+        file : writable file-like, optional
             The file to write the progress bar to.  Defaults to
             `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any), the scrollbar will
@@ -735,15 +705,24 @@ class ProgressBar:
             of the chunks of ``items`` that are submitted as separate tasks
             to the process pool.  A large step size may make the job
             complete faster if ``items`` is very long.
+
+        multiprocessing_start_method : str, optional
+            Useful primarily for testing; if in doubt leave it as the default.
+            When using multiprocessing, certain anomalies occur when starting
+            processes with the "spawn" method (the only option on Windows);
+            other anomalies occur with the "fork" method (the default on
+            Linux).
         """
 
         if multiprocess:
             function = _mapfunc(function)
             items = list(enumerate(items))
 
-        results = cls.map_unordered(function, items, multiprocess=multiprocess,
-                                    file=file, step=step,
-                                    ipython_widget=ipython_widget)
+        results = cls.map_unordered(
+            function, items, multiprocess=multiprocess,
+            file=file, step=step,
+            ipython_widget=ipython_widget,
+            multiprocessing_start_method=multiprocessing_start_method)
 
         if multiprocess:
             _, results = zip(*sorted(results))
@@ -753,8 +732,10 @@ class ProgressBar:
 
     @classmethod
     def map_unordered(cls, function, items, multiprocess=False, file=None,
-                      step=100, ipython_widget=False):
-        """
+                      step=100, ipython_widget=False,
+                      multiprocessing_start_method=None):
+        """Map function over items, reporting the progress.
+
         Does a `map` operation while displaying a progress bar with
         percentage complete. The map operation may run on arbitrary order
         on the items, and the results may be returned in arbitrary order.
@@ -784,7 +765,7 @@ class ProgressBar:
             If `True`, the progress bar will display as an IPython
             notebook widget.
 
-        file : writeable file-like object, optional
+        file : writable file-like, optional
             The file to write the progress bar to.  Defaults to
             `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any), the scrollbar will
@@ -796,6 +777,13 @@ class ProgressBar:
             of the chunks of ``items`` that are submitted as separate tasks
             to the process pool.  A large step size may make the job
             complete faster if ``items`` is very long.
+
+        multiprocessing_start_method : str, optional
+            Useful primarily for testing; if in doubt leave it as the default.
+            When using multiprocessing, certain anomalies occur when starting
+            processes with the "spawn" method (the only option on Windows);
+            other anomalies occur with the "fork" method (the default on
+            Linux).
         """
 
         results = []
@@ -815,14 +803,22 @@ class ProgressBar:
                     if (i % chunksize) == 0:
                         bar.update(i)
             else:
-                p = multiprocessing.Pool(
-                    processes=(int(multiprocess) if multiprocess is not True else None))
-                for i, result in enumerate(
-                    p.imap_unordered(function, items, chunksize=chunksize)):
-                    bar.update(i)
-                    results.append(result)
-                p.close()
-                p.join()
+                ctx = multiprocessing.get_context(multiprocessing_start_method)
+                if sys.version_info >= (3, 7):
+                    kwargs = dict(mp_context=ctx)
+                else:
+                    kwargs = {}
+                with ProcessPoolExecutor(
+                        max_workers=(int(multiprocess)
+                                     if multiprocess is not True
+                                     else None),
+                        **kwargs) as p:
+                    for i, f in enumerate(
+                            as_completed(
+                                p.submit(function, item)
+                                for item in items)):
+                        bar.update(i)
+                        results.append(f.result())
 
         return results
 
@@ -854,7 +850,7 @@ class Spinner:
             darkgrey, lightred, lightgreen, yellow, lightblue,
             lightmagenta, lightcyan, white.
 
-        file : writeable file-like object, optional
+        file : writable file-like, optional
             The file to write the spinner to.  Defaults to
             `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any, or special case hacks
@@ -982,7 +978,7 @@ class ProgressBarOrSpinner:
             lightred, lightgreen, yellow, lightblue, lightmagenta,
             lightcyan, white.
 
-        file : writable file-like object, optional
+        file : writable file-like, optional
             The file to write the to.  Defaults to `sys.stdout`.  If
             ``file`` is not a tty (as determined by calling its `isatty`
             member, if any), only ``msg`` will be displayed: the
@@ -1042,7 +1038,7 @@ def print_code_line(line, col=None, file=None, tabwidth=8, width=70):
         The character in the line to highlight.  ``col`` must be less
         than ``len(line)``.
 
-    file : writeable file-like object, optional
+    file : writable file-like, optional
         Where to write to.  Defaults to `sys.stdout`.
 
     tabwidth : int, optional
