@@ -60,8 +60,10 @@ def serialize_context_as(context):
     """
     old_context = BaseColumnInfo._serialize_context
     BaseColumnInfo._serialize_context = context
-    yield
-    BaseColumnInfo._serialize_context = old_context
+    try:
+        yield
+    finally:
+        BaseColumnInfo._serialize_context = old_context
 
 
 def dtype_info_name(dtype):
@@ -82,7 +84,7 @@ def dtype_info_name(dtype):
 
     Parameters
     ----------
-    dtype : str, np.dtype, type
+    dtype : str, `~numpy.dtype`, type
         Input dtype as an object that can be converted via np.dtype()
 
     Returns
@@ -260,7 +262,8 @@ class DataInfo(metaclass=DataInfoMeta):
     called ``info`` so that the DataInfo() object can be stored in the
     ``instance`` using the ``info`` key.  Because owner_cls.x is a descriptor,
     Python doesn't use __dict__['x'] normally, and the descriptor can safely
-    store stuff there.  Thanks to https://nbviewer.jupyter.org/urls/gist.github.com/ChrisBeaumont/5758381/raw/descriptor_writeup.ipynb
+    store stuff there.  Thanks to
+    https://nbviewer.jupyter.org/urls/gist.github.com/ChrisBeaumont/5758381/raw/descriptor_writeup.ipynb
     for this trick that works for non-hashable classes.
 
     Parameters
@@ -346,7 +349,14 @@ reference with ``c = col[3:5]`` followed by ``c.info``.""")
 
         if isinstance(value, DataInfo):
             info = instance.__dict__['info'] = self.__class__(bound=True)
-            for attr in info.attr_names - info.attrs_from_parent - info._attrs_no_copy:
+            attr_names = info.attr_names
+            if value.__class__ is self.__class__:
+                # For same class, attributes are guaranteed to be stored in
+                # _attrs, so speed matters up by not accessing defaults.
+                # Doing this before difference in for loop helps speed.
+                attr_names = attr_names & set(value._attrs)  # NOT in-place!
+
+            for attr in attr_names - info.attrs_from_parent - info._attrs_no_copy:
                 info._attrs[attr] = deepcopy(getattr(value, attr))
 
         else:
@@ -426,15 +436,16 @@ reference with ``c = col[3:5]`` followed by ``c.info``.""")
 
         Parameters
         ----------
-        option : str, function, list of (str or function)
+        option : str, callable, list of (str or callable)
             Info option, defaults to 'attributes'.
-        out : file-like object, None
+        out : file-like, None
             Output destination, defaults to sys.stdout.  If None then the
             OrderedDict with information attributes is returned
 
         Returns
         -------
-        info : OrderedDict if out==None else None
+        info : `~collections.OrderedDict` or None
+            `~collections.OrderedDict` if out==None else None
         """
         if out == '':
             out = sys.stdout
@@ -559,6 +570,18 @@ class BaseColumnInfo(DataInfo):
         for str_val in _pformat_col_iter(col, -1, False, False, {}):
             yield str_val
 
+    @property
+    def indices(self):
+        # Implementation note: the auto-generation as an InfoAttribute cannot
+        # be used here, since on access, one should not just return the
+        # default (empty list is this case), but set _attrs['indices'] so that
+        # if the list is appended to, it is registered here.
+        return self._attrs.setdefault('indices', [])
+
+    @indices.setter
+    def indices(self, indices):
+        self._attrs['indices'] = indices
+
     def adjust_indices(self, index, value, col_len):
         '''
         Adjust info indices after column modification.
@@ -604,8 +627,9 @@ class BaseColumnInfo(DataInfo):
 
         Parameters
         ----------
-        col_slice : Column or mixin
-            Sliced object
+        col_slice : `~astropy.table.Column` or mixin
+            Sliced object. If not a column, it must be a valid mixin, see
+            https://docs.astropy.org/en/stable/table/mixin_columns.html
         item : slice, list, or ndarray
             Slice used to create col_slice
         col_len : int
@@ -661,7 +685,8 @@ class BaseColumnInfo(DataInfo):
 
         Returns
         -------
-        attrs : dict of merged attributes
+        attrs : dict
+            Of merged attributes.
 
         """
         from astropy.table.np_utils import TableMergeError
