@@ -6,6 +6,7 @@
 import functools
 import inspect
 import textwrap
+import threading
 import types
 import warnings
 from inspect import signature
@@ -30,7 +31,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
     To mark an attribute as deprecated, use `deprecated_attribute`.
 
     Parameters
-    ------------
+    ----------
     since : str
         The release at which this API became deprecated.  This is
         required.
@@ -67,7 +68,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         The type of this object, if the automatically determined one
         needs to be overridden.
 
-    warning_type : warning
+    warning_type : Warning
         Warning to be issued.
         Default is `~astropy.utils.exceptions.AstropyDeprecationWarning`.
     """
@@ -239,7 +240,7 @@ def deprecated_attribute(name, since, message=None, alternative=None,
         If True, uses a AstropyPendingDeprecationWarning instead of
         ``warning_type``.
 
-    warning_type : warning
+    warning_type : Warning
         Warning to be issued.
         Default is `~astropy.utils.exceptions.AstropyDeprecationWarning`.
 
@@ -287,17 +288,17 @@ def deprecated_renamed_argument(old_name, new_name, since,
 
     Parameters
     ----------
-    old_name : str or list/tuple thereof
+    old_name : str or sequence of str
         The old name of the argument.
 
-    new_name : str or list/tuple thereof or `None`
+    new_name : str or sequence of str or None
         The new name of the argument. Set this to `None` to remove the
         argument ``old_name`` instead of renaming it.
 
-    since : str or number or list/tuple thereof
+    since : str or number or sequence of str or number
         The release at which the old argument became deprecated.
 
-    arg_in_kwargs : bool or list/tuple thereof, optional
+    arg_in_kwargs : bool or sequence of bool, optional
         If the argument is not a named argument (for example it
         was meant to be consumed by ``**kwargs``) set this to
         ``True``.  Otherwise the decorator will throw an Exception
@@ -305,18 +306,18 @@ def deprecated_renamed_argument(old_name, new_name, since,
         the decorated function.
         Default is ``False``.
 
-    relax : bool or list/tuple thereof, optional
+    relax : bool or sequence of bool, optional
         If ``False`` a ``TypeError`` is raised if both ``new_name`` and
         ``old_name`` are given.  If ``True`` the value for ``new_name`` is used
         and a Warning is issued.
         Default is ``False``.
 
-    pending : bool or list/tuple thereof, optional
+    pending : bool or sequence of bool, optional
         If ``True`` this will hide the deprecation warning and ignore the
         corresponding ``relax`` parameter value.
         Default is ``False``.
 
-    warning_type : warning
+    warning_type : Warning
         Warning to be issued.
         Default is `~astropy.utils.exceptions.AstropyDeprecationWarning`.
 
@@ -653,6 +654,7 @@ class classproperty(property):
     def __init__(self, fget, doc=None, lazy=False):
         self._lazy = lazy
         if lazy:
+            self._lock = threading.RLock()   # Protects _cache
             self._cache = {}
         fget = self._wrap_fget(fget)
 
@@ -667,17 +669,20 @@ class classproperty(property):
             self.__doc__ = doc
 
     def __get__(self, obj, objtype):
-        if self._lazy and objtype in self._cache:
-            return self._cache[objtype]
-
-        # The base property.__get__ will just return self here;
-        # instead we pass objtype through to the original wrapped
-        # function (which takes the class as its sole argument)
-        val = self.fget.__wrapped__(objtype)
-
         if self._lazy:
-            self._cache[objtype] = val
-
+            val = self._cache.get(objtype, _NotFound)
+            if val is _NotFound:
+                with self._lock:
+                    # Check if another thread initialised before we locked.
+                    val = self._cache.get(objtype, _NotFound)
+                    if val is _NotFound:
+                        val = self.fget.__wrapped__(objtype)
+                        self._cache[objtype] = val
+        else:
+            # The base property.__get__ will just return self here;
+            # instead we pass objtype through to the original wrapped
+            # function (which takes the class as its sole argument)
+            val = self.fget.__wrapped__(objtype)
         return val
 
     def getter(self, fget):
@@ -748,16 +753,20 @@ class lazyproperty(property):
     def __init__(self, fget, fset=None, fdel=None, doc=None):
         super().__init__(fget, fset, fdel, doc)
         self._key = self.fget.__name__
+        self._lock = threading.RLock()
 
     def __get__(self, obj, owner=None):
         try:
-            val = obj.__dict__.get(self._key, _NotFound)
-            if val is not _NotFound:
-                return val
-            else:
-                val = self.fget(obj)
-                obj.__dict__[self._key] = val
-                return val
+            obj_dict = obj.__dict__
+            val = obj_dict.get(self._key, _NotFound)
+            if val is _NotFound:
+                with self._lock:
+                    # Check if another thread beat us to it.
+                    val = obj_dict.get(self._key, _NotFound)
+                    if val is _NotFound:
+                        val = self.fget(obj)
+                        obj_dict[self._key] = val
+            return val
         except AttributeError:
             if obj is None:
                 return self
@@ -768,17 +777,16 @@ class lazyproperty(property):
         if self.fset:
             ret = self.fset(obj, val)
             if ret is not None and obj_dict.get(self._key) is ret:
-                # By returning the value set the setter signals that it took
-                # over setting the value in obj.__dict__; this mechanism allows
-                # it to override the input value
+                # By returning the value set the setter signals that it
+                # took over setting the value in obj.__dict__; this
+                # mechanism allows it to override the input value
                 return
         obj_dict[self._key] = val
 
     def __delete__(self, obj):
         if self.fdel:
             self.fdel(obj)
-        if self._key in obj.__dict__:
-            del obj.__dict__[self._key]
+        obj.__dict__.pop(self._key, None)    # Delete if present
 
 
 class sharedmethod(classmethod):
