@@ -1,6 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import urllib.request
 import os
+import locale
+import platform
 
 import pytest
 import numpy as np
@@ -10,7 +12,26 @@ import erfa
 from astropy.time import Time, TimeDelta
 from astropy.utils.iers import iers
 from astropy.utils.data import get_pkg_data_filename
+from astropy.tests.tests.test_imports import test_imports
 
+# Import every top-level astropy module as a test that the ERFA leap second
+# table is not updated for normal imports.
+test_imports()
+
+# Now test that the erfa leap_seconds table has not been updated. This must be
+# done at the module level, which unfortunately will abort the entire test run
+# if if fails. Running within a normal pytest test will not work because the
+# other tests will end up updating this attribute by virtue of doing Time UTC
+# transformations.
+assert erfa.leap_seconds._expires is None
+
+# Tests in this module assume that the erfa.leap_seconds attribute has been
+# updated from the `erfa` package built-in table to the astropy built-in
+# leap-second table. That has the effect of ensuring that the
+# `erfa.leap_seconds.expires` property is sufficiently in the future.
+iers_table = iers.LeapSeconds.auto_open()
+erfa.leap_seconds.update(iers_table)
+assert erfa.leap_seconds._expires is not None
 
 SYSTEM_FILE = '/usr/share/zoneinfo/leap-seconds.list'
 
@@ -48,6 +69,24 @@ class TestReading:
         assert ls['tai_utc'][-1] >= 37
         self.verify_day_month_year(ls)
 
+    def test_read_leap_second_dat_locale(self):
+        current = locale.setlocale(locale.LC_ALL)
+        try:
+            if platform.system() == 'Darwin':
+                locale.setlocale(locale.LC_ALL, 'fr_FR')
+            else:
+                locale.setlocale(locale.LC_ALL, 'fr_FR.utf8')
+
+            ls = iers.LeapSeconds.from_iers_leap_seconds(
+                iers.IERS_LEAP_SECOND_FILE)
+        except locale.Error as e:
+            pytest.skip(f'Locale error: {e}')
+        finally:
+            locale.setlocale(locale.LC_ALL, current)
+
+        # Below, >= to take into account we might ship and updated file.
+        assert ls.expires >= Time('2020-06-28', scale='tai')
+
     def test_open_leap_second_dat(self):
         ls = iers.LeapSeconds.from_iers_leap_seconds(
             iers.IERS_LEAP_SECOND_FILE)
@@ -209,7 +248,7 @@ class TestDefaultAutoOpen:
         assert ls.meta['data_url'] == iers.IERS_LEAP_SECOND_FILE
 
     # The test below is marked remote_data only to ensure it runs
-    # as an allowed-fail job on travis: i.e., we will notice it (eventually)
+    # as an allowed-fail job on CI: i.e., we will notice it (eventually)
     # but will not be misled in thinking that a PR is bad.
     @pytest.mark.remote_data
     def test_builtin_not_expired(self):
