@@ -31,6 +31,10 @@ __all__ = ['isiterable', 'silence', 'format_exception', 'NumpyRNGContext',
            'OrderedDescriptor', 'OrderedDescriptorContainer']
 
 
+# Because they are deprecated.
+__doctest_skip__ = ['OrderedDescriptor', 'OrderedDescriptorContainer']
+
+
 def isiterable(obj):
     """Returns `True` if the given object is iterable."""
 
@@ -194,8 +198,8 @@ def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
 
     """
     import webbrowser
-    import urllib.request
     from zlib import decompress
+    from astropy.utils.data import get_readable_fileobj
 
     if (not isinstance(obj, str) and
             hasattr(obj, '__module__') and
@@ -226,15 +230,10 @@ def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
 
     # Custom request headers; see
     # https://github.com/astropy/astropy/issues/8990
-    req = urllib.request.Request(
-        baseurl + 'objects.inv', headers={'User-Agent': f'Astropy/{version}'})
-
-    if timeout is None:
-        uf = urllib.request.urlopen(req)
-    else:
-        uf = urllib.request.urlopen(req, timeout=timeout)
-
-    try:
+    url = baseurl + 'objects.inv'
+    headers = {'User-Agent': f'Astropy/{version}'}
+    with get_readable_fileobj(url, encoding='binary', remote_timeout=timeout,
+                              http_headers=headers) as uf:
         oiread = uf.read()
 
         # need to first read/remove the first four lines, which have info before
@@ -254,8 +253,6 @@ def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
                              'has changed?'.format(baseurl + 'objects.inv'))
 
         compressed = oiread[(idx+1):]
-    finally:
-        uf.close()
 
     decompressed = decompress(compressed).decode('utf-8')
 
@@ -494,57 +491,17 @@ def did_you_mean(s, candidates, n=3, cutoff=0.8, fix=None):
     return ''
 
 
-@deprecated('4.0', alternative='Sphinx>=1.7 automatically inherits docstring')
-class InheritDocstrings(type):
-    """
-    This metaclass makes methods of a class automatically have their
-    docstrings filled in from the methods they override in the base
-    class.
+_ordered_descriptor_deprecation_message = """\
+The {func} {obj_type} is deprecated and may be removed in a future version.
 
-    If the class uses multiple inheritance, the docstring will be
-    chosen from the first class in the bases list, in the same way as
-    methods are normally resolved in Python.  If this results in
-    selecting the wrong docstring, the docstring will need to be
-    explicitly included on the method.
-
-    For example::
-
-        >>> import warnings
-        >>> with warnings.catch_warnings():
-        ...     # Ignore deprecation warning
-        ...     warnings.simplefilter('ignore')
-        ...     from astropy.utils.misc import InheritDocstrings
-        ...     class A(metaclass=InheritDocstrings):
-        ...         def wiggle(self):
-        ...             "Wiggle the thingamajig"
-        ...             pass
-        ...     class B(A):
-        ...         def wiggle(self):
-        ...             pass
-        >>> B.wiggle.__doc__
-        u'Wiggle the thingamajig'
-    """
-
-    def __init__(cls, name, bases, dct):
-        def is_public_member(key):
-            return (
-                (key.startswith('__') and key.endswith('__')
-                 and len(key) > 4) or
-                not key.startswith('_'))
-
-        for key, val in dct.items():
-            if ((inspect.isfunction(val) or inspect.isdatadescriptor(val)) and
-                    is_public_member(key) and
-                    val.__doc__ is None):
-                for base in cls.__mro__[1:]:
-                    super_method = getattr(base, key, None)
-                    if super_method is not None:
-                        val.__doc__ = super_method.__doc__
-                        break
-
-        super().__init__(name, bases, dct)
+    You can replace its functionality with a combination of the
+    __init_subclass__ and __set_name__ magic methods introduced in Python 3.6.
+    See https://github.com/astropy/astropy/issues/11094 for recipes on how to
+    replicate their functionality.
+"""
 
 
+@deprecated('4.3', _ordered_descriptor_deprecation_message)
 class OrderedDescriptor(metaclass=abc.ABCMeta):
     """
     Base class for descriptors whose order in the class body should be
@@ -629,6 +586,7 @@ class OrderedDescriptor(metaclass=abc.ABCMeta):
             return NotImplemented
 
 
+@deprecated('4.3', _ordered_descriptor_deprecation_message)
 class OrderedDescriptorContainer(type):
     """
     Classes should use this metaclass if they wish to use `OrderedDescriptor`
@@ -829,26 +787,6 @@ class OrderedDescriptorContainer(type):
                                                         members)
 
 
-def get_parameters(members):
-    """
-    Looks for ordered descriptors in a class definition and
-    copies such definitions in two new class attributes,
-    one being a dictionary of these objects keyed by their
-    attribute names, and the other a simple list of those names.
-
-    """
-    pdict = OrderedDict()
-    for name, obj in members.items():
-        if (not isinstance(obj, OrderedDescriptor)):
-            continue
-        if obj._name_attribute_ is not None:
-            setattr(obj, '_name', name)
-        pdict[name] = obj
-
-    # members['_parameter_vals_'] = pdict
-    members['_parameters_'] = pdict
-
-
 LOCALE_LOCK = threading.Lock()
 
 
