@@ -55,6 +55,12 @@ RANGE_CASES = [
      ['2014-05-01',
       '2014-10-01']),
 
+    # Interval of a few months
+    (('2014-11-22T12:30:30.9', '2015-02-22T12:30:32.1'),
+     ['2014-12-01',
+      '2015-01-01',
+      '2015-02-01']),
+
     # Interval of just over a month
     (('2014-03-22T12:30:30.9', '2014-04-23T12:30:32.1'),
      ['2014-04-01']),
