@@ -1,8 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-import os
-import sys
-
 from packaging.version import Version
 import pytest
 import numpy as np
@@ -15,19 +12,20 @@ from astropy.wcs import WCS
 from astropy.io import fits
 from astropy.coordinates import SkyCoord
 
+from astropy.utils.compat.context import nullcontext
+from astropy.utils.data import get_pkg_data_filename
+
 from astropy.visualization.wcsaxes.core import WCSAxes
 from astropy.visualization.wcsaxes.frame import (
     EllipticalFrame, RectangularFrame, RectangularFrame1D)
 from astropy.visualization.wcsaxes.utils import get_coord_meta
 from astropy.visualization.wcsaxes.transforms import CurvedTransform
 
-mpl_version = Version(matplotlib.__version__)
 ft_version = Version(matplotlib.ft2font.__freetype_version__)
-MATPLOTLIB_LT_31 = mpl_version < Version('3.1')
 FREETYPE_261 = ft_version == Version("2.6.1")
 TEX_UNAVAILABLE = not matplotlib.checkdep_usetex(True)
 
-DATA = os.path.abspath(os.path.join(os.path.dirname(__file__), 'data'))
+MATPLOTLIB_GT_3_4_2 = Version(matplotlib.__version__) > Version('3.4.2')
 
 
 def teardown_function(function):
@@ -75,16 +73,18 @@ COORDSYS= 'icrs    '
 """, sep='\n')
 
 
-@pytest.mark.skipif(MATPLOTLIB_LT_31 and sys.version_info >= (3, 9),
-                    reason='PY_SSIZE_T_CLEAN warning with Python 3.9 and '
-                    'Matplotlib 3.0, GH issue 10954')
 @pytest.mark.parametrize('grid_type', ['lines', 'contours'])
 def test_no_numpy_warnings(ignore_matplotlibrc, tmpdir, grid_type):
     ax = plt.subplot(1, 1, 1, projection=WCS(TARGET_HEADER))
     ax.imshow(np.zeros((100, 200)))
     ax.coords.grid(color='white', grid_type=grid_type)
 
-    with pytest.warns(None) as warning_lines:
+    if MATPLOTLIB_GT_3_4_2 and grid_type == 'contours':
+        ctx = pytest.raises(AttributeError, match='dpi')
+    else:
+        ctx = nullcontext()
+
+    with pytest.warns(None) as warning_lines, ctx:
         plt.savefig(tmpdir.join('test.png').strpath)
 
     # There should be no warnings raised if some pixels are outside WCS
@@ -97,7 +97,8 @@ def test_no_numpy_warnings(ignore_matplotlibrc, tmpdir, grid_type):
         w_msg = str(w.message)
         assert ('converting a masked element to nan' in w_msg or
                 'No contour levels were found within the data range' in w_msg or
-                'np.asscalar(a) is deprecated since NumPy v1.16' in w_msg)
+                'np.asscalar(a) is deprecated since NumPy v1.16' in w_msg or
+                'PY_SSIZE_T_CLEAN will be required' in w_msg)
 
 
 def test_invalid_frame_overlay(ignore_matplotlibrc):
@@ -115,7 +116,7 @@ def test_invalid_frame_overlay(ignore_matplotlibrc):
 
 def test_plot_coord_transform(ignore_matplotlibrc):
 
-    twoMASS_k_header = os.path.join(DATA, '2MASS_k_header')
+    twoMASS_k_header = get_pkg_data_filename('data/2MASS_k_header')
     twoMASS_k_header = fits.Header.fromtextfile(twoMASS_k_header)
     fig = plt.figure(figsize=(6, 6))
     ax = fig.add_axes([0.15, 0.15, 0.8, 0.8],
