@@ -17,10 +17,10 @@ def _fix(content, indent=0):
 
 def TWO_OR_MORE_ARGS(naxis, indent=0):
     return _fix(
-"""args : flexible
+f"""*args
     There are two accepted forms for the positional arguments:
 
-        - 2 arguments: An *N* x *{}* array of coordinates, and an
+        - 2 arguments: An *N* x *{naxis}* array of coordinates, and an
           *origin*.
 
         - more than 2 arguments: An array for each axis, followed by
@@ -30,14 +30,14 @@ def TWO_OR_MORE_ARGS(naxis, indent=0):
     Here, *origin* is the coordinate in the upper left corner of the
     image.  In FITS and Fortran standards, this is 1.  In Numpy and C
     standards this is 0.
-""".format(naxis), indent)
+""", indent)
 
 
 def RETURNS(out_type, indent=0):
-    return _fix("""result : array
-    Returns the {}.  If the input was a single array and
+    return _fix(f"""result : array
+    Returns the {out_type}.  If the input was a single array and
     origin, a single array is returned, otherwise a tuple of arrays is
-    returned.""".format(out_type), indent)
+    returned.""", indent)
 
 
 def ORIGIN(indent=0):
@@ -95,14 +95,14 @@ The first three (the distortion corrections) are done in parallel.
 
 Parameters
 ----------
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 {}
 
 Returns
 -------
-world : numpy.ndarray
+world : ndarray
     Returns an array of world coordinates as ``double array[ncoord][nelem]``.
 
 Raises
@@ -365,12 +365,13 @@ cmp : int, optional
     A bit field controlling the strictness of the comparison.  When 0,
     (the default), all fields must be identical.
 
-    The following constants may be or'ed together to loosen the
-    comparison.
+    The following constants, defined in the `astropy.wcs` module,
+    may be or'ed together to loosen the comparison.
 
     - ``WCSCOMPARE_ANCILLARY``: Ignores ancillary keywords that don't
-      change the WCS transformation, such as ``DATE-OBS`` or
-      ``EQUINOX``.
+      change the WCS transformation, such as ``XPOSURE`` or
+      ``EQUINOX``. Note that this also ignores ``DATE-OBS``, which does
+      change the WCS transformation in some cases.
 
     - ``WCSCOMPARE_TILING``: Ignore integral differences in
       ``CRPIXja``.  This is the 'tiling' condition, where two WCSes
@@ -751,7 +752,7 @@ keysel : sequence of flags
 
 Returns
 -------
-wcs_list : list of `~astropy.wcs.Wcsprm` objects
+wcs_list : list of `~astropy.wcs.Wcsprm`
 """
 
 fix = """
@@ -787,7 +788,7 @@ translate_units : str, optional
     Thus ``''`` doesn't do any unsafe translations, whereas ``'shd'``
     does all of them.
 
-naxis : int array[naxis], optional
+naxis : int array, optional
     Image axis lengths.  If this array is set to zero or ``None``,
     then `~astropy.wcs.Wcsprm.cylfix` will not be invoked.
 
@@ -819,7 +820,7 @@ Returns the offset as defined in the distortion lookup table.
 
 Returns
 -------
-coordinate : coordinate pair
+coordinate : (2,) tuple
     The offset from the distortion table for pixel point (*x*, *y*).
 """
 
@@ -876,7 +877,7 @@ Returns ``PVi_ma`` keywords for each *i* and *m* as list of tuples.
 
 Returns
 -------
-
+sequence of tuple
     Returned as a list of tuples of the form (*i*, *m*, *value*):
 
     - *i*: int.  Axis number, as in ``PVi_ma``, (i.e. 1-relative)
@@ -1111,7 +1112,7 @@ mixcel : int
     celestial latitude is given in ``world[self.lat]``, longitude
     returned in ``world[self.lng]``.
 
-vspan : pair of floats
+vspan : (float, float)
     Solution interval for the celestial coordinate, in degrees.  The
     ordering of the two limits is irrelevant.  Longitude ranges may be
     specified with any convenient normalization, for example
@@ -1128,14 +1129,14 @@ viter : int
     the search recommenced.  *viter* controls how many times the step
     size is halved.  The allowed range is 5 - 10.
 
-world : numpy.ndarray
+world : ndarray
     World coordinate elements as ``double array[naxis]``.  ``world[self.lng]`` and
     ``world[self.lat]`` are the celestial longitude and latitude, in
     degrees.  Which is given and which returned depends on the value
     of *mixcel*.  All other elements are given.  The results will be
     written to this array in-place.
 
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Pixel coordinates as ``double array[naxis]``.  The element indicated by *mixpix* is given and
     the remaining elements will be written in-place.
 
@@ -1318,7 +1319,7 @@ Converts pixel to world coordinates.
 Parameters
 ----------
 
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 {}
@@ -1328,7 +1329,7 @@ Returns
 result : dict
     Returns a dictionary with the following keys:
 
-    - *imgcrd*: numpy.ndarray
+    - *imgcrd*: ndarray
 
       - Array of intermediate world coordinates as ``double array[ncoord][nelem]``.  For celestial axes,
         ``imgcrd[][self.lng]`` and ``imgcrd[][self.lat]`` are the
@@ -1336,16 +1337,16 @@ result : dict
         spectral axes, ``imgcrd[][self.spec]`` is the intermediate
         spectral coordinate, in SI units.
 
-    - *phi*: numpy.ndarray
+    - *phi*: ndarray
 
       - Array as ``double array[ncoord]``.
 
-    - *theta*: numpy.ndarray
+    - *theta*: ndarray
 
       - Longitude and latitude in the native coordinate system of the
         projection, in degrees, as ``double array[ncoord]``.
 
-    - *world*: numpy.ndarray
+    - *world*: ndarray
 
       - Array of world coordinates as ``double array[ncoord][nelem]``.  For celestial axes,
         ``world[][self.lng]`` and ``world[][self.lat]`` are the
@@ -1353,7 +1354,7 @@ result : dict
         axes, ``world[][self.spec]`` is the intermediate spectral
         coordinate, in SI units.
 
-    - *stat*: numpy.ndarray
+    - *stat*: ndarray
 
       - Status return value for each coordinate as ``int array[ncoord]``. ``0`` for success,
         ``1+`` for invalid pixel coordinate.
@@ -1396,14 +1397,14 @@ paper`_ lookup-table correction.
 
 Parameters
 ----------
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 {}
 
 Returns
 -------
-foccrd : numpy.ndarray
+foccrd : ndarray
     Returns an array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 Raises
@@ -1464,14 +1465,14 @@ correction in parallel.
 
 Parameters
 ----------
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 {}
 
 Returns
 -------
-foccrd : numpy.ndarray
+foccrd : ndarray
     Returns an array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 Raises
@@ -1551,7 +1552,7 @@ Transforms world coordinates to pixel coordinates.
 
 Parameters
 ----------
-world : numpy.ndarray
+world : ndarray
     Array of world coordinates, in decimal degrees, as ``double array[ncoord][nelem]``.
 
 {}
@@ -1681,7 +1682,7 @@ Raises
 MemoryError
     Memory allocation failed.
 
-InvalidTabularParameters
+InvalidTabularParametersError
     Invalid tabular parameters.
 """
 
@@ -1692,7 +1693,7 @@ Sets ``PSi_ma`` keywords for each *i* and *m*.
 
 Parameters
 ----------
-ps : sequence of tuples
+ps : sequence of tuple
 
     The input must be a sequence of tuples of the form (*i*, *m*,
     *value*):
@@ -1744,23 +1745,23 @@ using the `SIP`_ convention in both directions.
 
 Parameters
 ----------
-a : numpy.ndarray
+a : ndarray
     The ``A_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``A_ORDER``.
 
-b : numpy.ndarray
+b : ndarray
     The ``B_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``B_ORDER``.
 
-ap : numpy.ndarray
+ap : ndarray
     The ``AP_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``AP_ORDER``.
 
-bp : numpy.ndarray
+bp : ndarray
     The ``BP_i_j`` polynomial for pixel to focal plane transformation as ``double array[m+1][m+1]``.
     Its size must be (*m* + 1, *m* + 1) where *m* = ``BP_ORDER``.
 
-crpix : numpy.ndarray
+crpix : ndarray
     The reference pixel as ``double array[2]``.
 
 Notes
@@ -1778,14 +1779,14 @@ polynomial distortion convention.
 
 Parameters
 ----------
-foccrd : numpy.ndarray
+foccrd : ndarray
     Array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 {}
 
 Returns
 -------
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Returns an array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 Raises
@@ -1805,14 +1806,14 @@ polynomial distortion convention.
 
 Parameters
 ----------
-pixcrd : numpy.ndarray
+pixcrd : ndarray
     Array of pixel coordinates as ``double array[ncoord][nelem]``.
 
 {}
 
 Returns
 -------
-foccrd : numpy.ndarray
+foccrd : ndarray
     Returns an array of focal plane coordinates as ``double array[ncoord][nelem]``.
 
 Raises
@@ -1978,7 +1979,7 @@ MemoryError
 InvalidSubimageSpecificationError
     Invalid subimage specification (no spectral axis).
 
-NonseparableSubimageCoordinateSystem
+NonseparableSubimageCoordinateSystemError
     Non-separable subimage coordinate system.
 
 Notes
@@ -2202,15 +2203,13 @@ To perform all distortion corrections and WCS transformation, use
 
 Parameters
 ----------
-sip : `~astropy.wcs.Sip` object or `None`
+sip : `~astropy.wcs.Sip` object or None
 
-cpdis : A pair of `~astropy.wcs.DistortionLookupTable` objects, or
-  ``(None, None)``.
+cpdis : (2,) tuple of `~astropy.wcs.DistortionLookupTable` or None
 
-wcsprm : `~astropy.wcs.Wcsprm` object
+wcsprm : `~astropy.wcs.Wcsprm`
 
-det2im : A pair of `~astropy.wcs.DistortionLookupTable` objects, or
-   ``(None, None)``.
+det2im : (2,) tuple of `~astropy.wcs.DistortionLookupTable` or None
 """
 
 Wcsprm = """
@@ -2234,7 +2233,7 @@ Sect. 5.2.1), integer (Sect. 5.2.3), and floating-point values
 
 Parameters
 ----------
-header : An `astropy.io.fits.Header`, string, or `None`.
+header : `~astropy.io.fits.Header`, str, or None.
   If ``None``, the object will be initialized to default values.
 
 key : str, optional
