@@ -8,6 +8,7 @@ from packaging.version import Version
 import numpy as np
 import pytest
 from numpy.testing import assert_equal, assert_allclose
+from itertools import product
 
 from astropy import units as u
 from astropy.time import Time
@@ -929,6 +930,23 @@ def test_sub_wcsapi_attributes():
     assert wcs_sub4.world_axis_names == ['', '']
 
 
+HEADER_POLARIZED = """
+CTYPE1  = 'HPLT-TAN'
+CTYPE2  = 'HPLN-TAN'
+CTYPE3  = 'STOKES'
+"""
+
+
+@pytest.fixture
+def header_polarized():
+    return Header.fromstring(HEADER_POLARIZED, sep='\n')
+
+
+def test_phys_type_polarization(header_polarized):
+    w = WCS(header_polarized)
+    assert w.world_axis_physical_types[2] == 'phys.polarization.stokes'
+
+
 ###############################################################################
 # Spectral transformations
 ###############################################################################
@@ -1010,3 +1028,42 @@ def test_spectralcoord_frame(header_spectral_frames):
             # the spectral coordinate unchanged
             sc_check = sc.with_observer_stationary_relative_to(expected_frame)
             assert_quantity_allclose(sc.quantity, sc_check.quantity)
+
+
+@pytest.mark.parametrize(('ctype3', 'observer'), product(['ZOPT', 'BETA', 'VELO', 'VRAD', 'VOPT'], [False, True]))
+def test_different_ctypes(header_spectral_frames, ctype3, observer):
+
+    header = header_spectral_frames.copy()
+    header['CTYPE3'] = ctype3
+    header['CRVAL3'] = 0.1
+    header['CDELT3'] = 0.001
+
+    if ctype3[0] == 'V':
+        header['CUNIT3'] = 'm s-1'
+    else:
+        header['CUNIT3'] = ''
+
+    header['RESTWAV'] = 1.420405752E+09
+    header['MJD-OBS'] = 55197
+
+    if observer:
+        header['OBSGEO-L'] = 144.2
+        header['OBSGEO-B'] = -20.2
+        header['OBSGEO-H'] = 0.
+        header['SPECSYS'] = 'BARYCENT'
+
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', FITSFixedWarning)
+        wcs = WCS(header)
+
+    skycoord, spectralcoord = wcs.pixel_to_world(0, 0, 31)
+
+    assert isinstance(spectralcoord, SpectralCoord)
+
+    if observer:
+        pix = wcs.world_to_pixel(skycoord, spectralcoord)
+    else:
+        with pytest.warns(AstropyUserWarning, match='No observer defined on WCS'):
+            pix = wcs.world_to_pixel(skycoord, spectralcoord)
+
+    assert_allclose(pix, [0, 0, 31], rtol=1e-6)
