@@ -8,11 +8,30 @@ import tempfile
 
 import hypothesis
 
+from astropy import __version__
+
 try:
-    from pytest_astropy_header.display import PYTEST_HEADER_MODULES
-    PYTEST_HEADER_MODULES['PyERFA'] = 'erfa'
+    from pytest_astropy_header.display import PYTEST_HEADER_MODULES, TESTED_VERSIONS
 except ImportError:
     PYTEST_HEADER_MODULES = {}
+    TESTED_VERSIONS = {}
+
+
+# This has to be in the root dir or it will not display in CI.
+def pytest_configure(config):
+    PYTEST_HEADER_MODULES['PyERFA'] = 'erfa'
+    PYTEST_HEADER_MODULES['Cython'] = 'cython'
+    PYTEST_HEADER_MODULES['Scikit-image'] = 'skimage'
+    PYTEST_HEADER_MODULES['asdf'] = 'asdf'
+    TESTED_VERSIONS['Astropy'] = __version__
+
+
+# This has to be in the root dir or it will not display in CI.
+def pytest_report_header(config):
+    # This gets added after the pytest-astropy-header output.
+    return (f'ARCH_ON_CI: {os.environ.get("ARCH_ON_CI", "undefined")}\n'
+            f'IS_CRON: {os.environ.get("IS_CRON", "undefined")}\n')
+
 
 # Tell Hypothesis that we might be running slow tests, to print the seed blob
 # so we can easily reproduce failures from CI, and derive a fuzzing profile
@@ -26,7 +45,7 @@ hypothesis.settings.register_profile(
 hypothesis.settings.register_profile(
     'fuzzing', deadline=None, print_blob=True, max_examples=1000
 )
-default = 'fuzzing' if (os.environ.get('TRAVIS_EVENT_TYPE') == 'cron' and os.environ.get('TRAVIS_CPU_ARCH') != 'arm64') else 'ci'  # noqa: E501
+default = 'fuzzing' if (os.environ.get('IS_CRON') == 'true' and os.environ.get('ARCH_ON_CI') not in ('aarch64', 'ppc64le')) else 'ci'  # noqa: E501
 hypothesis.settings.load_profile(os.environ.get('HYPOTHESIS_PROFILE', default))
 
 # Make sure we use temporary directories for the config and cache
